<?php

namespace Portfoliya_Pro\Controllers;

use SolidieLib\_Array;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\Reaction;
use Portfoliya\Models\Release;
use Portfoliya\Models\User;
use Portfoliya_Pro\Helpers\Utilities;
use Portfoliya_Pro\Models\Cart;
use Portfoliya_Pro\Models\ContentPack;
use Portfoliya_Pro\Models\Contents;
use Portfoliya_Pro\Models\Sale;
use WC_Payment_Gateways;

class ContentController {
	const PREREQUISITES = array(
		'addToCart' => array(
			'nopriv' => true
		),
		'getCart' => array(
			'nopriv' => true
		),
		'getPurchasedContents' => array(),
		'getSinglePurchaseInfo' => array(),
		'getOrderItemInfo' => array(
			'role' => 'administrator'
		),
		'updateItemInfo' => array(
			'role' => 'administrator'
		),
		'addStandalonePlan' => array(
			'role' => 'administrator'
		),
		'getWishlist' => array(

		),
		'getContentPlansData' => array(

		)
	);

	/**
	 * Get purchased content list for personal dashboard
	 *
	 * @return void
	 */
	public static function getPurchasedContents() {
		$content_list = Contents::getPurchasedContents( get_current_user_id() );
		wp_send_json_success( array( 'contents' => $content_list ) );
	}

	/**
	 * Add content to cart
	 *
	 * @param int $product_id The product ID to add to cart
	 * @param int $variation_id The product variation ID to add to cart
	 * 
	 * @return void
	 */
	public static function addToCart( int $product_id, int $variation_id ) {
		
		if ( ! function_exists( 'wc_get_product' ) || ! wc_get_product( $product_id ) ) {
			wp_send_json_error( array( 'message' => esc_html__( 'Product Not Fund!', 'portfoliya' ) ) );
			exit;
		}
	
		// Add the product to the cart
		$added = WC()->cart->add_to_cart( $product_id, 1, $variation_id );

		if ( ! $added ) {

			$error_message = wc_print_notices( true );
			
			wp_send_json_error(
				array(
					'message' => ! empty( $error_message ) ? trim( strip_tags( $error_message ) ) : 'An unknown error occurred.'
				)
			);
		}

		wp_send_json_success(
			array( 
				'cart'    => Cart::getCart(),
				'message' => esc_html__( 'Added to cart', 'portfoliya' ), 
			)
		);
		exit;
	}

	/**
	 * Get cart and purchased contents
	 *
	 * @param int $content_id The content ID to get cart 
	 * 
	 * @return void
	 */
	public static function getCart( int $content_id ) {

		$purchases = Contents::getPurchasedContents( get_current_user_id(), $content_id );

		// Loop through purchases and add releases for content packs
		foreach ( $purchases as $index => $purchase ) {
			
			// Non empty content ID means it is not content pack, and releases will there be already
			if ( ! empty( $purchase['content_id'] ) ) {
				continue;
			}

			$release = Release::getRelease( $content_id );
			if ( ! empty( $release ) ) {
				$release['download_url'] = add_query_arg( array( 'sale_id' => $purchase['sale_id'] ), $release['download_url'] );
			}

			$purchases[ $index ]['release'] = $release;
		}

		// Group purchases by variation ID to support multiple sale on the same
		$purchases = _Array::groupRows( $purchases, 'variation_id' );

		// Check if there's any payment gateway configured
		$gateways = WC_Payment_Gateways::instance()->get_available_payment_gateways();
		
		wp_send_json_success(
			array(
				'purchases'           => (object) $purchases,
				'cart'                => Cart::getCart(),
				'has_payment_gateway' => ! empty( $gateways )
			)
		);
	}

	/**
	 * Get purchased content pack data
	 *
	 * @param int $sale_id The sale ID to get purchased pack info
	 *
	 * @return void
	 */
	public static function getSinglePurchaseInfo( int $sale_id ) {
		
		$sale_id = absint( $sale_id );
		$data    = ! empty( $sale_id ) ? ContentPack::getSaleData( get_current_user_id(), $sale_id ) : array();

		if ( empty( $data ) || ! is_array( $data ) ) {
			wp_send_json_error( array( 'message' => esc_html__( 'Sale not found', 'portfoliya' ) ) );
		}
		
		wp_send_json_success(
			array(
				'sale'      => ( object ) $data['sale'],
				'downloads' => $data['downloads'],
				'renewals'  => Sale::getRenewalOrders( $data['sale']['sale_id'] )
			)
		);
	}

	/**
	 * Get sale info by order ID and the purchased variation ID
	 *
	 * @param integer $order_id
	 * @param integer $variation_id
	 * @return void
	 */
	public static function getOrderItemInfo( int $order_id, int $variation_id ) {

		// Get parent order data for renewals
		$parent_sale  = Sale::getParentSale( $order_id, $variation_id );
		$parent_order = null;
		
		if ( $parent_sale ) {

			// Override the sub order ID with the main to get content access stats
			$order_id = $parent_sale['order_id'];
			
			// Send parent order info to show link
			$parent_order = array(
				'url' => get_edit_post_link( $order_id, null ),
				'id'  => $order_id
			);
		}

		$sale = Sale::getSale( compact( 'order_id', 'variation_id' ) );
		
		if ( ! empty( $sale ) ) {

			$sale_plan = Sale::getSalePlan( $sale['sale_id'] );
			if ( is_array( $sale_plan ) ) {
				$sale_plan['parent_order'] = $parent_order;

				if ( ! $parent_sale ) {
					$sale_plan['renewals'] = Sale::getRenewalOrders( $sale_plan['sale_id'] );
				}
			}
			
			wp_send_json_success( array( 'sale' => $sale_plan ) );
		}

		wp_send_json_error( array( 'message' => __( 'Sale not found', 'portfoliya' ) ) );
	}

	/**
	 * Updated portfoliya order item info
	 *
	 * @param array $item_info
	 * @return void
	 */
	public static function updateItemInfo( int $sale_id, array $item_info ) {
		
		$args = array();

		if ( isset( $item_info['enabled'] ) ) {
			$args['enabled'] = ( bool ) $item_info['enabled'];
		}

		if ( isset( $item_info['expires_on'] ) ) {
			$args['expires_on'] = empty( $item_info['expires_on'] ) ? null : $item_info['expires_on'];
		}
		
		if ( isset( $item_info['license_or_content_limit'] ) ) {
			$args['license_or_content_limit'] = empty( $item_info['license_or_content_limit'] ) ? null : ( int ) $item_info['license_or_content_limit'];
		}

		Sale::updateSale( $sale_id, $item_info );

		wp_send_json_success( array( 'sale' => Sale::getSalePlan( $sale_id ) ) );
	}

	/**
	 * Get wishlist for user dashboard
	 *
	 * @return void
	 */
	public static function getWishlist() {
		$wishlist = Reaction::wishlist( 0 )->getReactions( get_current_user_id() );
		wp_send_json_success( array( 'wishlist' => $wishlist ) );
	}

	/**
	 * Get content plans and plan control value in content editor ideally
	 *
	 * @param string $content_type
	 * @return void
	 */
	public static function getContentPlansData( string $content_type ) {

		// Send plans setting for new creation
		$plan_settings            = Utilities::getContentPlanSettings( $content_type );
		$contributor_plan_control = ( bool ) AdminSetting::get( "general.contributor_content_plan_control" );

		wp_send_json_success( 
			array(
				'content_type_plans'       => $plan_settings,
				'contributor_plan_control' => User::hasAdministrativeRole( get_current_user_id() ) || $contributor_plan_control,
			)
		);
	}

	/**
	 * Add single standalone plan to settings
	 *
	 * @param array $plan
	 * @return void
	 */
	public static function addStandalonePlan( string $content_type, array $plan ) {

		$settings = AdminSetting::get();
		if ( ! is_array( $settings['contents'][ $content_type ] ?? null ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid content type', 'portfoliya' ) ) );
		}

		if ( ! is_array( $settings['contents'][ $content_type ]['plans'] ?? '' ) ) {
			$settings['contents'][ $content_type ]['plans'] = array();
		}

		$settings['contents'][ $content_type ]['plans'][] = $plan;

		AdminSetting::save( $settings, true );

		wp_send_json_success();
	}
}
