<?php

namespace Portfoliya_Pro\Controllers;

use Portfoliya\Models\Contents;
use Portfoliya\Models\Popularity;
use Portfoliya\Models\User;
use Portfoliya\Setup\AdminPage;
use Portfoliya_Pro\Models\Contributor;
use Portfoliya_Pro\Models\Earning;
use Portfoliya_Pro\Models\Sale;
use Portfoliya_Pro\Models\Withdrawal;
use Portfoliya_Pro\Setup\Dashboard;

class EarningController {
	
	const PREREQUISITES = array(
		'getMyEarningStats'      => array(),
		'getSalesList'           => array(),
		'saveWithdrawalMethod'   => array(),
		'getWithdrawalMethods'   => array(),
		'requestWithdrawal'      => array(),
		'getWithdrawalRequests'  => array(),
		'changeWithdrawalStatus' => array(),
		'getPortfoliyaReports'      => array(
			'role' => 'administrator'
		),
	);

	/**
	 * Get users earning stat
	 *
	 * @return void
	 */
	public static function getMyEarningStats() {
		wp_send_json_success(
			array( 
				'stats'              => Earning::getEarningStats( get_current_user_id() ),
				'withdrawal_methods' => ( object ) Withdrawal::getMethods( get_current_user_id() ),
				'withdrawal_minimum' => Withdrawal::getMinimumWithdrawal(),
			) 
		);
	}

	/**
	 * Get sales data for 
	 *
	 * @param integer $page The page offset to get sales for
	 * @param bool    $is_admin Whether it is called from admin scope or not.
	 * 
	 * @return void
	 */
	public static function getSalesList( int $page = 1, bool $is_admin = false ) {

		// If it is admin scope, check if current user has administrative role
		if ( $is_admin && ! User::hasAdministrativeRole( get_current_user_id() ) ) {
			wp_send_json_error( array( 'message' => __( 'Access denied!', 'portfoliya' ) ) );
		}

		$args = array(
			'contributor_id' => $is_admin ? null : get_current_user_id(),
			'parent_id'      => false,
			'page'           => $page
		);

		$sales        = Sale::getSales( $args );
		$sales        = $is_admin ? $sales : Sale::prepareSalesForContributor( $sales );
		$segmentation = Sale::getSales( $args, true );

		wp_send_json_success(
			array(
				'sales' => $sales, 
				'segmentation' => $segmentation 
			)
		);
	}

	/**
	 * Save contributors' withdrawal method
	 *
	 * @param string $method
	 * @param array $fields
	 * @return void
	 */
	public static function saveWithdrawalMethod( string $method, array $fields ) {
		Withdrawal::updateWithdrawalMethod( get_current_user_id(), $method, $fields );
		wp_send_json_success( array( 'message' => __( 'Withdrawal method saved', 'portfoliya' ) ) );
	}

	/**
	 * Get saved withdrawal method for user
	 *
	 * @return void
	 */
	public static function getWithdrawalMethods() {
		wp_send_json_success( 
			array( 
				'methods' => (object) Withdrawal::getMethods( get_current_user_id() ),
			)
		);
	}

	/**
	 * Request withdrawal
	 *
	 * @param integer $amount
	 * @param string $method
	 * @return void
	 */
	public static function requestWithdrawal( float $amount, string $method ) {
		
		$user_id = get_current_user_id();
		$stats   = Earning::getEarningStats( $user_id );
		$min     = Withdrawal::getMinimumWithdrawal();

		// The amount should not be less than minimum withdrawal amount
		if ( $amount < $min ) {
			wp_send_json_error( array( 'message' => sprintf( __( 'Minimum withdrawal amount %d', 'portfoliya' ), $min ) ) );
		}

		// The amount should not be greater than available balance
		if ( $amount > $stats['available'] ) {
			wp_send_json_error( array( 'message' => __( 'Insufficient balance', 'portfoliya' ) ) );
		}

		// The method info should not be empty
		if ( empty( Withdrawal::getMethods( $user_id, $method ) ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid withdrawal method or yet to complete', 'portfoliya' ) ) );
		}

		// Finally create withdrawal entry
		Withdrawal::requestWithdrawal( $user_id, $method, $amount );

		wp_send_json_success( array( 'message' => __( 'Withdrawal request submitted', 'portfoliya' ) ) );
	}

	/**
	 * Get withdrawal list. Used both in user dashboard and requeste reviewer in admin dashboard.
	 * However in contributor view, they can see only own requests.
	 *
	 * @param integer $page For pagination purpose
	 * @param boolean $is_admin
	 * @return void
	 */
	public static function getWithdrawalRequests( int $page, bool $is_admin = false, string $status = '', string $contributor_email = '', string $order = 'DESC' ) {

		// To Do: Add admin role check and in fact revamp the administrator role overall

		$args = array(
			'contributor_id'    => $is_admin ? null : get_current_user_id(),
			'contributor_email' => $contributor_email,
			'status'            => $status,
			'order'             => $order === 'ASC' ? 'ASC' : 'DESC',
			'page'              => $page
		);

		$requests     = Withdrawal::getRequests( $args );
		$segmentation = Withdrawal::getRequests( $args, true );

		wp_send_json_success(
			array( 
				'withdrawals'  => $requests,
				'segmentation' => $segmentation
			) 
		);
	}

	/**
	 * Change withdrawal status by reviewer or cancel request by the contributor
	 *
	 * @param integer $withdrawal_id
	 * @param string $status
	 * @return void
	 */
	public static function changeWithdrawalStatus( int $withdrawal_id, string $status ) {

		$user_id    = get_current_user_id();
		$withdrawal = Withdrawal::getRequestById( $withdrawal_id );

		// Withdrawl can not be inexistent
		if ( empty( $withdrawal ) ) {
			wp_send_json_error( array( 'message' => __( 'The request was not found', 'portfoliya' ) ) );
		}

		// Only the requestor contributor can cancel
		if ( $status === 'cancelled' && $withdrawal['contributor_id'] !== $user_id ) {
			wp_send_json_error( array( 'message' => __( 'You are not authorized to cancel!', 'portfoliya' ) ) );
		}

		// The current user must have admin level access for non cancel actions like reject, sent etc.
		if ( $status !== 'cancelled' && ! User::hasAdministrativeRole( $user_id )) {
			wp_send_json_error( array( 'message' => __( 'You are not authorized fot the action!', 'portfoliya' ) ) );
		}

		// Request can't be cancelled once it is changed to something else from pending status
		if ( $status === 'cancelled' && $withdrawal['withdrawal_status'] !== 'pending' ) {
			wp_send_json_error( array( 'message' => __( 'The request can not be cancelled as processing started already', 'portfoliya' ) ) );
		}

		// If already cancelled, nothing else to do
		if ( $withdrawal['withdrawal_status'] === 'cancelled' ) {
			wp_send_json_error( array( 'message' => __( 'The withdrawal request has been cancelled meanwhile!', 'portfoliya' ) ) );
		}

		if ( ! in_array( $status, Withdrawal::STATUSES ) ) {
			wp_send_json( array( 'message' => __( 'Invalid withdrawal status', 'portfoliya' ) ) );
		}

		// Finally change the status to desired on
		Withdrawal::changeRequestStatus( $withdrawal_id, $status );

		wp_send_json_success( array( 'message' => __( 'Withdrawal request status has been updated', 'portfoliya' ) ) );
	}

	/**
	 * Get reports page data
	 *
	 * @return void
	 */
	public static function getPortfoliyaReports() {
		wp_send_json_success(
			array(
				'overview'            => Sale::getRevenueStats(),
				'sales_trend'         => Sale::getSalesTrend(),
				'download_trend'      => Popularity::getTrend(),
				'inventory_link'      => add_query_arg( array( 'page' => AdminPage::INVENTORY_SLUG ), admin_url( 'admin.php' ) ),
				'contributors_link'   => add_query_arg( array( 'page' => Dashboard::SLUG_CONTRIBUTORS ), admin_url( 'admin.php' ) ),
				'withdrawals_link'    => add_query_arg( array( 'page' => Dashboard::SLUG_WITHDRAWALS ), admin_url( 'admin.php' ) ),
				'recent_sales'        => Sale::getSales(
					array(
						'limit' => 5, 
					)
				),
				'recent_contributors' => Contributor::getContributors( 
					array( 
						'status' => 'approved', 
						'limit'  => 4,
					) 
				),
				'recent_withdrawals'  => Withdrawal::getRequests(
					array(
						'limit'  => 4,
						'status' => 'sent',
					) 
				),
				'recent_contents'     => Contents::getContents(
					array( 
						'limit'          => 5, 
						'content_status' => 'publish',
						'order_by'       => 'newest',
					) 
				),
			)
		);
	}
}
