<?php
namespace Portfoliya_Pro\Models;

use SolidieLib\_Array;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\Field;
use Portfoliya\Models\Manifest;

class ContentPack {

	/**
	 * The meta key to define content type for the content pack product
	 *
	 * @var string
	 */
	public static $meta_key = 'portfoliya-content-pack';

	/**
	 * User meta key to store content pack download stats
	 *
	 * @var string
	 */
	CONST DOWNLOADS_META_KEY = 'portfoliya-content-pack-downloads';

	/**
	 * Store temporary sale ID to generate 
	 *
	 * @var string
	 */
	private static $sale_id_volatile;

	/**
	 * Create or update content pack product
	 * Note: Only one product per each content type can be created
	 *
	 * @param string $content_type
	 * @param array $plans
	 * @return void
	 */
	public static function createContentPack( $content_type, $plans ) {

		// Get existing product for the content type
		$existing_pack = self::getContentPackByType( $content_type );
		
		// Prepare the product details for the content pack
		$product = array(
			'product_id'    => ! empty( $existing_pack ) ? $existing_pack['product_id'] : 0,
			'product_title' => sprintf( esc_html__( '%s Content Pack', 'portfoliya' ), Manifest::getContentTypeLabel( $content_type ) ),
			'plans'         => $plans
		);

		$product_id = Contents::addProduct( $product, 0 );

		// Assign the pack identifer meta now
		update_post_meta( $product_id, self::$meta_key, $content_type );
	}

	/**
	 * Get content packs by content type
	 *
	 * @param string $content_type The content type to get content pack for
	 * 
	 * @return array
	 */
	public static function getContentPackPlans( string $content_type ) {
		
		// Firstly get content pack product IDs for the type
		$product = self::getContentPackByType( $content_type );

		if ( empty( $product ) ) {
			return array();
		}

		$plans =  Contents::getContentVariations( $product['product_id'], $content_type );

		return array_values( 
			array_filter(
				$plans,
				function ( $plan ) {
					return ( $plan['plan']['sales_model'] ?? '' ) === 'pack';
				}
			)
		);
	}

	/**
	 * Get content packs for all the content types
	 *
	 * @param string|array|null $type Specific type to get plans of. All will be provided if not provided.
	 * 
	 * @return array
	 */
	public static function getBulkContentPackPlans( $type = null ) {
		
		$content_types = ! empty( $type ) ? _Array::getArray( $type, true ) : array_keys( AdminSetting::get( 'contents' ) );
		$plans         = array();

		foreach ( $content_types as $content_type ) {
			$plans[ $content_type ] = self::getContentPackPlans( $content_type );
		}

		return is_string( $type ) ? ( $plans[ $type ] ?? array() ) : $plans;
	}

	/**
	 * Adjust bundle sale commission for a specific contributor when their content gets downloaded/accessed through bundle.
	 *
	 * @param array $content
	 * @param int $sale_id
	 * @return void
	 */
	public static function adjustBundleSaleCommission( $content, $sale_id ) {
		
		global $wpdb;

		$sale = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT * FROM {$wpdb->portfoliya_sales} WHERE sale_id=%d",
				$sale_id
			),
			ARRAY_A
		);

		if ( empty( $sale ) ) {
			return;
		}

		// The total commission paid for the bundle sale
		$paid_commission = (float) $wpdb->get_var(
			$wpdb->prepare(
				"SELECT 
					SUM(commission) 
				FROM 
					{$wpdb->portfoliya_bundle_commissions}
				WHERE
					sale_id=%d",
				$sale['sale_id']
			)
		);

		$remaining_commissions  = $sale['commission'] - $paid_commission;
		$remaining_downloads   = $sale['license_or_content_limit'] - $sale['downloads_in_season'];
		$payable_to_individual = ( $remaining_commissions > 0 && $remaining_downloads > 0 ) ? $remaining_commissions / $remaining_downloads : 0;

		// Adjust the contributors commission if payable amount is greater than zero
		if ( ! ( $payable_to_individual > 0 )  ) {
			return;
		}
			
		// Get the current commission earning of the contributor
		$commission = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT 
					* 
				FROM 
					{$wpdb->portfoliya_bundle_commissions} 
				WHERE 
					sale_id=%d 
					AND contributor_id=%d",
				$sale['sale_id'],
				$content['contributor_id']
			),
			ARRAY_A
		);

		if ( empty( $commission ) ) {
			$wpdb->insert(
				$wpdb->portfoliya_bundle_commissions,
				array( 
					'sale_id'        => $sale['sale_id'],
					'contributor_id' => $content['contributor_id'],
					'commission'      => $payable_to_individual
				)
			);
		} else {
			$wpdb->update(
				$wpdb->portfoliya_bundle_commissions,
				array( 'commission' => (float) $commission['commission'] + $paid_commission ),
				array( 'commission_id' => $commission['commission_id'] )
			);
		}
	}

	/**
	 * Validate content pack sale by user id and sale id, and the sale must be not expired, limit reached etc.
	 *
	 * @param string $content_type
	 * @param int $customer_id
	 * @param int $sale_id
	 * @return bool
	 */
	public static function validatePackSale( $content_type, $customer_id, $sale_id ) {

		// Get the content product by the type
		// Note: There will always be a single product per content type to represent content pack
		$product = ContentPack::getContentPackByType( $content_type );
		if ( empty( $product ) ) {
			return false;
		}

		global $wpdb;

		$sale_id =  $wpdb->get_var(
			$wpdb->prepare(
				"SELECT 
					sale_id 
				FROM 
					{$wpdb->portfoliya_sales} 
				WHERE 
					product_id=%d 
					AND customer_id=%d 
					AND sale_id=%d 
					AND enabled=1
					AND order_status='completed'
					AND (expires_on IS NULL OR expires_on>NOW()) 
					AND (license_or_content_limit IS NULL OR license_or_content_limit>downloads_in_season) 
				LIMIT 1",
				$product['product_id'],
				$customer_id,
				$sale_id
			)
		);

		return $sale_id ? true : false;
	}

	/**
	 * Get content pack by product ID
	 *
	 * @param int $product_id
	 * @return array
	 */
	public static function getContentPackByProduct( $product_id ) {

		$content_type = get_post_meta( $product_id, self::$meta_key, true );

		// Empty meta means it is not a content pack
		if ( empty( $content_type ) ) {
			return null;
		}
		
		$resp = array(
			'content_type' => $content_type,
			'content_id'   => 0,
			'product_id'   => $product_id,
		);

		return $resp;
	}

	/**
	 * Get content pack by content type when the product ID is unknown
	 *
	 * @param string $content_type
	 * @return array|null
	 */
	public static function getContentPackByType( $content_type ) {
		$product_id = Contents::getPostIDByMeta( self::$meta_key, $content_type, 'product' );
		if ( empty( $product_id ) ) {
			return null;
		}

		return array(
			'content_type' => $content_type,
			'content_id'   => 0,
			'product_id'   => $product_id
		);
	}

	/**
	 * Assign product meta data to help UI to render content packs as similar to contents.
	 * Especially in purchased contents list.
	 *
	 * @param array $contents
	 * @return array
	 */
	public static function assignPackMeta( $contents ) {
		foreach ( $contents as $index => $content ) {
			// Non empty means non content pack
			if ( ! empty( $content['content_id'] ) ) {
				continue;
			}

			// As it is not linked to content, add meta data manually
			$contents[ $index ]['content_title']    = get_the_title( $content['product_id'] );
			$contents[ $index ]['content_type']     = get_post_meta( $content['product_id'], self::$meta_key, true );
			// $contents[ $index ]['product']['plans'] = get_post_meta( $content['product_id'], self::$meta_key, true );
		}

		return $contents;
	}
	
	/**
	 * Get download stats for content packs
	 *
	 * @param int $user_id
	 * @param int|null $pack_sale_id
	 * @return array
	 */
	public static function getDonwloadStats( $user_id, $pack_sale_id = null ) {
		$stats = get_user_meta( $user_id, self::DOWNLOADS_META_KEY, true );
		$stats = is_array( $stats ) ? $stats : array();

		// Return the download stats for the pack sale id, or the entire one
		return null !== $pack_sale_id ? ( $stats[ $pack_sale_id ] ?? array() ) : $stats;
	}

	/**
	 * Increase content pack donwload stats
	 *
	 * @param int  $user_id
	 * @param int  $sale_id
	 * @param int  $content_id
	 * @param bool $singular_entry
	 * 
	 * @return void
	 */
	public static function increaseStats( $user_id, $pack_sale_id, $content_id, $singular_entry = false ) {
		// Add download history to the downloader user meta
		$stats = self::getDonwloadStats( $user_id );
		if ( ! isset( $stats[ $pack_sale_id ] ) ) {
			$stats[ $pack_sale_id ] = array();
		}

		// Singular entry is used for tutorial only as download count doesn't makes sense for it.
		if ( $singular_entry && in_array( $content_id, array_column( $stats[ $pack_sale_id ], 'id' ) ) ) {
			return;
		}

		$stats[ $pack_sale_id ][] = array(
			'time' => strtotime( gmdate('Y-m-d H:i:s') ),
			'id'   => $content_id
		);

		update_user_meta( $user_id, self::DOWNLOADS_META_KEY, $stats );

		// Add download counter in sales table where content access limit is defined/not unlimited.
		global $wpdb;
		$wpdb->query(
			$wpdb->prepare(
				"UPDATE 
					{$wpdb->portfoliya_sales} 
				SET 
					downloads_in_season=downloads_in_season+1 
				WHERE 
					sale_id=%d
					AND license_or_content_limit IS NOT NULL",
				$pack_sale_id
			)
		);
	}

	/**
	 * Get content pack sale data by sale ID
	 *
	 * @param int $sale_id
	 * @return array
	 */
	public static function getSaleData( $customer_id, $sale_id ) {
		
		global $wpdb;

		$sale = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT * FROM {$wpdb->portfoliya_sales} WHERE sale_id=%d AND customer_id=%d",
				$sale_id,
				$customer_id
			),
			ARRAY_A
		);

		if ( empty( $sale ) ) {
			return null;
		}

		$sale = _Array::castRecursive( $sale );

		// Get downloads stat
		self::$sale_id_volatile = $sale_id;
		$downloads              = self::getDonwloadStats( $customer_id, $sale_id );
		$history                = array();

		// Add release download URL filter to append sale ID
		add_filter( 'portfoliya_release_download_link', array( __CLASS__, 'addSaleIdToDownloadUrl' ), 11 );

		// Loop through content download history for the pack and prepare data
		foreach ( $downloads as $index => $download ) {
			$history[] = array(
				'download_time' => $download['time'],
				'content'       => Contents::getContentByContentID( $download['id'] ),
			);
		}
		
		// Remove the filter as it is not necessary anymore
		remove_filter( 'portfoliya_release_download_link', array( __CLASS__, 'addSaleIdToDownloadUrl' ) );

		return array(
			'sale'      => $sale,
			'downloads' => $history
		);
	}

	/**
	 * Add sale ID to download URL
	 *
	 * @param string $url
	 * @return string
	 */
	public static function addSaleIdToDownloadUrl( $url ) {
		return add_query_arg( array( 'sale_id' => self::$sale_id_volatile ), $url );
	}

	/**
	 * Check if a product is portfoliya content pack
	 *
	 * @param int $product_id
	 * @return boolean
	 */
	public static function isProductContentPack( $product_id ) {
		return ! empty( self::getContentPackByProduct( $product_id ) );
	}

	/**
	 * Determine whether a single content pack variation auto enabled
	 *
	 * @param string $pack_id
	 * @return boolean
	 */
	public static function isPackAutoEnabled( $pack_id ) {

		$auto  = true;
		$packs = AdminSetting::get( 'contents' );
		
		foreach ( $packs as $type => $pack ) {
			
			$plans = _Array::getArray( $pack['plans'] ?? null );

			foreach ( $plans as $plan ) {
				if ( ( $plan['id'] ?? null ) === $pack_id ) {
					$auto = ( bool ) ( $plan['auto_enable'] ?? true );
					break 2;
				}
			}
		}

		return $auto;
	}
}
