<?php

namespace Portfoliya_Pro\Models;

use SolidieLib\_Array;
use SolidieLib\_Number;
use SolidieLib\_String;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\Contents as ContentsFree;
use Portfoliya\Models\Field;
use Portfoliya_Pro\Helpers\Utilities;

class Contents extends ContentsFree {
	/**
	 * Licensing variation key for contents
	 */
	const LICENSING_VARIATION = 'plan';

	/**
	 * Variation prefix to set with product IDs
	 */
	const LICENSING_VARIATION_PREFIX = 'portfoliya-product-variation-';

	/**
	 * Meta key for optional purchase flag
	 */
	const PURCHASE_OPTIONAL_KEY = 'is_purchase_optional';

	/**
	 * Check if a content is free or not
	 *
	 * @param int|string $content_id Content ID if numeric, otherwise content slug.
	 * @param bool       $optional_as_free Whether to treat optional purchasable contents as free.
	 * 
	 * @return boolean
	 */
	public static function isContentFree( $content_id, $optional_as_free ) {
		
		$where_col  = is_numeric( $content_id ) ? 'content_id' : 'content_slug';
		$product_id = Field::contents()->getField( array( $where_col => $content_id ), 'product_id' );
		$is_free    = empty( $product_id );

		// Treat optional purchase contents as empty
		if ( ! $is_free && $optional_as_free ) {
			$is_free = self::isProductPurchaseOptional( $product_id );
		}

		return $is_free;
	}

	/**
	 * Check if a product purchase is optional to download the linked content
	 *
	 * @param int $product_id
	 * @return boolean
	 */
	public static function isProductPurchaseOptional( $product_id ) {
		return get_post_meta( $product_id, self::PURCHASE_OPTIONAL_KEY, true ) ? true : false;
	}

	/**
	 * Get Content by product id and post name.
	 *
	 * @param string|int  $product_id_or_name To ge content by product ID or product post name
	 * @param string|null $field To get specific field from the content
	 * @param bool        $public_only Whether to get only published product
	 * 
	 * @return object|int|string|null
	 */
	public static function getContentByProduct( $product_id, $field = null, $public_only = true ) {
		if ( ! is_numeric( $product_id ) ) {
			$product = get_page_by_path( $product_id, OBJECT, 'product' );

			if ( empty( $product ) ) {
				return null;
			}

			$product_id = $product->ID;
		}

		return self::getContentByField( 'product_id', $product_id, $field, $public_only );
	}

	/**
	 * Check if a product is portfoliya content
	 * Note: This will return same for content pack too as the meta key is same.
	 *
	 * @param int $product_id
	 * @param bool $check_pack
	 * 
	 * @return boolean
	 */
	public static function isProductContent( $product_id, $check_pack = true ) {
		$content = ! empty( get_post_meta( $product_id, self::PRODUCT_META_KEY_FOR_CONTENT, true ) );

		if ( empty( $content ) && $check_pack ) {
			$content = ContentPack::isProductContentPack( $product_id );
		}

		return $content;
	}
	
	/**
	 * Filter contents from order items
	 *
	 * @param array $contents All the order items from order regardless of portfoliya or others
	 * @return array
	 */
	private static function filterContentsFromOrderItems( array $items ) {
		$contents  = array();
		
		foreach ( $items as $item ) {
			// Here contents is actually woocommerce order item. 
			$product_type = $item->get_product()->get_type();
			$product_id   = $item->get_product_id();
			$variation_id = $item->get_variation_id(); // Purchased product variation ID. 
			$variation    = new \WC_Product_Variation( $variation_id );
			$content      = self::getContentByProduct( $product_id );
			$var_info     = $content ? self::getVariationInfo( $variation, $content['content_type'] ) : null;

			// Skip non-content products or unsupported variation.
			// We support only variable product
			if ( ! $content || ! $var_info || 'variation' !== $product_type ) {
				continue;
			}

			$content['variation'] = $var_info;
			$contents[] = $content;
		}

		return $contents;
	}

	/**
	 * Filter contents packs from order items
	 *
	 * @param array $contents All the order items from order regardless of portfoliya or others
	 * @return array
	 */
	private static function filterContentPacksFromOrderItems( array $items ) {
		$contents  = array();
		
		foreach ( $items as $item ) {
			// Here contents is actually woocommerce order item. 
			$product_type = $item->get_product()->get_type();
			$product_id   = $item->get_product_id();
			$variation_id = $item->get_variation_id(); // Purchased product variation ID. 
			$variation    = new \WC_Product_Variation( $variation_id );
			$content      = ContentPack::getContentPackByProduct( $product_id );
			$var_info     = $content ? self::getVariationInfo( $variation, $content['content_type'] ) : null;

			// Skip non-content products or unsupported variation.
			// We only variable product
			if ( ! $content || ! $var_info || 'variation' !== $product_type ) {
				continue;
			}

			$content['variation'] = $var_info;
			$contents[] = $content;
		}

		return $contents;
	}

	public static function getProductVariationsByContentSlug( string $content_slug, bool $enabled_only = true ) {

		$product_id = self::getContentByField( 'content_slug', $content_slug, 'product_id' );
		if ( empty( $product_id ) ) {
			return array();
		}

		$content_type = self::getContentTypeByProduct( $product_id );
		$variations = self::getContentVariations( $product_id, $content_type );

		// Indexify by plan title for easy render
		$indexed = array();
		foreach ( $variations as $variation ) {
			if ( !$enabled_only || $variation['enable'] ) {
				$indexed[ $variation['plan']['plan_name'] ] = $variation;
			}
		}

		return $indexed;
	}

	/**
	 * Return variation label
	 *
	 * @param \WC_Product_Variation|int $variation
	 * @return array|null An associative array
	 *   - 'variation_id' (int): Variation ID, variation post type ID technically
	 *   - 'plan_name' (string): Plan name
	 *   - 'plan_description' (string): Plan description
	 *   - 'plan_key' (string): Plan ID came from content types settings. Variation attribute/name in product scope.
	 *   - 'price' (double): Variation price
	 *   - 'regular_price' (double): Variation regular price
	 *   - 'sale_price' (double): Variation sale price
	 *   - 'enable' (bool): Whether the variation is enabled or not
	 */
	public static function getVariationInfo( $variation, $content_type ) {
		$data       = null;
		$variation  = is_numeric( $variation ) ? new \WC_Product_Variation( $variation ) : $variation;
		if ( empty( $variation ) ) {
			return $data;
		}

		$attributes   = $variation->get_attributes();
		$variation_id = $variation->get_id();

		if ( isset( $attributes[ self::LICENSING_VARIATION ] ) ) {

			$plan_key = $attributes[ self::LICENSING_VARIATION ];
			$plan     = self::getContentTypePlan( $content_type, $plan_key );

			// Determine sale price
			$sale_price = $variation->get_sale_price();
			$_price     = $variation->get_price();
			if ( empty( $sale_price ) && ! empty( $_price ) ) {
				$sale_price = $_price;
			}

			$data = array(
				'plan'          => $plan,
				'regular_price' => (float) $variation->get_regular_price(),
				'sale_price'    => (float) $sale_price,
				'variation_id'  => $variation_id,
				'product_id'    => $variation->get_parent_id(),
				'enable'        => 'publish' === $variation->get_status(),
			);
		}

		return $data;
	}
	
	/**
	 * Get available variations for the content product
	 *
	 * @param int    $product_id The product ID to get variations of
	 * @param string $content_type The content type for the linked content
	 * 
	 * @return array
	 */
	public static function getContentVariations( $product_id, $content_type ) {
		$product     = ( ! empty( $product_id ) && function_exists( 'wc_get_product' ) ) ? wc_get_product( $product_id ) : null;
		$_variations = array();

		if ( empty( $product ) ) {
			return $_variations;
		}
		
		if ( $product->is_type( 'variable' ) ) {
			
			$variations = get_posts(
				array(
					'post_parent' => $product_id,
					'post_status' => array( 'publish', 'private', 'draft' ),
					'post_type'   => 'product_variation',
					'numberposts' => -1
				)
			);

			// Get the content types pack variations too
			$pack_product = ContentPack::getContentPackByType( $content_type );
			if ( ! empty( $pack_product ) && $product_id != $pack_product['product_id'] ) {
				$pack_variations = get_posts(
					array(
						'post_parent' => $pack_product['product_id'],
						'post_status' => array( 'publish', 'private', 'draft' ),
						'post_type'   => 'product_variation',
						'numberposts' => -1
					)
				);
				$variations = array_merge( $variations, $pack_variations );
			}

			$mapping = array();

			// Get variation informations
			foreach ( $variations as $index => $variation ) {

				// Get the incroporated variation info
				$info = self::getVariationInfo( $variation->ID, $content_type );

				if ( ! empty( $info['plan'] ) ) {
					$mapping[ $info['plan']['id'] ] = $info;
				}
			}

			// Get plan keys from the content type settings
			$content_plans = _Array::indexify( Utilities::getContentPlanSettings( $content_type ), 'id' );
			
			// Assign default plan if variation not created for it
			foreach ( $content_plans as $key => $plan ) {
				$_variations[] = $mapping[ $key ] ?? array( 'plan' => $plan, 'enable' => false );
			}
		} 
		
		return $_variations;
	}

	/**
	 * Assign contents to customer after linked product order complete.
	 * Portfoliya just simulates the subscription model in minimal way using simple variable product.
	 * The first ordered variation by a customer gets stored in the sales table.
	 * Subsequent order ID gets stored in main order meta, and the sale row gets updated with latest renew, expiry date and reset download counter for content pack.
	 *
	 * @param object $order
	 * @return void
	 */
	public static function processPurchase( $order ) {

		global $wpdb;
		$order_id = $order->get_id();

		if ( empty( $order_id ) || ! is_numeric( $order_id ) ) {
			return;
		}

		// Skip if sale entry created already
		$exists = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT sale_id FROM {$wpdb->portfoliya_sales} WHERE order_id=%d", 
				$order_id 
			) 
		);
		if ( ! empty( $exists ) ) {
			return;
		}

		$customer_id    = $order->get_customer_id();
		$order_items    = $order->get_items();
		$contents       = self::filterContentsFromOrderItems( $order_items );
		$order_date_gmt = gmdate( 'Y-m-d H:i:s', $order->get_date_created()->getOffsetTimestamp() );

		// Get the percentage contributor will get
		$commission_rate = _Number::getInt( AdminSetting::get( 'contributor_revenue_percentage' ), 0, 100 ) ;

		// Add content packs also to the array for easy processing
		$contents = array_merge( $contents, self::filterContentPacksFromOrderItems( $order_items ) );

		// Loop through portfoliya contents
		foreach ( $contents as $content ) {

			// Skip if no Portfoliya plan found
			if ( empty( $content['variation']['plan'] ?? null ) ) {
				continue;
			}

			// Calculate commission
			$sale_price = (int)$content['variation']['sale_price'];
			$commission = ( $commission_rate / 100 ) * $sale_price;
			
			// Prepare order info to create sales data
			$content_limit = $content['variation']['plan']['license_or_content_limit'];
			$variation_id  = $content['variation']['variation_id'];
			$order_status  = str_replace( 'wc-', '', $order->get_status() );

			// Get the first order info made by the customer for the variation
			// So, multiple sale for same variation will not be stored in sales table
			// Rather subsequent orders IDs will be stored as main order meta to use the data later
			$main_order = Sale::getSale(
				array(
					'customer_id'  => $customer_id,
					'variation_id' => $variation_id,
				)
			);

			// Create new entry in the sales table
			$valid_days = Utilities::periodToDays( $content['variation']['plan']['subscription_model'] );
			$expires_on = $valid_days ? ( new \DateTime( $order_date_gmt ) )->modify( "+{$valid_days} days")->format('Y-m-d') : null;

			$wpdb->insert(
				$wpdb->portfoliya_sales,
				array(
					'content_id'          => $content['content_id'],
					'customer_id'		  => $customer_id,
					'order_id'            => $order_id,
					'product_id'          => $content['product_id'],
					'variation_id'        => $variation_id,
					'sale_price'          => $content['variation']['sale_price'],
					'commission'          => $commission,
					'commission_rate'     => $commission_rate,
					'purchase_date'       => $order_date_gmt,
					'parent_id'	          => ( $main_order && $main_order['expires_on'] ) ? ( (int) $main_order['sale_id'] ) : 0, // If the main order is lifetime, don't make it child. Child is for renewal only.
					'order_status'        => $order_status,
					'order_complete_date' => $order_status === 'completed' ? gmdate( 'Y-m-d H:i:s' ) : null,
					'expires_on'          => $expires_on,
					'enabled'             => $main_order ? null : 1,

					// Access limit should be used from main order only. 
					// Renewal order is for tracking re-purchase and extending validity only.
					'license_or_content_limit' => $main_order ? null : $content_limit,
				)
			);

			// Generate license keys if it is app and new order. 
			if ( empty( $main_order ) && 'app' === $content['content_type'] ) {
				$sale = array(
					'product_id'   => $content['product_id'],
					'order_id'     => $order_id,
					'variation_id' => $variation_id,
					'sale_id'      => $wpdb->insert_id
				);
				Licensing::generateLicenseKeys( $sale, $content_limit );
			}
		}
	}

	/**
	 * Sync sale status with order status change
	 *
	 * @param int $order_id
	 * @param string $old_status
	 * @param string $new_status
	 *
	 * @return void
	 */
	public static function syncSaleStatus( $order_id, $old_status, $new_status ) {

		// Unify order status
		$new_status = str_replace( 'wc-', '', $new_status );
		$old_status = str_replace( 'wc-', '', $old_status );

		// Check status history
		$is_complete  = $new_status === 'completed';
		$was_complete = $old_status === 'completed';

		global $wpdb;

		// Get the purchased variations in that order
		$sales = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT 
					variation_id,
					parent_id,
					DATEDIFF(expires_on, purchase_date) AS validity_period,
					license_or_content_limit
				FROM 
					{$wpdb->portfoliya_sales} 
				WHERE 
					order_id=%d",
				$order_id
			),
			ARRAY_A
		);

		$sales = _Array::castRecursive( $sales );

		// Loop through variations and adjust expiry to main order, and update the variation complete_date
		foreach ( $sales as $sale ) {

			// Update the status of itself
			Field::sales()->updateField(
				array( 
					'order_status'        => $new_status,
					'order_complete_date' => $is_complete ? gmdate( 'Y-m-d H:i:s' ) : null,
				),
				array(
					'order_id'     => $order_id,
					'variation_id' => $sale['variation_id']
				)
			);

			// Add or subtract main order expiry on renewal order status change
			if ( ( $is_complete || $was_complete ) && ! empty( $sale['parent_id'] ) && $sale['validity_period'] > 0 ) {
				
				$method = $is_complete ? 'DATE_ADD' : 'DATE_SUB';
				$download_count = '';

				if ( null !== $sale['license_or_content_limit'] ) {
					$operator       = $is_complete ? '-' : '+';
					$download_count =  ", downloads_in_season=downloads_in_season{$operator}license_or_content_limit";
				}
				
				$wpdb->query(
					$wpdb->prepare(
						"UPDATE 
							{$wpdb->portfoliya_sales} 
						SET 
							expires_on={$method}(expires_on, INTERVAL %d DAY)
							{$download_count}
						WHERE
							sale_id=%d
							AND expires_on IS NOT NULL",
						$sale['validity_period'],
						$sale['parent_id']
					)
				);
			}
		}
	}

	/**
	 * Create/update product and connect to content.
	 * If content ID is 0, it means the product is content pack, and not linked to any single content. 
	 *
	 * @param array  $product The product data array to create or update existing one
	 * @param int    $content_id The content ID to connect product to. Pass 0 for content pack.
	 *
	 * @return int
	 */
	public static function addProduct( array $product, $content_id ) {

		$product_id = $product['product_id'] ?? null;

		// Make sure the product still exist
		if ( empty( $product_id ) || ! is_int( $product_id ) || empty( get_post( $product_id ) ) ) {
			$product_id = null;
		}

		// If product id exist, update the title and change the status to publish
		if ( ! empty( $product_id ) ) {
			wp_update_post(
				array(
					'ID'           => $product_id,
					'post_title'   => $product['product_title'],
					'post_status'  => 'publish'
				)
			);

		} else {
			// If product id doesn't exist, create new product then.
			$_product = new \WC_Product_Variable();
			$_product->set_name( $product['product_title'] );
			// $product->set_image_id( 90 );
			$_product->save();

			// Get the new ID
			$product_id = $_product->get_id();

			// Marke it as sold as individually as it is digital content
			update_post_meta( $product_id, '_sold_individually', 'yes' );
			update_post_meta( $product_id, '_manage_stock', 'no' );
			update_post_meta( $product_id, '_stock_status', 'instock' );
			update_post_meta( $product_id, '_backorders', 'no' );
			update_post_meta( $product_id, '_virtual', 'yes' );
			update_post_meta( $product_id, '_downloadable', 'no' );
		}

		// Sync plans
		self::syncVariations( $product_id, $product['plans'] );

		// Update product meta
		update_post_meta( $product_id, self::PRODUCT_META_KEY_FOR_CONTENT, $content_id );

		// Update product id in contents table. Zero means it is content pack product rather.
		if ( $content_id > 0 ) {
			Field::contents()->updateField(
				array( 'product_id' => $product_id ),
				array( 'content_id' => $content_id )
			);
		}

		return $product_id;
	}

	/**
	 * Sync variations with product
	 *
	 * @param int $product_id The connected WooCommerce product ID to add plans to
	 * @param array $content_plans The plans to create/update variation with for the product ID
	 *
	 * @return void
	 */
	private static function syncVariations( $product_id, $content_plans ) {

		// Plans array need to be valid
		if ( empty( $content_plans ) || ! is_array( $content_plans ) ) {
			return;
		}

		// Create attributes first
		$plan_ids = array_map(
			function( $plan ) {
				return $plan['plan']['id'];
			},
			$content_plans
		);
		self::createProductAttribute( $product_id, _Array::flattenArray( $plan_ids ) );

		/**
		 * Loop through the plans and create variation one by one for the product.
		 */
		foreach ( $content_plans as $plan ) {
			
			// Assign dynamic prices per content
			$plan['regular_price'] = $plan['regular_price'] ?? $plan['plan']['default_price'] ?? 0.00;

			// Determine sale price
			$sale_price = $plan['sale_price'] ?? $plan['plan']['default_sale_price'] ?? $plan['plan']['default_price'] ?? null;

			// If sale price is there, use it if it doesn't exceed regular price.
			$plan['sale_price'] = ( empty( $sale_price ) || $sale_price > $plan['regular_price'] ) ? '' : $sale_price;
			
			self::createProductVariation( $product_id, $plan );
		}
	}

	/**
	 * Create product attributes for variations
	 *
	 * @param int $product_id
	 * @param array $attributes
	 * @return void
	 */
	private static function createProductAttribute( $product_id, $attribute_values ) {

		// Get existing attributes
		$product_attributes = _Array::getArray( get_post_meta( $product_id, '_product_attributes', true ) );
		
		// Update product attribute settings
		$product_attributes[ self::LICENSING_VARIATION ] = array(
			'name'         => self::LICENSING_VARIATION,
			'value'        => implode( ' | ', $attribute_values ),
			'position'     => 0,
            'is_visible'   => 1,
            'is_variation' => 1,
            'is_taxonomy'  => 0,
		);

		update_post_meta( $product_id, '_product_attributes', $product_attributes );
	}

	/**
	 * Create or update variations for a product using provided plan configuration array
	 *
	 * @param int $product_id The product ID to create variations for
	 * @param array $plan The plan configuration array
	 * @return void
	 */
	private static function createProductVariation( $product_id, $plan ) {
		
		$plan_key         = $plan['plan']['id'];
		$existing_pointer = self::LICENSING_VARIATION_PREFIX . $product_id;

		// Get existing variation for the plan
		$variation_id       = Contents::getPostIDByMeta( $existing_pointer, $plan_key, 'product_variation' );
		$variation_title    = get_the_title( $product_id ) . ' - ' . $plan['plan']['plan_name'];
		$variation_desc     = ! empty( $plan['plan']['description'] ) ? $plan['plan']['description'] : $plan['plan']['plan_name'];
		$variation_status   = ( $plan['enable'] ?? false ) ? 'publish' : 'private';

		// Create a new variation if not created yet
		if ( empty( $variation_id ) ) {
			$variation_id = wp_insert_post(
				array(
					'post_title'  => $variation_title,
					'post_parent' => $product_id,
					'post_type'   => 'product_variation',
					'post_status' => $variation_status,
					'post_content' => $variation_desc,
				)
			);

		} else {
			// Update the variation title 
			wp_update_post(
				array(
					'ID'           => $variation_id,
					'post_title'   => $variation_title,
					'post_status'  => $variation_status,
					'post_content' => $variation_desc,
				)
			);
		}
		
		// Set variation data
		update_post_meta( $variation_id, $existing_pointer, $plan_key );
		update_post_meta( $variation_id, 'attribute_' . self::LICENSING_VARIATION, $plan_key );
		update_post_meta( $variation_id, '_regular_price', $plan['regular_price'] );
		update_post_meta( $variation_id, '_sale_price', $plan['sale_price'] );
		update_post_meta( $variation_id, '_price', $plan['sale_price'] );
		update_post_meta( $variation_id, '_stock', null );
		update_post_meta( $variation_id, '_virtual', 'yes' );
		update_post_meta( $variation_id, '_downloadable', 'no' );
		update_post_meta( $variation_id, '_variation_description', $variation_desc );
		update_post_meta( $variation_id, '_manage_stock', 'no' );
		update_post_meta( $variation_id, '_stock_status', 'instock' );
		update_post_meta( $variation_id, '_sold_individually', 'yes' );
		update_post_meta( $variation_id, '_backorders', 'no' );
	}

	/**
	 * Disconnect product from content that marks a content as free
	 *
	 * @param int $content_id
	 * @return void
	 */
	public static function disconnectProduct( $content_id, $change_product_status = true ) {
		$product_id = Field::contents()->getField( array( 'content_id' => $content_id ), 'product_id' );
		
		if ( ! empty( $product_id ) ) {
			
			// Mark the product as private as it will not be used since the content itself is going to be free
			if ( $change_product_status ) {
				wp_update_post(
					array(
						'ID'          => $product_id,
						'post_status' => 'private',
					)
				);
			}
			
			// Set the product ID as null that will mean the content is free.
			Field::contents()->updateField(
				array( 'product_id' => null ), 
				array( 'content_id' => $content_id )
			);
		}
	}

	/**
	 * Get content type plan from settings by specific plan key
	 *
	 * @return array|null Associative data array of individual plan from content type settings
	 *   - id (string): Auto generated unique ID
	 *   - plan_name
	 *   - description
	 *   - sales_model
	 *   - subscription_model
	 *   - default_price
	 *   - license_key_or_content_limit
	 */
	public static function getContentTypePlan( $content_type, $plan_key ) {
		
		$content_plans = Utilities::getContentPlanSettings( $content_type );
		$plan          = _Array::find( $content_plans, 'id', $plan_key, array() );

		return $plan;
	}

	/**
	 * Check if a specific file is paid content
	 *
	 * @param int $file_id
	 * @param string|null $content_type
	 * @return bool
	 */
	public static function isFilePaid( $file_id, $content_type = null ) {

		global $wpdb;

		$type_clause = ! empty( $content_type ) ? $wpdb->prepare( " AND _content.content_type=%s", $content_type ) : '';

		$release_id = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT _release.release_id 
				FROM {$wpdb->portfoliya_releases} _release
					INNER JOIN {$wpdb->portfoliya_contents} _content ON _release.content_id=_content.content_id
				WHERE 
					_release.file_id=%d 
					AND _content.product_id IS NOT NULL 
					{$type_clause}",
				$file_id
			)
		);

		return ! empty( $release_id );
	}

	/**
	 * Get purchased content data by specific customer ID
	 *
	 * @param int $customer_id
	 *
	 * @return array
	 */
	public static function getPurchasedContents( $customer_id, $content_id = null ) {

		global $wpdb;

		$where_clause = '';

		// Content ID filter
		$variation_ids = array();
		if ( ! empty( $content_id ) ) {
			$content        = ContentsFree::getContentByContentID( $content_id );
			$variation_ids  = array_column( $content['product']['plans'], 'variation_id' );
			$ids_places     = _String::getSQLImplodesPrepared( $variation_ids );
			$where_clause  .= " AND sale.variation_id IN ({$ids_places})";
		}

		$sales = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT
					content.content_title,
					content.content_type,
					content.content_slug,
					content.product_id,
					sale.*,
					expires_on,
					UNIX_TIMESTAMP(sale.purchase_date) AS purchase_date,
					(sale.expires_on IS NOT NULL AND sale.expires_on<NOW()) AS is_expired,
					(sale.license_or_content_limit IS NOT NULL AND sale.downloads_in_season>=sale.license_or_content_limit) AS limit_reached 
				FROM {$wpdb->portfoliya_sales} sale
					LEFT JOIN {$wpdb->portfoliya_contents} content ON sale.content_id=content.content_id
				WHERE 
					sale.customer_id=%d 
					AND sale.parent_id=0 
					{$where_clause}
				ORDER BY sale.purchase_date DESC",
				$customer_id
			),
			ARRAY_A
		);

		if ( empty( $sales ) ) {
			return array();
		}

		$statuses = array();
		foreach ( $sales as $index => $sale ) {
				
			// Assign order status
			$order_id = $sale['order_id'];
			if ( ! isset( $statuses[ $order_id ] ) ) {
				$status                = $sale['order_status'];
				$statuses[ $order_id ] = function_exists( 'wc_get_order_status_name' ) ? wc_get_order_status_name( $status ) : $status;
			}
			$sales[ $index ]['order_status_label']  = $statuses[ $order_id ];
			$sales[ $index ]['is_download_allowed'] = empty( $sale['is_expired'] ) && empty( $sale['limit_reached'] );
		}

		$sales = self::assignContentMeta( _Array::castRecursive( $sales ) );
		$sales = ContentPack::assignPackMeta( $sales );

		// Keep the purchased plan only, remove others.
		foreach ( $sales as $index => $sale ) {
			$variation_id   = $sale['variation_id'];
			$product_plans  = $sale['product']['plans'];
			$purchased_plan = null;

			foreach ( $product_plans as $_plan ) {
				if ( ( $_plan['variation_id'] ?? null ) == $variation_id ) {
					$purchased_plan = $_plan;
					break;
				}
			}

			$sales[ $index ]['renewals'] = Sale::getRenewalOrders( $sale['sale_id'] );
			$sales[ $index ]['product']['plan'] = $purchased_plan;
			unset( $sales[ $index ]['product']['plans'] );
		}

		return $sales;
	}

	/**
	 * Add product info to a single content data array
	 *
	 * @param array $content Single content data array
	 * @return void
	 */
	public static function addSingleContentMeta( array $content ) {

		// Assign the content type for content pack
		if ( empty( $content['content_id'] ) ) {
			$content['content_type'] = get_post_meta( $content['product_id'], ContentPack::$meta_key, true );
		}
		
		$content['product'] = array(
			'product_id'                => $content['product_id'],
			'monetization'              => empty( $content['product_id'] ) ? 'free' : 'paid',
			self::PURCHASE_OPTIONAL_KEY => self::isProductPurchaseOptional( $content['product_id'] ),
			'plans'                     => self::getContentVariations( $content['product_id'], $content['content_type'] )
		);

		// If there's content ID, get content packs for it. 
		// If no content ID, then it is content pack itself.
		if ( ! empty( $content['content_id'] ) && ! empty( $content['content_type'] ) ) {

			// Get the whole plan settings and enable status for this content
			$pack_statuses = PackLink::getLinking( $content['content_id'] );
			
			// Include the content pack product variations 
			$pack_plans = &$content['product']['plans'];

			// Set the pack status for this content
			foreach ( $pack_plans as $index => $plan ) {

				$plan_id     = $plan['plan']['id'];
				$sales_model = $plan['plan']['sales_model'];
				$auto_enable = $plan['plan']['auto_enable'] ?? false;

				if ( 'pack' === $sales_model ) {
					$pack_plans[ $index ]['enable'] = $pack_statuses[ $plan_id ] ?? $auto_enable;
				}
			}
		}

		return $content;
	}

	/**
	 * Get product title and append variation
	 *
	 * @param int $product_id
	 * @param int $variation_id
	 * 
	 * @return string
	 */
	public static function getProductVariationTitle( $product_id, $variation_id ) {

		$content_type    = self::getContentTypeByProduct( $product_id );
		$product_title   = get_the_title( $product_id );
		$variation_title = '';

		if ( ! empty( $content_type ) ) {
			$variation = self::getVariationInfo( $variation_id, $content_type );
			
			if ( ! empty( $variation ) ) {
				$variation_title = ' - ' . $variation['plan']['plan_name'];
			}
		}

		return $product_title . $variation_title;
	}

	/**
	 * Retrieve a products content type
	 *
	 * @return string
	 */
	public static function getContentTypeByProduct( $product_id ) {
		// First try getting from content pack meta
		$content_type = get_post_meta( $product_id, ContentPack::$meta_key, true );

		// If empty then it's probably content itself
		// Try getting from content table
		if ( empty( $content_type ) ) {
			$content_type = Field::contents()->getField( array( 'product_id' => $product_id ), 'content_type' );
		}

		return $content_type;
	}

	/**
	 * Retrieve content ID by linked product ID
	 *
	 * @param int $product_id
	 * @return int|null
	 */
	public static function getContentIdByProductId( $product_id ) {
		return Field::contents()->getField( array( 'product_id' => $product_id ), 'content_id' );
	}

	/**
	 * Get plan name by plan key
	 *
	 * @param string $plan_key
	 * @param string  $fallback
	 * @return string
	 */
	public static function getPlanNameByKey( $plan_key, $fallback = '' ) {
		$content_types = AdminSetting::get( 'contents');

		// Loop throug all the content type and their plans to find the plan with the key
		foreach ( $content_types as $content ) {
			$plans = $content['plans'] ?? array();
			foreach ( $plans as $plan ) {
				if ( is_array( $plan ) && $plan_key === ( $plan['id'] ?? null ) ) {
					return $plan['plan_name'];
				}
			}
		}

		return $fallback;
	}

	/**
	 * Check how many posts made by the user in last 24 hours
	 *
	 * @param int $user_id
	 * @return int
	 */
	public static function getPostCountin24Hours( $user_id ) {
		
		global $wpdb;

		return ( int ) $wpdb->get_var(
			$wpdb->prepare(
				"SELECT 
					COUNT(content_id) 
				FROM 
					{$wpdb->portfoliya_contents} 
				WHERE 
					contributor_id=%d 
					AND created_at >= NOW() - INTERVAL 1 DAY",
				$user_id
			)
		);
	}

	/**
	 * Get variation label by variation ID that was randomly generated by React
	 *
	 * @param int    $product_id
	 * @param string $variation_id
	 * @param string $fallback
	 * @return string
	 */
	public static function getVariationLabel( $product_id, $variation_id, $fallback = '' ) {
		
		// Get content type
		$content_type = Field::contents()->getField( array( 'product_id' => $product_id ), 'content_type' );

		// Get plans
		if ( ! empty( $content_type ) ) {
			$plans = _Array::getArray( AdminSetting::get( 'contents.' . $content_type . '.plans' ) );
			foreach ( $plans as $plan ) {
				if ( $variation_id === ( $plan['id'] ?? null ) ) {
					return $plan['plan_name'];
				}
			}
		}

		return $fallback;
	}
}
