<?php

namespace Portfoliya_Pro\Models;

use Portfoliya\Helpers\Utilities;
use SolidieLib\_Array;
use Portfoliya\Models\User;

class Contributor {
	/**
	 * The user meta key to store contributor status
	 */
	const FLAG_META_KEY = 'portfoliya-contributor-status';

	const CONTRIBUTOR_STATUSES = array(
		'pending',
		'approved',
		'rejected',
		'banned'
	);

	/**
	 * Check whether a user is approved for contribution
	 *
	 * @param int $user_id
	 * @return boolean
	 */
	public static function getStatus( $user_id ) {
		return User::hasAdministrativeRole( $user_id ) ? 'approved' : get_user_meta( $user_id, self::FLAG_META_KEY, true );
	}

	/**
	 * Update contributor status for user
	 *
	 * @param int $user_id
	 * @param string $status
	 *
	 * @return void
	 */
	public static function updateStatus( $user_id, $status ) {
		if ( empty( $status ) ) {
			delete_user_meta( $user_id, self::FLAG_META_KEY );
		} else {
			update_user_meta( $user_id, self::FLAG_META_KEY, $status );
		}
	}

	/**
	 * Get contributors by args filters
	 *
	 * @param array $args
	 * @return array
	 */
	public static function getContributors( $args, $segmentation = false ) {
		
		global $wpdb;

		$limit  = Utilities::getLimit();
		$page   = Utilities::getPage( $args['page'] ?? null );
		$offset = ( $page - 1 ) * $limit;

		$where_clause = '';

		// Contributor status filter
		if ( ! empty( $args['status'] ) ) {
			$where_clause .= $wpdb->prepare( ' AND _meta.meta_value=%s', $args['status'] );
		}

		// Contributor user email filter
		if ( ! empty( $args['search'] ) ) {
			$keyword = $args['search'];
			$where_clause .= $wpdb->prepare(
				' AND (_user.user_email=%s OR _user.display_name LIKE %s)', 
				$keyword, 
				"%{$wpdb->esc_like( $keyword )}%"
			);
		}

		// If it is segmentation
		if ( $segmentation ) {

			$total_count = (int) $wpdb->get_var(
				$wpdb->prepare(
					"SELECT 
						COUNT(DISTINCT _user.ID)
					FROM {$wpdb->users} _user
						LEFT JOIN {$wpdb->portfoliya_contents} _content ON _user.ID=_content.contributor_id
						LEFT JOIN {$wpdb->usermeta} _meta ON _meta.user_id=_user.ID AND _meta.meta_key=%s
					WHERE
						(
							_content.content_id IS NOT NULL 
							OR _meta.meta_value IS NOT NULL
						) {$where_clause}",
					self::FLAG_META_KEY
				)
			);

			$page_count = ceil( $total_count / $limit );

			return array(
				'total_count' => $total_count,
				'page_count'  => $page_count,
				'page'        => $page,
				'limit'       => $limit,
			);
		}

		$users = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT DISTINCT
					_user.ID AS user_id,
					_user.display_name,
					_user.user_email,
					UNIX_TIMESTAMP(_user.user_registered) AS reg_unix,
					_meta.meta_value AS status
				FROM {$wpdb->users} _user
					LEFT JOIN {$wpdb->portfoliya_contents} _content ON _user.ID=_content.contributor_id
					LEFT JOIN {$wpdb->usermeta} _meta ON _meta.user_id=_user.ID AND _meta.meta_key=%s
				WHERE
					(
						_content.content_id IS NOT NULL 
						OR _meta.meta_value IS NOT NULL
					) {$where_clause}
				ORDER BY 
					_user.user_registered DESC
				LIMIT %d OFFSET %d",
				self::FLAG_META_KEY,
				$limit,
				$offset
			),
			ARRAY_A
		);

		if ( empty( $users ) ) {
			return array();
		}

		$users         = _Array::castRecursive( $users );
		$user_ids      = array_column( $users, 'user_id' );
		$contributions = self::getContributionCounts( $user_ids );
		$earnings      = Earning::getEarningStats( $user_ids );

		// Assign meta data
		foreach ( $users as $index => $user ) {
			$users[ $index ]['avatar_url']         = get_avatar_url( $user['user_id'] );
			$users[ $index ]['contribution_count'] = ( int ) ( $contributions[ $user['user_id'] ] ?? 0 );
			$users[ $index ]['earning_stats']      = $earnings[ $user['user_id'] ] ?? null;
		}

		return $users;
	}

	/**
	 * Get contribution counts per contributors
	 *
	 * @param array $contributor_ids Contributor user IDs
	 * @return array
	 */
	public static function getContributionCounts( array $contributor_ids ) {

		if ( empty( $contributor_ids ) ) {
			return array();
		}

		global $wpdb;

		$ids_in = implode( ',', $contributor_ids );

		$counts = $wpdb->get_results(
			"SELECT 
				COUNT(content_id) AS contribution_count, 
				contributor_id 
			FROM 
				{$wpdb->portfoliya_contents} 
			WHERE 
				contributor_id IN ({$ids_in})
				AND content_status='publish'
			GROUP BY 
				contributor_id",
			ARRAY_A
		);

		$counts = _Array::castRecursive( $counts );
		$counts = _Array::indexify( $counts, 'contributor_id', 'contribution_count' );

		return $counts;
	}
}