<?php

namespace Portfoliya_Pro\Models;

use Portfoliya\Helpers\Utilities;
use SolidieLib\_Array;
use SolidieLib\_Number;
use SolidieLib\_String;
use Portfoliya\Models\AdminSetting;

class Earning {

	/**
	 * Get total earned amount that order is complete and the payment not in delayed period AKA awaiting.
	 *
	 * @param int|array $user_id
	 * @return array
	 */
	private static function getTotalEarnedAmount( $user_id ) {

		global $wpdb;

		$delay     = _Number::getInt( AdminSetting::get( 'contributor_revenue_availability_delay', 0 ) );
		$ids_place = _String::getSQLImplodesPrepared( _Array::getArray( $user_id, true ) );

		// Get non bundle total earning
		$earned = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT 
					_content.contributor_id,
					SUM(_sale.commission) AS commission
				FROM 
					{$wpdb->portfoliya_sales} _sale
					INNER JOIN {$wpdb->portfoliya_contents} _content ON _sale.content_id=_content.content_id
				WHERE 
					_sale.order_status='completed'
					AND _sale.order_complete_date IS NOT NULL
					AND _sale.order_complete_date < DATE_SUB(NOW(), INTERVAL %d DAY)
					AND _content.contributor_id IN ({$ids_place})
				GROUP BY
					_content.contributor_id",
				$delay
			),
			ARRAY_A
		);
		$earned = _Array::groupRows( _Array::castRecursive( $earned ), 'contributor_id', 'commission', true );

		// Get bundle total earn
		$b_earned = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT 
					_cmsn.contributor_id,
					SUM(_cmsn.commission) AS commission
				FROM 
					{$wpdb->portfoliya_bundle_commissions} _cmsn
					INNER JOIN {$wpdb->portfoliya_sales} _sale ON _sale.sale_id=_cmsn.sale_id
				WHERE 
					_sale.order_status='completed'
					AND _sale.order_complete_date IS NOT NULL
					AND _sale.order_complete_date < DATE_SUB(NOW(), INTERVAL %d DAY)
					AND _cmsn.contributor_id IN ({$ids_place})
				GROUP BY
					_cmsn.contributor_id",
				$delay
			),
			ARRAY_A
		);
		$b_earned = _Array::groupRows( _Array::castRecursive( $b_earned ), 'contributor_id', 'commission', true );

		// Merge these
		foreach ( $b_earned as $b_uid => $b_earning ) {
			$earned[ $b_uid ] = ( $earned[ $b_uid ] ?? 0 ) + $b_earning;
		}

		return $earned;
	}

	/**
	 * Get awaiting payment that would be available after the specified delay. 
	 * This amount is not withdrawn, in fact not even available to request withdrawal.
	 * 
	 * @param int|array $user_id
	 * @return array
	 */
	private static function getTotalAwaitingAmount( $user_id ) {

		$delay     = _Number::getInt( AdminSetting::get( 'contributor_revenue_availability_delay', 0 ) );
		$ids_place = _String::getSQLImplodesPrepared( _Array::getArray( $user_id, true ) );

		global $wpdb;

		// Get total non bundle sale awaiting amount
		$awaiting = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT 
					_content.contributor_id,
					SUM(_sale.commission) AS commission
				FROM 
					{$wpdb->portfoliya_sales} _sale
					INNER JOIN {$wpdb->portfoliya_contents} _content ON _sale.content_id=_content.content_id
				WHERE 
					_sale.order_status='completed'
					AND (_sale.order_complete_date IS NULL OR _sale.order_complete_date >= DATE_SUB(NOW(), INTERVAL %d DAY))
					AND _content.contributor_id IN($ids_place)
				GROUP BY
					_content.contributor_id",
				$delay
			),
			ARRAY_A
		);
		$awaiting = _Array::groupRows( _Array::castRecursive( $awaiting ), 'contributor_id', 'commission', true );

		// Get total bundle awaiting sale amount
		$b_awaiting = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT 
					_cmsn.contributor_id,
					SUM(_cmsn.commission) AS commission
				FROM 
					{$wpdb->portfoliya_bundle_commissions} _cmsn
					INNER JOIN {$wpdb->portfoliya_sales} _sale ON _sale.sale_id=_cmsn.sale_id
				WHERE 
					_sale.order_status='completed'
					AND (_sale.order_complete_date IS NULL OR _sale.order_complete_date >= DATE_SUB(NOW(), INTERVAL %d DAY))
					AND _cmsn.contributor_id IN ({$ids_place})
				GROUP BY
					_cmsn.contributor_id",
				$delay
			),
			ARRAY_A
		);
		
		$b_awaiting = _Array::groupRows( _Array::castRecursive( $b_awaiting ), 'contributor_id', 'commission', true );

		// Merge these
		foreach ( $b_awaiting as $b_uid => $b_earning ) {
			$awaiting[ $b_uid ] = ( $awaiting[ $b_uid ] ?? 0 ) + $b_earning;
		}

		return $awaiting;
	}

	/**
	 * Get total withdrawan amount by user. Withdrawal status should not be in rejected or cancelled.
	 *
	 * @param int|array $user_id
	 * @return void
	 */
	private static function getTotalWithdrawan( $user_id ) {

		$ids_place = _String::getSQLImplodesPrepared( _Array::getArray( $user_id, true ) );

		global $wpdb;

		// The total withdrawl amount that is in pending, processing or sent status.
		$withdrawn = $wpdb->get_results(
			"SELECT 
				contributor_id,
				SUM(amount) AS amount
			FROM 
				{$wpdb->portfoliya_withdrawals} 
			WHERE 
				withdrawal_status NOT IN ('rejected', 'cancelled')
				AND contributor_id IN({$ids_place})
			GROUP BY
				contributor_id",
			ARRAY_A
		);
		
		return _Array::groupRows( _Array::castRecursive( $withdrawn ), 'contributor_id', 'amount', true );
	}

	/**
	 * Get earning stats by contribut ID
	 *
	 * @param array|int $user_id
	 * @return array
	 */
	public static function getEarningStats( $user_id ) {

		$user_ids  = _Array::getArray( $user_id, true );
		$withdrawn = self::getTotalWithdrawan( $user_ids );
		$earned    = self::getTotalEarnedAmount( $user_ids );
		$awaiting  = self::getTotalAwaitingAmount( $user_ids );
		
		// Prepare per user earning stat
		$data_array = array();
		foreach ( $user_ids as $u_id ) {

			$_earned    = $earned[ $u_id ] ?? 0;
			$_withdrawn = $withdrawn[ $u_id ] ?? 0;

			// The total amount that can be requested withdrawal. It doesn't include already withdrawn and awaiting ones.
			$_available = $_earned - $_withdrawn;

			$data_array[ $u_id ] = array(
				'withdrawn' => Utilities::numberFormat( $_withdrawn ), 
				'available' => Utilities::numberFormat( $_available ) , 
				'earned'    => Utilities::numberFormat( $_earned ), 
				'awaiting'  => Utilities::numberFormat( $awaiting[ $u_id ] ?? 0 ),
			);
		}

		return ! is_array( $user_id ) ? $data_array[ $user_id ] : $data_array;
	}
}
