<?php

namespace Portfoliya_Pro\Models;

use SolidieLib\_Array;
use SolidieLib\_String;
use Portfoliya\Models\Field;

class Licensing {
	/**
	 * Generate license keys for content sale
	 *
	 * @param array $sale The sale data array to create license keys for
	 * @param int|null $limit How many to generate
	 * @param bool $skip_if_exists Whether to run generator if there are license keys already
	 * @return bool
	 */
	public static function generateLicenseKeys( array $sale, $limit, $skip_if_exists = true ) {
		
		$sale_id = $sale['sale_id'] ?? 0;

		// Skip if already automated creation has run
		if ( empty( $sale_id ) || ( $skip_if_exists === true && ! empty( self::getLicenseKeys( $sale_id ) ) ) ) {
			return false;
		}

		// Limit null means unlimited license, so create 40 initially. Customers can generate more from dashboard later.
		if ( empty( $limit ) ) {
			$limit = 40;
		}

		// Generate license keys
		global $wpdb;
		for( $i = 0; $i < $limit; $i++ ) {
			$license_key = array(
				'sale_id'     => $sale_id,
				'license_key' => self::generateLicenseKey( $sale ),
			);

			$wpdb->insert(
				$wpdb->portfoliya_license_keys,
				$license_key
			);
		}

		return true;
	}

	/**
	 * Check if the license key exists for the sale ID
	 *
	 * @param int $sale_id
	 * @param string $license_key
	 * @return bool
	 */
	private static function licenseKeyExists( $sale_id, $license_key ) {
		global $wpdb;
		$exists = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT license_id FROM {$wpdb->portfoliya_license_keys} WHERE sale_id=%d AND license_key=%s LIMIT 1",
				$sale_id,
				$license_key
			)
		);

		return ! empty( $exists );
	}

	/**
	 * Generate single license for a sale
	 *
	 * @param array $sale The sale data array to generate license key for
	 * @return string
	 */
	private static function generateLicenseKey( array $sale ) {
		$product_id   = str_pad( $sale['product_id'], 5, '0', STR_PAD_LEFT );
		$order_id     = str_pad( $sale['order_id'], 5, '0', STR_PAD_LEFT );
		$variation_id = str_pad( $sale['variation_id'], 5, '0', STR_PAD_LEFT );
		$sale_id      = str_pad( $sale['sale_id'], 5, '0', STR_PAD_LEFT );
		$random       = _String::getRandomString( 5 );
		$final_key    = "{$product_id}-{$order_id}-{$variation_id}-{$sale_id}-{$random}";

		// Generate new key if it exists
		if ( self::licenseKeyExists( $sale['sale_id'], $final_key ) ) {
			$final_key = self::generateLicenseKey( $sale );
		}

		return $final_key;
	}

	/**
	 * Regenerate existing license key
	 *
	 * @param array $sale_plan The existing sale
	 * @param int $license_id Existing license key ID
	 * @return void
	 */
	public static function regenerateLicenseKey( $sale_plan, $license_id ) {
		
		$license_key = self::generateLicenseKey( $sale_plan );
		
		global $wpdb;
		$wpdb->update(
			$wpdb->portfoliya_license_keys,
			array( 
				'license_key' => $license_key, 
				'endpoint' => null 
			),
			array( 
				'license_id' => $license_id,
				'sale_id'    => $sale_plan['sale_id']
			)
		);
	}

	/**
	 * Create new license especially for unlimited or there can be added more. 
	 *
	 * @param array $sale
	 * @param integer $limit
	 * @return bool
	 */
	public static function addNewLicenseKey( $sale_plan, $limit = 1 ) {
		return self::generateLicenseKeys( $sale_plan, $limit, false );
	}

	/**
	 * Get license info
	 *
	 * @param string $license_key
	 * @param integer $content_id
	 * 
	 * @return array
	 */
	public static function getLicenseInfo( $license_key, $content_id ) {

		// Check if there is any sale associated with the license key.
		$fragments = explode( '-', $license_key );
		$sale_id   = (int) ( $fragments[3] ?? 0 );
		$licenses  = $sale_id ? self::getLicenseKeys( $sale_id, $content_id, $license_key ) : array();

		foreach ( $licenses as $license ) {
			if ( $license['license_key'] !== $license_key ) {
				continue;
			}

			$plan_name = '';
			$customer  = get_userdata( $license['customer_id'] );

			if ( class_exists( 'WC_Product_Variation' ) ) {
				$variation = new \WC_Product_Variation( (int) $license['variation_id'] );
				$variation = Contents::getVariationInfo( $variation, $license['content_type'] );
				$plan_name = $variation['plan']['plan_name'];
			}
			
			$license['product_name'] = get_the_title( $license['product_id'] );
			$license['plan_name']    = $plan_name;
			$license['licensee']     = $customer ? $customer->display_name : '(Customer Not Found)';

			return $license;
		}
	}

	/**
	 * Get license key by license ID
	 *
	 * @param int $license_id
	 * @return void
	 */
	public static function getLicenseKeyById( $license_id ) {
		return Field::license_keys()->getField(
			array( 'license_id' => $license_id ), 
			'license_key'
		);
	}

	/**
	 * Get license keys
	 *
	 * @param int|array $sale_id    Sale ID or array of sale IDs
	 * @param int|null $content_id  The product ID
	 * 
	 * @return array
	 */
	public static function getLicenseKeys( $sale_id, $content_id = null, $license_key = null, $customer_id = null ) {
		
		global $wpdb;

		$where_clause = '';

		// Content filter
		if ( null !== $content_id ) {
			$where_clause .= $wpdb->prepare( " AND sale.content_id=%d", $content_id );
		}
		
		// License key filter
		if ( null !== $license_key ) {
			$where_clause .= $wpdb->prepare( " AND license.license_key=%s", $license_key );
		}
		
		// Customer ID filter
		if ( null !== $customer_id ) {
			$where_clause .= $wpdb->prepare( " AND sale.customer_id=%d", $customer_id );
		}
		
		// Sale ID filter
		$sale_ids   = array_values( _Array::getArray( $sale_id, true, 0 ) );
		$ids_places = _String::getSQLImplodesPrepared( $sale_ids );

		$keys = $wpdb->get_results(
			"SELECT 
				license.*, 
				sale.sale_id, 
				sale.expires_on, 
				sale.variation_id, 
				sale.customer_id,
				sale.content_id,
				content.product_id, 
				content.content_type
			FROM 
				{$wpdb->portfoliya_license_keys} license 
				INNER JOIN {$wpdb->portfoliya_sales} sale ON sale.sale_id=license.sale_id
				INNER JOIN {$wpdb->portfoliya_contents} content ON sale.content_id=content.content_id
			WHERE 
				license.sale_id IN ({$ids_places}) 
				{$where_clause} 
			ORDER BY license.license_id DESC",
			ARRAY_A
		);

		$keys = ( is_array( $keys ) && ! empty( $keys ) ) ? $keys : array();
		$keys = _Array::castRecursive( $keys );
		$keys = _Array::groupRows( $keys, 'sale_id' );

		return ! is_array( $sale_id ) ? ( $keys[ $sale_id ] ?? array() ) : $keys;
	}

	/**
	 * Get generated license key count for the sale ID
	 *
	 * @param int $sale_id
	 * @return int
	 */
	public static function getKeyCount( $sale_id ) {
		
		global $wpdb;
		
		$count = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT 
					COUNT(license_id) 
				FROM 
					{$wpdb->portfoliya_license_keys} 
				WHERE
					sale_id=%d",
				$sale_id
			)
		);

		return ( int ) $count;
	}

	/**
	 * Delete license key by id and sale ID
	 *
	 * @param int $license_id
	 * @param int|null $sale_id
	 * @return void
	 */
	public static function deleteKey( $license_id, $sale_id = null ) {

		$where               = array();
		$where['license_id'] = $license_id;

		if ( ! empty( $sale_id ) ) {
			$where['sale_id'] = $sale_id;
		}

		global $wpdb;
		$wpdb->delete(
			$wpdb->portfoliya_license_keys,
			$where
		);
	}
}
