<?php

namespace Portfoliya_Pro\Models;

use SolidieLib\_Array;
use Portfoliya\Helpers\Utilities;

class Message {

	/**
	 * Send message
	 *
	 * @param integer $sender_id
	 * @param integer $recipient_id
	 * @param string $message
	 *
	 * @return int
	 */
	public static function sendMessage( int $sender_id, int $recipient_id, string $message ) {
		
		global $wpdb;
		
		$wpdb->insert(
			$wpdb->portfoliya_messages,
			array(
				'message_content' => $message,
				'message_type'    => 'text',
				'sender_id'       => $sender_id,
				'recipient_id'    => $recipient_id,
				'sent_time'       => Utilities::gmDate(),
			)
		);

		return $wpdb->insert_id;
	}

	/**
	 * Get messages
	 *
	 * @param int $sender_id
	 * @param int $recipient_id
	 * @return array
	 */
	public static function getMessages( $sender_id, $recipient_id, bool $mark_seen, array $args = array() ) {
		
		global $wpdb;

		$where_clauses = array();

		// Load previous messages
		if ( ! empty( $args['older_than'] ) ) {
			$where_clauses[] = $wpdb->prepare( 'message_id<%d', $args['older_than'] );
		}

		// Load new messages that are not inside text list
		if ( ! empty( $args['later_than'] ) ) {
			$where_clauses[] = $wpdb->prepare( 'message_id>%d', $args['later_than'] );
		}

		$where_clause = ! empty( $where_clauses ) ? ' AND (' . implode( ' OR ', $where_clauses ) . ')' : '';

		$messages = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT * 
				FROM {$wpdb->portfoliya_messages} 
				WHERE 
					((sender_id=%d AND recipient_id=%d) OR (sender_id=%d AND recipient_id=%d))
					{$where_clause}
				ORDER BY 
					message_id DESC
				LIMIT 100",
				$sender_id,
				$recipient_id,
				$recipient_id,
				$sender_id
			),
			ARRAY_A
		);

		if ( $mark_seen ) {
			$gm_date = Utilities::gmDate();
			$wpdb->update(
				$wpdb->portfoliya_messages,
				array( 'read_time' => $gm_date, 'seen_time' => $gm_date ),
				array( 'recipient_id' => $sender_id, 'read_time' => null )
			);
		}

		return array_reverse( _Array::castRecursive( $messages ) );
	}

	/**
	 * Get unseen message count
	 *
	 * @param int $user_id
	 * @return int
	 */
	public static function getUnseeCount( $user_id ) {
		
		global $wpdb;

		return (int) $wpdb->get_var(
			$wpdb->prepare(
				"SELECT 
					COUNT(message_id) 
				FROM 
					{$wpdb->portfoliya_messages} 
				WHERE 
					recipient_id=%d 
					AND seen_time IS NULL",
				$user_id
			)
		);
	}

	/**
	 * Delete messages by user
	 *
	 * @param int $user_id
	 * @return void
	 */
	public static function deleteByUser( $user_id ) {

		global $wpdb;

		$wpdb->query( 
			$wpdb->prepare( 
				"DELETE FROM {$wpdb->portfoliya_messages} WHERE sender_id=%d OR recipient_id=%d", 
				$user_id, 
				$user_id 
			) 
		);
	}

	/**
	 * Get individual thread/room/peer informations
	 *
	 * @param int $recipient_id
	 * @param array $message_ids
	 * @return array
	 */
	private static function getThreadInfo( $recipient_id, $message_ids ) {

		if ( empty( $message_ids ) ) {
			return array();
		}
		
		global $wpdb;

		$ids = implode( ',', $message_ids );

		$threads = $wpdb->get_results(
			"SELECT 
				_msg.sender_id,
				_msg.recipient_id,
				_msg.message_id,
				_msg.message_content,
				_msg.read_time,
				_msg.seen_time,
				UNIX_TIMESTAMP(_msg.sent_time) as sent_time
			FROM
				{$wpdb->portfoliya_messages} _msg
			WHERE
				_msg.message_id IN ({$ids})
			ORDER BY 
				_msg.message_id DESC",
			ARRAY_A
		);

		$threads   = _Array::castRecursive( $threads );
		$new_array = array();

		foreach ( $threads as $thread ) {

			$u1 = min( $thread['sender_id'], $thread['recipient_id'] );
			$u2 = max( $thread['sender_id'], $thread['recipient_id'] );
			$pointer = $u1 . '_' . $u2;
			if ( isset( $new_array[ $pointer ] ) ) {
				continue;
			}
			
			// Determine peer ID
			$peer_id = $thread['sender_id'] == $recipient_id ? $thread['recipient_id'] : $thread['sender_id'];
			
			// Prepare thread list
			$thread['peer_id']      = $peer_id;
			$thread['avatar_url']   = get_avatar_url( $peer_id );
			$thread['display_name'] = get_userdata( $peer_id )->display_name;
			$thread['is_unread']    = $thread['recipient_id'] == $recipient_id && empty( $thread['read_time'] );
			$thread['is_unseen']    = $thread['recipient_id'] == $recipient_id && empty( $thread['seen_time'] );
		
			$new_array[ $pointer ] = $thread;
		}

		return array_values( $new_array );
	}

	/**
	 * Get messages threads for a epecific recipient
	 *
	 * @param int $recipient_id
	 * @param boolean $unseen_only
	 * @param boolean $mark_seen
	 * @return array
	 */
	public static function getMessageThreads( $recipient_id, $unseen_only = false, $mark_seen = false ) {
		
		global $wpdb;

		$where_clause = '';

		if ( $unseen_only ) {
			$where_clause .= ' AND seen_time IS NULL';
		}

		$message_ids = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT 
					MAX(message_id) AS message_id
				FROM 
					{$wpdb->portfoliya_messages}
				WHERE 
					recipient_id = %d OR sender_id = %d
					{$where_clause}
				GROUP BY 
					sender_id",
				$recipient_id,
				$recipient_id
			)
		);
		
		if ( $mark_seen ) {
			$wpdb->update(
				$wpdb->portfoliya_messages,
				array( 'seen_time' => Utilities::gmDate() ),
				array( 'recipient_id' => $recipient_id, 'seen_time' => null )
			);
		}

		return self::getThreadInfo( $recipient_id, $message_ids );
	}
}
