<?php

namespace Portfoliya_Pro\Models;

use Portfoliya\Helpers\Utilities;
use SolidieLib\_Array;
use SolidieLib\_DateTime;
use SolidieLib\_Number;
use SolidieLib\_String;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\Contents as PortfoliyaContents;

class Sale {

	/**
	 * Get bulk sales
	 * 
	 * @param array $args Args to get sales by.
	 *
	 * @return array
	 */
	public static function getSales( array $args, $segmentation = false ) {

		$page   = Utilities::getPage( $args['page'] ?? null );
		$limit  = Utilities::getLimit( $args['limit'] ?? null );
		$offset = ( $page - 1 ) * $limit;
		$delay  = _Number::getInt( AdminSetting::get( 'contributor_revenue_availability_delay', 0 ) );

		// Prepare where conditions clause
		global $wpdb;
		$where_clause = '';

		// Generic sales filter
		$to_filter = array( 'customer_id', 'order_id', 'variation_id' );
		foreach ( $to_filter as $column ) {
			if ( isset( $args[ $column ] ) ) {
				$where_clause .= $wpdb->prepare( " AND _sale.{$column}=%d", $args[ $column ] );
			}
		}

		// Contributor filter
		if ( ! empty( $args['contributor_id'] ) ) {
			$where_clause .= $wpdb->prepare( ' AND _content.contributor_id=%d', $args['contributor_id'] );
		}

		// Whether to load only main order, not renewals
		if ( ! isset( $args['parent_id'] ) || is_numeric( $args['parent_id'] ) ) {
			$where_clause .= $wpdb->prepare( ' AND _sale.parent_id=%d', $args['parent_id'] ?? 0 );
		}

		if ( $segmentation ) {
			
			$total_count = ( int ) $wpdb->get_var(
				"SELECT 
					COUNT(DISTINCT _sale.sale_id)
				FROM 
					{$wpdb->portfoliya_sales} _sale 
					INNER JOIN {$wpdb->portfoliya_contents} _content ON _sale.content_id=_content.content_id
				WHERE 
					1=1 {$where_clause}"
			);

			$page_count = ceil( $total_count / $limit );

			return array(
				'total_count' => $total_count,
				'page_count'  => $page_count,
				'page'        => $page,
				'limit'       => $limit,
			);
		}

		// Get sales data by arguments
		$sales = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT 
					_sale.*,
					(_sale.order_complete_date IS NULL OR _sale.order_complete_date >= DATE_SUB(NOW(), INTERVAL %d DAY)) AS is_awaiting,
					_content.content_title,
					_content.content_type
				FROM 
					{$wpdb->portfoliya_sales} _sale 
					LEFT JOIN {$wpdb->portfoliya_contents} _content ON _sale.content_id=_content.content_id
				WHERE 
					1=1 {$where_clause} 
				ORDER BY 
					_sale.purchase_date DESC
				LIMIT %d OFFSET %d",
				$delay,
				$limit,
				$offset
			),
			ARRAY_A
		);

		$sales = _Array::castRecursive( $sales );

		// Add the plan details
		foreach ( $sales as $index => $sale ) {
			$sale_plan = Sale::getSalePlan( $sale );
			$sales[ $index ]['is_awaiting'] = ( int ) $sale['is_awaiting'];
			$sales[ $index ]['variation']   = is_array( $sale_plan ) ? ( $sale_plan['variation'] ?? null ) : null;
		}

		return $sales;
	}

	/**
	 * Get single sale
	 *
	 * @param array $args
	 * @param mixed $fallback
	 *
	 * @return array|null
	 */
	public static function getSale( array $args, $fallback = null ) {

		$sales = self::getSales( array_merge( $args, array( 'limit' => 1 ) ) );

		return ! empty( $sales ) ? $sales[0] : $fallback;
	}

	/**
	 * Check if the user has purchased the content
	 *
	 * @param int $content_id
	 * @param int $user_id
	 * @param boolean $check_validity
	 * 
	 * @return boolean
	 */
	public static function hasCustomerPurchase( $content_id, $user_id, $check_validity ) {
		$exp_clause = $check_validity ? " AND enabled=1 AND order_status='completed' AND (expires_on IS NULL OR expires_on>NOW())" : '';

		global $wpdb;
		$has = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT 
					sale_id 
				FROM 
					{$wpdb->portfoliya_sales} 
				WHERE 
					content_id=%d 
					AND customer_id=%d 
					AND parent_id=0 
					{$exp_clause}",
				$content_id,
				$user_id
			)
		);

		return $has;
	}

	/**
	 * Get the plan of a single sale. Note, multiple product 
	 *
	 * @param int|array $sale_id Sale ID or sale data array
	 * @return array
	 */
	public static function getSalePlan( $sale_id ) {
		global $wpdb;
		$sale = is_array( $sale_id ) ? $sale_id : $wpdb->get_row(
			$wpdb->prepare(
				"SELECT 
					sale.*, 
					content.content_title,
					content.content_type 
				FROM 
					{$wpdb->portfoliya_sales} sale
					LEFT JOIN {$wpdb->portfoliya_contents} content ON sale.content_id=content.content_id
				WHERE sale.sale_id=%d",
				$sale_id
			),
			ARRAY_A
		);

		if ( empty( $sale ) ) {
			return null;
		}

		$sale = _Array::castRecursive( $sale );

		if ( empty( $sale['content_id'] ) ) {
			$pack = ContentPack::getContentPackByProduct( $sale['product_id'] );
			$sale['content_type'] = $pack ? $pack['content_type'] : null;
		} else {
			$sale['content_permalink'] = PortfoliyaContents::getPermalink( $sale['content_id'] );
		}
		
		// Add the plan name
		$sale['variation'] = Contents::getVariationInfo( $sale['variation_id'], $sale['content_type'] );

		return $sale;
	}

	/**
	 * Update single sale information
	 *
	 * @param integer $sale_id
	 * @param array $info
	 * @return void
	 */
	public static function updateSale( $sale_id, array $info ) {
		global $wpdb;
		$wpdb->update(
			$wpdb->portfoliya_sales,
			$info,
			array( 'sale_id' => $sale_id )
		);
	}

	/**
	 * Get parent sale row by order id and variation id
	 *
	 * @param int $order_id
	 * @param int $variation_id
	 * @return array|null
	 */
	public static function getParentSale( $order_id, $variation_id ) {
		
		global $wpdb;

		$parent = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT * FROM {$wpdb->portfoliya_sales} WHERE sale_id=(SELECT parent_id FROM {$wpdb->portfoliya_sales} WHERE order_id=%d AND variation_id=%d)",
				$order_id,
				$variation_id
			),
			ARRAY_A
		);

		return ! empty( $parent ) ? $parent : null;
	}

	/**
	 * Get sub orders for a main order
	 *
	 * @param int $sale_id Sale IDs which are root level
	 * @return array
	 */
	public static function getRenewalOrders( $sale_id ) {
		
		$singular   = ! is_array( $sale_id );
		$sale_ids   = ! $singular ? $sale_id : array( $sale_id );
		$ids_places = _String::getSQLImplodesPrepared( $sale_ids );

		global $wpdb;

		$renewals = $wpdb->get_results(
			"SELECT 
				*, 
				UNIX_TIMESTAMP(purchase_date) AS purchase_date 
			FROM 
				{$wpdb->portfoliya_sales} 
			WHERE 
				parent_id!=0 
				AND parent_id IN({$ids_places})",
			ARRAY_A
		);

		$renewals = _Array::castRecursive( $renewals );

		// Add order edit link
		if ( is_admin() ) {
			foreach ( $renewals as $index => $renewal ) {
				$renewals[ $index ]['order_url'] = get_edit_post_link( $renewal['order_id'], null );
			}
		}

		$group = _Array::groupRows( $renewals, 'parent_id' );

		return $singular ? ( $group[ $sale_id ] ?? array() ) : $group;
	}

	/**
	 * Get all the order details
	 *
	 * @param int|object $order_id
	 * @return array
	 */
	public static function getOrderDetails( $order_id ) {

		// Get the order object
		$order = is_object( $order_id ) ? $order_id : wc_get_order( $order_id );

		// Initialize an array to store order details
		$order_details = array();


		// Check if the order object exists
		if ( $order ) {
			// Order ID
			$order_details['order_id'] = $order->get_id();

			// Billing and shipping addresses
			$order_details['billing_address']  = $order->get_formatted_billing_address();
			$order_details['shipping_address'] = $order->get_formatted_shipping_address();
			$order_details['billing_email']    = $order->get_billing_email();
			$order_details['billing_phone']    = $order->get_billing_phone();

			// Order items
			$items = $order->get_items();
			$order_details['items'] = array();
			foreach ( $items as $item_id => $item ) {
				$product = $item->get_product();
				$order_details['items'][] = array(
					'id'       => $item->get_id(),
					'name'     => $item->get_name(),
					'quantity' => $item->get_quantity(),
					'subtotal' => $order->get_item_subtotal( $item, false, false ),
				);
			}

			$status = str_replace( 'wc-', '', $order->get_status() );

			$order_details['order_subtotal']     = $order->get_subtotal();
			$order_details['order_total']        = $order->get_total();
			$order_details['order_currency']     = $order->get_currency();
			$order_details['order_status']       = $status;
			$order_details['order_status_label'] = function_exists( 'wc_get_order_status_name' ) ? wc_get_order_status_name( $status ) : $status;
			$order_details['order_date']         = $order->get_date_created();
			$order_details['payment_method']     = $order->get_payment_method_title();
			$order_details['date']               = $order->get_date_created()->format( 'F j, Y' );
		}

		return $order_details;
	}

	/**
	 * Prepare sales data for contributor view
	 *
	 * @param array $sales
	 * @return array
	 */
	public static function prepareSalesForContributor( $sales ) {

		$allowed_keys = array(
			'sale_id',
			'content_title',
			'commission',
			'purchase_date',
			'order_status',
			'order_complete_date',
			'is_awaiting',
			'variation'
		);

		$new_array = array();

		foreach ( $sales as $sale ) {

			$new_element = array();
			
			foreach ( $allowed_keys as $key ) {
				$new_element[ $key ] = $sale[ $key ] ?? null;
			}

			$new_array[] = $new_element;
		}

		return $new_array;
	}

	/**
	 * Delete sales by specific field
	 *
	 * @param string $field
	 * @param int $value
	 * @return void
	 */
	public static function disableSalesByField( $field, $value ) {

		global $wpdb;

		// Disable the sales access
		$wpdb->update(
			$wpdb->portfoliya_sales,
			array( 'enabled' => 0 ),
			array( $field => $value )
		);
	}

	/**
	 * Get reveneue stats
	 *
	 * @return array
	 */
	public static function getRevenueStats() {
		
		global $wpdb;

		// Get total
		$total = $wpdb->get_row(
			"SELECT 
				SUM(sale_price) AS total_revenue,
				SUM(commission) AS total_commission
			FROM 
				{$wpdb->portfoliya_sales}
			WHERE 
				order_status='completed'",
			ARRAY_A
		);
		
		// Get commission
		$gross_revenue          = ( int ) $total['total_revenue'];
		$contributor_commission = ( int ) $total['total_commission'];

		// Get refunds
		$total_refunds = ( int ) $wpdb->get_var(
			"SELECT 
				SUM(sale_price) 
			FROM 
				{$wpdb->portfoliya_sales} 
			WHERE 
				order_status='refunded'"
		);

		// Get net
		$net_revenue = $gross_revenue - $contributor_commission - $total_refunds;

		return compact( 
			'gross_revenue', 
			'contributor_commission', 
			'total_refunds', 
			'net_revenue' 
		);
	}

	/**
	 * Get sales trend per month
	 *
	 * @return array
	 */
	public static function getSalesTrend() {

		global $wpdb;
		$sales_trend = $wpdb->get_results(
			"SELECT 
				DATE_FORMAT(purchase_date, '%Y-%m') AS sales_month, 
				SUM(sale_price) AS total_revenue
			FROM 
				{$wpdb->portfoliya_sales}
			WHERE 
				order_status='completed' AND
				purchase_date >= DATE_SUB(CURDATE(), INTERVAL 1 YEAR)
			GROUP BY 
				sales_month
			ORDER BY 
				sales_month ASC",
			ARRAY_A
		);

		$sales_trend = _Array::castRecursive( $sales_trend );

		$trend = array(
			'data'   => array(),
			'labels' => array()
		);

		foreach ( $sales_trend as $sales ) {
			$trend['data'][]   = $sales['total_revenue'];
			$trend['labels'][] = $sales['sales_month'];
		}

		if ( count( $trend['data'] ) === 1 ) {
			array_unshift( $trend['data'], 0 );
			array_unshift( $trend['labels'], _DateTime::getPreviousYearMonth( $trend['labels'][0] ) );
		}

		return $trend;
	}
}
