<?php

namespace Portfoliya_Pro\Models;

use Portfoliya\Helpers\Utilities;
use SolidieLib\_Array;
use SolidieLib\_Number;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\Field;

class Withdrawal {

	/**
	 * User meta key to store withdrawal methods
	 */
	const METHOD_META_KEY = 'portfoliya-contributor-withdrawal-methods';

	/**
	 * Possible withdrawal statuses
	 */
	const STATUSES = array(
		'pending',
		'processing',
		'sent',
		'rejected',
		'cancelled'
	);

	/**
	 * Update the payment method
	 *
	 * @param int $user_id
	 * @param string $method
	 * @param array $fields
	 * @return void
	 */
	public static function updateWithdrawalMethod( $user_id, $method, $fields ) {
		
		$values = array();

		// Prepare updated method
		switch ( $method ) {
			case 'paypal'   :
			case 'payoneer' :
			case 'wise'     :
			case 'interac'  :
				if ( ! empty( $fields['email'] ) ) {
					$values['email'] = $fields['email'];
				}
				break;
		}

		// Get existing methods to merge into
		$methods            = self::getMethods( $user_id );
		$methods[ $method ] = $values;

		update_user_meta( $user_id, self::METHOD_META_KEY, $methods );
	}

	/**
	 * Get payment method of a user
	 *
	 * @param int $user_id
	 * @param string|null $method_single
	 *
	 * @return void
	 */
	public static function getMethods( $user_id, $method_single = null ) {
		
		$saved_methods = _Array::getArray( get_user_meta( $user_id, self::METHOD_META_KEY, true ) );
		$enableds      = _Array::getArray( AdminSetting::get( 'supported_withdrawal_methods' ) );
		$methods       = array();

		foreach ( $enableds as $method ) {
			$methods[ $method ] = _Array::getArray( $saved_methods[ $method ] ?? null );
		}

		return $method_single ? ( $methods[ $method_single ] ?? null ) : $methods;
	}

	/**
	 * Get the minimum withdrawal amount
	 *
	 * @return int
	 */
	public static function getMinimumWithdrawal() {
		return _Number::getInt( AdminSetting::get( 'contributor_minimum_withdrawal_amount' ), 1 );
	}

	/**
	 * Create withdrawal request entry
	 *
	 * @param int $user_id
	 * @param string $method
	 * @param int $amount
	 * @return void
	 */
	public static function requestWithdrawal( $user_id, $method, $amount ) {
		global $wpdb;
		$wpdb->insert(
			$wpdb->portfoliya_withdrawals,
			array(
				'amount'            => $amount,
				'contributor_id'    => $user_id,
				'withdrawal_status' => 'pending',
				'request_date'      => gmdate( 'Y-m-d H:i:s' ),
				'withdrawal_method' => $method,
				'method_details'    => maybe_serialize( self::getMethods( $user_id, $method ) )
			)
		);
	}

	/**
	 * Get single withdrawal BY withdrawal ID
	 *
	 * @param int $withdrawal_id
	 * @return array|null
	 */
	public static function getRequestById( $withdrawal_id ) {
		$withdrawals = self::getRequests(
			array( 
				'withdrawal_id' => $withdrawal_id,
				'limit'         => 1
			) 
		);

		return $withdrawals[0] ?? null;
	}

	/**
	 * Get withdrawal requests
	 *
	 * @param array $args
	 * @param bool $segmentation
	 *
	 * @return array
	 */
	public static function getRequests( $args, $segmentation = false ) {

		$page   = Utilities::getPage( $args['page'] ?? null );
		$limit  = Utilities::getLimit( $args['limit'] ?? null );
		$offset = ( $page - 1 ) * $limit;
		$order  = $args['order'] ?? 'DESC';

		global $wpdb;
		
		$where_clause = '';

		// Contributor ID filter
		if ( ! empty( $args['contributor_id'] ) ) {
			$where_clause .= $wpdb->prepare( ' AND _withdrawal.contributor_id=%d', $args['contributor_id'] );
		}

		// Contributor email filter
		if ( ! empty( $args['contributor_email'] ) ) {
			$where_clause .= $wpdb->prepare( ' AND _user.user_email=%s', $args['contributor_email'] );
		}

		// Withdrawal request status
		if ( ! empty( $args['status'] ) ) {
			$where_clause .= $wpdb->prepare( ' AND _withdrawal.withdrawal_status=%s', $args['status'] );
		}

		// Withdrawal ID filter
		if ( ! empty( $args['withdrawal_id'] ) ) {
			$ids = _Array::getArray( $args['withdrawal_id'], true );
			$ids = implode( ',', $ids );
			$where_clause .= " AND _withdrawal.withdrawal_id IN ($ids)";
		}

		// If it is segmentation
		if ( $segmentation ) {

			$total_count = ( int ) $wpdb->get_var(
				"SELECT 
					COUNT(DISTINCT _withdrawal.withdrawal_id)
				FROM 
					{$wpdb->portfoliya_withdrawals}  _withdrawal
					INNER JOIN {$wpdb->users} _user ON _withdrawal.contributor_id=_user.ID
				WHERE 
					1=1 {$where_clause}"
			);

			$page_count = ceil( $total_count / $limit );

			return array(
				'total_count' => $total_count,
				'page_count'  => $page_count,
				'page'        => $page,
				'limit'       => $limit,
			);
		}

		$requests = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT 
					_withdrawal.*,
					UNIX_TIMESTAMP(_withdrawal.request_date) AS request_date,
					UNIX_TIMESTAMP(_withdrawal.sent_date) AS sent_date,
					_user.user_email,
					_user.display_name
				FROM 
					{$wpdb->portfoliya_withdrawals} _withdrawal
					INNER JOIN {$wpdb->users} _user ON _withdrawal.contributor_id=_user.ID
				WHERE 
					1=1 {$where_clause}
				ORDER BY 
					_withdrawal.request_date {$order}
				LIMIT 
					%d OFFSET %d",
				$limit,
				$offset
			),
			ARRAY_A
		);

		$requests = _Array::castRecursive( $requests );

		foreach ( $requests as $index => $request ) {
			$requests[ $index ]['method_details'] = maybe_unserialize( $request['method_details'] );
			$requests[ $index ]['avatar_url']     = get_avatar_url( $request['contributor_id'] );
		}

		return $requests;
	}

	/**
	 * Change request status
	 *
	 * @param int $withdrawal_id
	 * @param string $status
	 * @return void
	 */
	public static function changeRequestStatus( $withdrawal_id, $status ) {

		$updates = array( 'withdrawal_status' => $status );

		if ( $status === 'sent' ) {
			$updates['sent_date'] =  gmdate( 'Y-m-d H:i:s' );
		}

		Field::withdrawals()->updateField(
			$updates,
			array( 'withdrawal_id' => $withdrawal_id )
		);
	}

	/**
	 * Delete withdrawal by contributor id
	 *
	 * @param int $user_id
	 * @return void
	 */
	public static function deleteWithdrawalByContributor( $user_id ) {
		
		global $wpdb;

		$wpdb->delete(
			$wpdb->portfoliya_withdrawals,
			array( 'contributor_id' => $user_id )
		);
	}
}
