<?php

namespace Portfoliya_Pro\Setup;

use Portfoliya\Models\Contents;
use Portfoliya_Pro\Models\ContentPack;
use Portfoliya_Pro\Models\Sale;
use Portfoliya_Pro\Models\Contents as ProContents;

class Content {
	public function __construct() {

		// Add default pro settings to the manifest
		add_filter( 'portfoliya_manifest', array( $this, 'addDefaultSettings' ) );

		// Apply restriction on paid tutorial access if visitor doesn't have capabilities
		add_action( 'portfoliya_load_lesson_before', array( $this, 'restrictPaidTutorial' ), 10, 2 );

		// Load content packs in gallery page to filter contents with
		add_filter( 'portfoliya_gallery_resources', array( $this, 'loadContentPacksInGallery' ) );

		// Modify get contents query
		add_filter( 'portfoliya_get_contents_where_clause', array( $this, 'getContentsWhere' ), 10, 2 );

		// Delete linked sale on content delete
		add_action( 'portfoliya_content_delete', array( $this, 'deleteContentData' ), 10, 2 );

		// Add bundle plans list in content list response to use as filter dropdown
		add_filter( 'portfoliya_content_list_response', array( $this, 'bundlePlansProvide' ), 10, 2 );
	}

	/**
	 * Restrict paid tutorial access
	 *
	 * @param int $content_id
	 * @param string $lesson_path
	 * @return void
	 */
	public function restrictPaidTutorial( $content_id, $lesson_path ) {

		// Check paid content access
		$user_id = get_current_user_id();
		if ( ! Contents::isUserCapableToManage( $content_id, $user_id ) ) {
			if ( ! ProContents::isContentFree( $content_id, true ) ) {

				if ( empty( $user_id ) ) {
					wp_send_json_error( array( 'message' => __('Please login first. You may need to purchase this course if not already.') ) );
				}

				// Get the purchase
				$purchases          = ProContents::getPurchasedContents( $user_id, $content_id );
				$has_valid_purchase = false;
				$pack_sale          = false;

				foreach ( $purchases as $purchase ) {
					if ( $purchase['is_download_allowed'] ) {

						$has_valid_purchase = true;

						// If no content ID, it means pack purchase
						if ( empty( $purchase['content_id'] ) ) {
							$pack_sale = $purchase;
						}
						break;
					}
				}

				if ( ! $has_valid_purchase ) {
					wp_send_json_error( array( 'message' => __('You need purchase this tutorial first. If you did already, then it is expired.') ) );
				}

				// So the user is eligible to access, now 
				if ( $has_valid_purchase && $pack_sale ) {
					// Enlist the content as accessed through pack purchase
					ContentPack::increaseStats( get_current_user_id(), $pack_sale['sale_id'], $content_id, true );
				}
			}
		}
	}

	/**
	 * Add default settings to manifest
	 *
	 * @param array $manifest
	 * 
	 * @return array
	 */
	public function addDefaultSettings( array $manifest ) {
		
		$manifest['settings']['general'] = array_merge(
			$manifest['settings']['general'],
			array(
				'frontend_dashboard_path'            => 'my-dashboard',
				'contribution_distribute_model'      => array(),
				'public_contributor_approval'        => true,
				'redirect_product_to_content_page'   => true,
				'redirect_product_status_code'       => 302,
				'content_lesson_attachment_max_size' => 1,
				'content_lesson_attachment_supports' => array(),
				'public_contribution_approval'       => 'none',
				// 'contributor_messaging_enable'       => true,
				'daily_contributor_content_limit'    => 10,
			)
		);

		foreach ( $manifest['settings']['contents'] as $content_type => $content ) {
			$manifest['settings']['contents'][ $content_type ]['purchasable'] = $content_type !== 'classified';
		}
		
		return $manifest;
	}

	/**
	 * Undocumented function
	 *
	 * @param array $contents
	 * @param array $meta_array
	 * @return array
	 */
	public function contentMeta( array $contents, array $meta_array ) {

		// Same content will be available multiple times in the array because of different plans/sale id even though same order. 
		// We need to merge them here based on unique content ID by adding plan as array.
		$new_array = array();
		foreach ( $contents as $index => $content ) {
			// Create placeholder in new array
			if ( ! isset( $new_array[ $content->content_id ] ) ) {
				$new_array[ $content->content_id ]        = $content;
				$new_array[ $content->content_id ]->plans = array();
			}

			// Check if sales info available
			if ( empty( $content->sale_id ) ) {
				continue;
			}

			// Add plan to the plan array. Same content may have multiple plan purchased over times.
			$plan = Sale::getSalePlan( $content->sale_id );
			if ( ! empty( $plan ) ) {
				$new_array[ $content->content_id ]->plans[] = $plan;
			}
			
			// Remove the sale ID now as the array is getting consolidated.
			unset( $new_array[ $content->content_id ]->sale_id );
		}

		// Finally replace the content array to be returned
		$contents = array_values( $new_array );
	}

	/**
	 * Load content packs in gallery for filtering contents
	 *
	 * @param array  $resources
	 * @param string $content_type
	 * @param array  $content_settings
	 * @return void
	 */
	public function loadContentPacksInGallery( $resources ) {

		$resources['content_pack_plans'] = ContentPack::getBulkContentPackPlans();

		return $resources;
	}

	/**
	 * Delete sale on content delete
	 *
	 * @param int $content_id
	 * @param array $content
	 * @return void
	 */
	public function deleteContentData( $content_id, $content ) {

		global $wpdb;

		Sale::disableSalesByField( 'content_id', $content_id );

		// Delete associated woocommerce product
		$product_id = ! empty( $content['product_id'] ) ? $content['product_id'] : Contents::resurrectProductID( $content_id );
		if ( ! empty( $product_id ) ) {
			wp_delete_post( $product_id, true );
		}

		// Delete content pack linking
		$wpdb->delete(
			$wpdb->portfoliya_content_pack_link,
			array( 'content_id' => $content_id )
		);
	}

	/**
	 * Add bundle filter for contents query
	 *
	 * @param string $where_clause
	 * @param array $args
	 * @return string
	 */
	public function getContentsWhere( $where_clause, $args ) {

		global $wpdb;
		
		if ( ! empty( $args['content_type'] ?? null ) && ! empty( $args['content_pack_plan'] ) ) {
				
			$is_auto = ContentPack::isPackAutoEnabled( $args['content_pack_plan'] );

			if ( $is_auto ) {
				$where_clause .= $wpdb->prepare( 
					' AND (pack.pack_id IS NULL OR (pack.pack_id=%s AND pack.enabled=1))', 
					$args['content_pack_plan'] 
				);
			} else {
				$where_clause .= $wpdb->prepare( 
					' AND pack.pack_id=%s AND pack.enabled=1', 
					$args['content_pack_plan'] 
				);
			}

			// Include only paid in bundle filter, bundle can't be free
			$where_clause .= ' AND content.product_id IS NOT NULL';
		}
		
		return $where_clause;
	}

	/**
	 * Add content plans to use as filter dropdown
	 *
	 * @param array $response
	 * @param array $args
	 * @return array
	 */
	public function bundlePlansProvide( array $response, array $args ) {

		if ( $args['is_gallery'] ) {
			return $response;
		}

		$response['bundles'] = ContentPack::getBulkContentPackPlans( $args['filters']['content_type'] );
		
		return $response;
	}
}
