<?php

namespace Portfoliya_Pro\Setup;

use Portfoliya\Helpers\Utilities;
use Portfoliya\Main as PortfoliyaMain;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\User;
use Portfoliya_Pro\Main;
use Portfoliya_Pro\Setup\Route as SetupRoute;

class Dashboard {

	/**
	 * Slug for backend dashboard withdrawal requests page
	 */
	const SLUG_WITHDRAWALS = 'portfoliya-withdrawal-requests';

	/**
	 * Slug for contributor list page
	 */
	const SLUG_CONTRIBUTORS = 'portfoliya-contributors';

	/**
	 * The page key for my dashboard
	 */
	const SLUG_MY_DASHBOARD = 'portfoliya-my-dashboard';

	/**
	 * The page key for my reports
	 */
	const SLUG_REPORTS = 'portfoliya-reports';
	
	/**
	 * Register frontend and backend stuffs
	 */
	public function __construct() {
		add_filter( 'portfoliya_frontend_routes', array( $this, 'registerDashboard' ) );
		add_action( 'portfoliya_admin_menu_before_settings', array( $this, 'registerBackendPages' ) );
		add_action( 'portfoliya_admin_menu_after_settings', array( $this, 'registerBackendPages2' ) );
		add_action( 'admin_init', array( $this, 'redirectToFrontendDashboard' ) );
		add_filter( 'plugin_action_links_' . Main::$configs->basename, array( $this, 'dashboardLink' ) );
		add_filter( 'portfoliya_pro_upgrade_card_home', array( $this, 'upgradeCardAlter' ) );
	}

	/**
	 * Show dashboard link in plugins page for pro
	 *
	 * @param array $actions
	 * @return void
	 */
	public function dashboardLink( array $actions ) {

		$actions['portfoliya_pro_dashboard_link'] = '<a href="' . esc_url( Utilities::getBackendPermalink( 'portfoliya' ) ) . '">
			<span style="color: #00aa00; font-weight: bold;">' .
				__( 'Dashboard', 'portfoliya' ) .
			'</span>
		</a>';

		return $actions;
	}

	/**
	 * Register route for user dashboard
	 *
	 * @param array $routes
	 * @return void
	 */
	public function registerDashboard( array $routes ) {
		$path = self::getPath();

		if ( ! empty( $path ) ) {
			$routes[] = $path;
		}

		return $routes;
	}

	/**
	 * Get custom path for user dashboard
	 *
	 * @return string
	 */
	public static function getPath( $append = '' ) {
		return AdminSetting::get( 'frontend_dashboard_path', '' ) . $append;
	}

	/**
	 * Check if current page is user dashboard
	 *
	 * @return boolean
	 */
	public static function isFrontDashboard() {
		$page  = ! is_admin() ? get_query_var( SetupRoute::KEY ) : '';
		$page  = trim( $page, '/' );
		$page  = explode( '/', $page );
		$_path = $page[0] ?? '';

		$path = self::getPath();
		return ! is_admin() && ! empty( $path ) && $_path === $path;
	}

	/**
	 * Generate dashboard URL for specific path
	 *
	 * @param string $path
	 * @return string
	 */
	public static function getUrl( $path = null ) {
		return get_home_url() . '/' . self::getPath() . '/' . ( $path ? $path . '/' : '' );
	}

	/**
	 * Check whether the page is woocommerce order page in backend dashboard
	 *
	 * @return boolean
	 */
	public static function isWCOrderPage() {
		return is_admin() && ( ( $_GET['page'] ?? '' ) === 'wc-orders' || ( function_exists( 'get_the_ID' ) && get_post_field( 'post_type', get_the_ID() ) === 'shop_order' ) );
	}

	/**
	 * Register backend dashboard pages
	 *
	 * @return void
	 */
	public function registerBackendPages() {
		
		$role = User::getPortfoliyaAdminRole();

		add_submenu_page(
			PortfoliyaMain::$configs->root_menu_slug,
			esc_html__( 'Withdrawals', 'portfoliya' ), 
			esc_html__( 'Withdrawals', 'portfoliya' ), 
			$role, 
			self::SLUG_WITHDRAWALS, 
			array( $this, 'withdrawalRequestsPage' ) 
		);

		add_submenu_page(
			PortfoliyaMain::$configs->root_menu_slug,
			esc_html__( 'Contributors', 'portfoliya' ), 
			esc_html__( 'Contributors', 'portfoliya' ), 
			$role, 
			self::SLUG_CONTRIBUTORS, 
			array( $this, 'contributorsPage' ) 
		);

		add_submenu_page(
			PortfoliyaMain::$configs->root_menu_slug,
			esc_html__( 'Reports', 'portfoliya' ), 
			esc_html__( 'Reports', 'portfoliya' ), 
			$role, 
			self::SLUG_REPORTS, 
			array( $this, 'reportsPage' ) 
		);
	}

	/**
	 * Register menus after settings
	 *
	 * @return void
	 */
	public function registerBackendPages2() {

		$role = User::getPortfoliyaAdminRole();

		add_submenu_page(
			PortfoliyaMain::$configs->root_menu_slug,
			esc_html__( 'My Dashboard', 'portfoliya' ), 
			esc_html__( 'My Dashboard', 'portfoliya' ), 
			$role, 
			self::SLUG_MY_DASHBOARD,
			function(){}
		);
	}

	/**
	 * Redirect to user dashboard
	 *
	 * @return void
	 */
	public function redirectToFrontendDashboard() {
		if ( ( $_GET['page'] ?? '' ) === self::SLUG_MY_DASHBOARD ) {
			wp_safe_redirect( Dashboard::getUrl() );
			exit;
		}
	}

	/**
	 * Page content for withdrawal requests page
	 *
	 * @return void
	 */
	public function withdrawalRequestsPage() {
		echo '<div id="portfoliya_backend_withdrawal_requests"></div>';
	}

	/**
	 * Page content for contributor list
	 *
	 * @return void
	 */
	public function contributorsPage() {
		echo '<div id="portfoliya_contributor_list_page"></div>';
	}

	/**
	 * Page content for reports
	 *
	 * @return void
	 */
	public function reportsPage() {
		echo '<div id="portfoliya_reports_page"></div>';
	}

	/**
	 * Alter upgrade card with dashboard URL in plugin home
	 *
	 * @return string
	 */
	public function upgradeCardAlter() {
		ob_start();
		?>
		<a class="col-12" href="<?php echo self::getUrl() ?>" target="_blank">
			<div class="service-item d-flex flex-column justify-content-center text-center rounded">
				<div class="service-icon btn-square">
					<i class="fa fa-tachometer-alt fa-2x"></i>
				</div>
				<h5 class="mb-3"><?php _e( 'User Dashboard', 'portfoliya' ); ?></h5>
				<p><?php _e( 'Dedicated dashboard for contributors and customers.', 'portfoliya' ); ?></p>
			</div>
		</a>
		<?php
		return ob_get_clean();
	}
}
