<?php
namespace Portfoliya_Pro\Setup;

use SolidieLib\_Array;
use Portfoliya\Models\Field;
use Portfoliya\Models\Token;
use Portfoliya\Models\User;
use Portfoliya_Pro\Main;
use Portfoliya_Pro\Models\ContentPack;
use Portfoliya_Pro\Models\Contents;
use Portfoliya_Pro\Models\Licensing;
use Portfoliya_Pro\Models\Sale;

class Downloader {

	public function __construct() {
		add_filter( 'portfoliya_release_download_link', array( $this, 'addAPICredentials' ), 10, 2 );
		add_action( 'portfoliya_load_file_before', array( $this, 'restrictDownload' ) );
	}

	/**
	 * Add release credential to release download links
	 *
	 * @param string $url
	 * @param array $payload
	 * @return string
	 */
	public function addAPICredentials( $url, $payload ) {

		$release    = $payload['release'];
		$license_id = $payload['license_id'];

		// If it is not paid app, no need to add tokenizations.
		// If it is other type of paid contents, it will be validated through portfoliya_load_file_before hook.
		if ( ! Contents::isFilePaid( $release['file_id'], 'app' ) ) {
			return $url;
		}

		$token = Token::generateToken(
			array(
				'file_id'    => $release['file_id'],
				'license_id' => $license_id,
			)
		);

		$args = array(
			'tid'   => $token['id'],
			'token' => $token['token'],
		);
		
		return add_query_arg( $args, $url );
	}

	/**
	 * Validate access to release file before downloading
	 *
	 * @param int $file_id The file ID which is about to be downloaded
	 *
	 * @return void
	 */
	public function restrictDownload( int $file_id ) {
		
		$args = $_GET;

		// Pass if the file is not paid or the user is administrator.
		// Or even if the file ID is empty, then subsequent scripts will take care of it.
		// The purpose here is to restrict only paid contents.
		if ( empty( $file_id ) || ! Contents::isFilePaid( $file_id ) || User::validateRole( get_current_user_id(), 'administrator' ) ) {
			// Return means alowed to download
			return;
		}

		// Get the content by file ID. 
		// Every single release will have unique file. 
		// So get content ID by file ID.
		$content_id = Field::releases()->getField( array( 'file_id' => $file_id ), 'content_id' );
		$content    = Contents::getContentByContentID( $content_id );

		// If the optional purchase is enabled for downloading, no need further check
		if ( is_array( $content['product'] ?? null ) && ( $content['product'][ Contents::PURCHASE_OPTIONAL_KEY ] ?? false ) ) {
			// Return means allowed to download
			return;
		}

		// If it is app and downloading through licensing API
		if ( 'app' === $content['content_type'] && ! empty( $args['tid'] ) && ! empty( $args['token'] ) ) {
			
			$data       = _Array::getArray( Token::getData( $args['tid'], $args['token'] ) );
			$license_id = $data['license_id'] ?? 0;
			$_file_id   = $data['file_id'] ?? 0;

			if ( $file_id === $_file_id ) {
				// Check if the license is valid
				$license_key = $license_id ? Licensing::getLicenseKeyById( $license_id ) : null;
				if ( ! empty( $license_key ) ) {
					$license = Licensing::getLicenseInfo( $license_key, $content_id );
					if ( ! empty( $license ) ) {
						// Return means allowed to download
						return;
					}
				}
			}
		}

		// So finally, no matter the content type. 
		// Download through user logged in and purchased status.

		$user_id = get_current_user_id();

		// Authors can download their own contents without valid purchase
		if ( $content['contributor_id'] == $user_id ) {
			// Return means allowed to download
			return;
		}

		// Empty sale ID means it is single content purchase. 
		if ( empty( $args['sale_id'] ) ) {
			if ( Sale::hasCustomerPurchase( $content['content_id'], $user_id, true ) ) {
				// Return means allowed to download
				return;
			}

		} else if ( ContentPack::validatePackSale( $content['content_type'], $user_id, $args['sale_id'] ) ) {

			// Non empty sale ID means it is content pack, and donwloading the content through the pack
			// Content pack download URLs will always have sale ID as GET paramater.

			// Increase the contributor commission for the download 
			ContentPack::adjustBundleSaleCommission( $content, $args['sale_id'] );

			// Increase the content pack download count
			ContentPack::increaseStats( $user_id, $args['sale_id'], $content['content_id'] );

			// And return to allow download by subsequent scripts
			return;
		}
		
		// Still not returned means no valid reason found to download, and show error page.
		include Main::$configs->dir . 'templates/release-download-error.php';
		http_response_code( 403 );
		exit;
	}
}
