<?php
/**
 * Manage inbound update, license key activation requests
 */
namespace Portfoliya_Pro\Setup;

use SolidieLib\_Array;
use Portfoliya\Main;
use Portfoliya\Models\AdminSetting;
use Portfoliya_Pro\Models\Contents;
use Portfoliya\Models\Field;
use Portfoliya_Pro\Models\Licensing;
use Portfoliya\Models\Release;

/**
 * Inbound Rest API request handler class
 */
class RestAPI {
	
	/**
	 * Request data
	 *
	 * @var array Get parameters
	 */
	private $get;
	
	/**
	 * Request data
	 *
	 * @var array Post parameters
	 */
	private $post;

	/**
	 * Required fields for all type of actions like update check, download update etc.
	 *
	 * @var array
	 */
	const REQUIRED_PARAMS = array(
		'app_id', // Actually content slug technically
		'endpoint',
		'action'
	);

	public function __construct() {
		add_action( 'init', array( $this, 'licenseAPI' ) );
	}

	/**
	 * Get API path for app updates and license activation.
	 * It will return path with both the leading and trailing slash.
	 *
	 * @return string
	 */
	public static function getAPIPath() {

		// Default path 
		$path = '/portfoliya-api/';

		// Get path from app settings
		$settings_path = AdminSetting::get( 'contents.app.api_path' );
		$settings_path = is_string( $settings_path ) ? str_replace( ' ', '', $settings_path ) : '';

		// Prepare final path
		$path = ! empty( $settings_path ) ? $settings_path : $path;
		$path = trim( $path, '/' );
		$path = "/{$path}/";

		return $path;
	}

	/**
	 * The dispatcher to handle different type of request actions
	 *
	 * @return void
	 */
	public function licenseAPI() {

		// Check if it is api request
		$url         = explode( '?', Main::$configs->current_url );
		$current_url = trailingslashit( $url[0] );
		$portfoliya_api = get_home_url() . self::getAPIPath();
		if ( $portfoliya_api !== $current_url ) {
			return;
		}

		// Unslash request data
		$this->get  = _Array::getArray( wp_unslash( $_GET ) );
		$this->post = _Array::getArray( wp_unslash( $_POST ) ); 

		// Set locale if need
		$locale = $this->get['locale'] ?? $this->post['locale'] ?? null;
		if ( ! empty( $locale ) && is_string( $locale ) ) {
			setlocale( LC_ALL, $locale );
		}

		// Loop through required fields and check if exists
		foreach ( self::REQUIRED_PARAMS as $field ) {
			if ( empty( $this->post[ $field ] ) ) {
				wp_send_json_error( array( 'message' => esc_html__( 'Invalid Request Data', 'portfoliya' ) ) );
				exit;
			}

			$this->post[ $field ] = sanitize_text_field( $this->post[ $field ] );
		}

		// Check and get license data. 
		// It will terminate if the license is not usable.
		$license_data = $this->getLicenseData();

		// Process request
		switch ( $this->post['action'] ) {
			case 'activate-license' :
				if ( $license_data['is_free'] ?? false ) {
					// wp_send_json_error( array( 'message' => esc_html__( 'Free apps do not need license activation', 'portfoliya' ) ) );
					// exit;
				}
				
				$this->activateLicense( $license_data );
				break;

			case 'update-check' :
				$this->checkUpdate( $license_data );
				break;

			default :
				wp_send_json_error( array( 'message' => esc_html__( 'Invalid action!', 'portfoliya'  ) ) );
				exit;
		}
	}

	/**
	 * Get license info
	 *
	 * @return array
	 */
	private function getLicenseData() {

		// Check if content exists
		$content_id   = Contents::getContentIdBySlug( $this->post['app_id'] );
		if ( empty( $content_id ) ) {
			$message = esc_html__( 'App not found', 'portfoliya' );
			if ( 'activate-license' === $this->post['action'] ) {
				$message = esc_html__( 'The app not found to activate license for.', 'portfoliya' );
			}
			wp_send_json_error( array( 'message' => $message ) );
			exit;
		}

		// If content is free, return identifier, so callers can decide next operations
		if ( Contents::isContentFree( $content_id, false ) ) {
			return array(
				'is_free'    => true,
				'content_id' => $content_id,
				'license_id' => 0
			);
		}

		// If no free, get license info
		// If no license found, then it is either malformed or maybe content id is not same for the license, or maybe even the license key not passed.
		$l_key        = $this->post['license_key'] ?? '';
		$license_info = ! empty( $l_key ) ? Licensing::getLicenseInfo( $l_key, $content_id ) : null;
		if ( empty( $license_info ) ) {
			wp_send_json_error(
				array( 
					'message'   => esc_html__( 'Invalid License Key', 'portfoliya'  ),
					'activated' => false
				) 
			);
			exit;
		}

		// If the action is activate, then current endpoint must be null or same as provided endpoint (In case duplicate call) which means slot availabe for the dnpoint. 
		if ( 
				'activate-license' === $this->post['action']
				&& null !== $license_info['endpoint'] 
				&& $this->post['endpoint'] !== $license_info['endpoint']
		) {
			wp_send_json_error(
				array(
					'message'   => esc_html__( 'The license key is in use somewhere else already.', 'portfoliya'  ),
					'activated' => false
				)
			);
			exit;
		}

		// If the action is non activate, then the both endpoint must match to check update or download. 
		if ( $this->post['action'] !== 'activate-license' && $license_info['endpoint'] !== $this->post['endpoint'] ) {
			wp_send_json_error(
				array( 
					'message'   => esc_html__( 'The license key is not associated with your endpoint.', 'portfoliya'  ), 
					'activated' => false
				)
			);
			exit;
		}

		return $license_info;
	}

	/**
	 * Activate license key
	 *
	 * @param array $license
	 * @return void
	 */
	private function activateLicense( array $license ) {

		if ( $license['endpoint'] === $this->post['endpoint'] ) {
			$message = esc_html__( 'The license is activated already', 'portfoliya'  );
		} else {
			Field::license_keys()->updateField(
				array( 'endpoint' => $this->post['endpoint'] ),
				array( 'license_id' => $license['license_id'] )
			);
		
			$message = esc_html__( 'License activated succefully', 'portfoliya'  );
		}
		
		wp_send_json_success(
			array(
				'license_key' => $license['license_key'],
				'activated'   => true,
				'licensee'    => $license['licensee'],
				'expires_on'  => $license['expires_on'],
				'plan_name'   => $license['plan_name'],
				'message'     => $message,
				'endpoint'    => $this->post['endpoint'],
			)
		);
		exit;
	}

	/**
	 * Update check for content
	 *
	 * @param array $license
	 * @return void
	 */
	private function checkUpdate( array $license ) {

		$release = Release::getRelease( $license['content_id'], null, $license['license_id'] ?? 0, $this->post['endpoint'] );
		if ( ! $release ) {
			wp_send_json_error( array( 'message' => esc_html__( 'No release found.' ) ) );
			exit;
		}

		wp_send_json_success(
			array(
				'content_permalink' => $release['content_permalink'],
				'version'           => $release['version'],
				'release_timestamp' => $release['release_date'],
				'changelog'         => ( string ) ( $release['changelog'] ?? ''),
				'download_url'      => $release['download_url'],
			)
		);
		
		exit;
	}
}
