<?php
/**
 * Manage product related functionalities
 * 
 * @package portfoliya
 */
namespace Portfoliya_Pro\Setup;

use SolidieLib\_Array;
use Portfoliya\Helpers\Utilities;
use Portfoliya\Main as PortfoliyaMain;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\Field;
use Portfoliya\Models\User;
use Portfoliya\Setup\Route;
use Portfoliya_Pro\Helpers\Utilities as HelpersUtilities;
use Portfoliya_Pro\Main;
use Portfoliya_Pro\Models\Sale;
use Portfoliya_Pro\Models\ContentPack;
use Portfoliya_Pro\Models\Contents;
use Portfoliya_Pro\Models\PackLink;
use SolidieLib\_Number;

/**
 * WooCommerce class
 */
class WooCommerce {

	/**
	 * Register hooks to process monetizations
	 */
	public function __construct() {

		// Prevent content operation if public contribution is disabled
		add_filter( 'portfoliya_before_create_update_content', array( $this, 'preventPublicontribution' ), 10, 2 );

		// Create or update product after content saving
		add_action( 'portfoliya_content_created', array( $this, 'setUpMonetization' ), 10, 3 );
		add_action( 'portfoliya_content_updated', array( $this, 'setUpMonetization' ), 10, 3 );

		// Mark content as free when product gets trashed
		add_action( 'wp_trash_post', array( $this, 'freeContentOnProductTrash' ) );

		// Create content pack product after settings saved
		add_action( 'portfoliya_settings_updated', array( $this, 'syncPlanProducts' ) );

		// Add linked product data to contents
		add_filter( 'portfoliya_contents_meta', array( $this, 'addBulkContentMeta' ) );

		// Create sales entry after purchase and update it on order status change
		add_action( 'woocommerce_payment_complete', array( $this, 'afterPurchaseAction' ) );
		add_action( 'woocommerce_order_status_changed', array( $this, 'orderStatusChange' ), 100, 4 );
		add_action( 'woocommerce_update_order', array( $this, 'syncOrderInfoUpdate' ), 10, 1 );

		add_action( 'admin_notices', array( $this, 'adminNotices' ) );
		add_action( 'wp', array( $this, 'redirectProduct' ) );

		// Hide linked products from everywhere
		// add_action( 'pre_get_posts', array( $this, 'hideProductFromNormalViews' ) );

		// Modify variation name to human readable text in various places
		add_action( 'woocommerce_cart_item_name', array( $this, 'overrideProductTitle' ), 100, 3 );
		add_filter( 'woocommerce_order_item_display_meta_key', array( $this, 'alterPlanKey' ), 10, 1 );
		add_filter( 'woocommerce_order_item_display_meta_value', array( $this, 'alterPlanName' ), 10, 2 );
		add_filter( 'woocommerce_order_item_get_name', array( $this, 'productGetName' ), 10, 2 );
		add_filter( 'woocommerce_product_variation_get_name', array( $this, 'productGetName' ), 10, 2 );
		add_filter( 'woocommerce_get_item_data', array( $this, 'itemMeta' ), 10, 2 );

		// Show purchase info and controls in woocommerce order page after items
		add_action( 'woocommerce_after_order_itemmeta', array( $this, 'orderItemInfo' ), 10, 3 );

		// Show portfoliya state to product title
		add_filter( 'display_post_states', array( $this, 'addPortfoliyaPostState' ), 10, 2 );

		// Auto complete order on payment completion
		add_action( 'woocommerce_payment_complete', array( $this, 'autoCompleteOrders' ) );

		// Delete sales entry on order delete
		add_action( 'before_delete_post', array( $this, 'deleteSaleOnOrderDelete' ) );

		// Add custom content to WooCommerce order details page
		add_action( 'woocommerce_after_order_details', array( $this, 'orderDetailsPageContents' ), 100 );	
		
		// Override variation dropdown option labels in WooCommerce product/variation editor panel
		add_filter( 'woocommerce_variation_option_name', array( $this, 'overrideVariationOptionName' ), 10, 4 );

		// Alter woocommerce variation title to remove generated id string
		add_filter( 'woocommerce_product_variation_title', array( $this, 'alterVariationTitle' ), 10, 4 );

		add_filter( 'portfoliya_get_content_plans', array( $this, 'getProductPlans' ), 10, 3 );
	}

	public function getProductPlans( $plans, $content_slug, $enabled_only = true ) {
		return Contents::getProductVariationsByContentSlug( $content_slug, $enabled_only );
	}

	/**
	 * Replace generated variation ID from contents
	 *
	 * @param string $title
	 * @param object $variation
	 * @param string $title_base
	 * @param string $title_suffix
	 * @return string
	 */
	public function alterVariationTitle(  $title, $variation, $title_base, $title_suffix ) {

		$variation_label = ! empty( $title_suffix ) ? Contents::getVariationLabel( $variation->get_parent_id(), $title_suffix, '' ) : null;
		if ( ! empty( $variation_label ) ) {
			$title = $title_base . ' - ' . $variation_label;
		}

		return $title;
	}

	/**
	 * Show human readable variation label in product editor
	 *
	 * @param string $name
	 * @param mixed $option
	 * @param string $attr_name
	 * @param object $prod
	 * @return string
	 */
	public function overrideVariationOptionName( $name, $option, $attr_name, $prod ) {
		
		if ( Contents::LICENSING_VARIATION === $attr_name ) {
			$name = Contents::getVariationLabel( $prod->get_id(), $name, $name );
		}

		return $name;
	}

	/**
	 * Prevent public contribution if not enabled
	 *
	 * @param bool $proceed
	 * @param array $args
	 * @return bool|array
	 */
	public function preventPublicontribution( $proceed, $args ) {

		$uid = get_current_user_id();

		// Check if contribution is enabled or not
		$contribute_models = _Array::getArray( AdminSetting::get( 'contribution_distribute_model' ) );
		if ( empty( $args['content']['is_admin'] ) && empty( $contribute_models ) && ! User::hasAdministrativeRole( $uid ) ) {
			return array( 'message' => __( 'Public contribution is not enabled', 'portfoliya' ) );
		}

		// Check if daily limit reached or not
		if ( empty( $args['content']['content_id'] ) && ! User::hasAdministrativeRole( $uid ) ) {
			$limit = _Number::getInt( AdminSetting::get( 'daily_contributor_content_limit', 1 ) );
			if ( Contents::getPostCountin24Hours( $uid ) >= $limit ) {
				return array( 
					'blackout' => true,
					'message'  => sprintf( __( 'You have reached daily post limit %d', 'portfoliya' ), $limit )
				);
			}
		}

		return $proceed;
	}

	/**
	 * Redirect WooCommerce prodcuct to our custom URLs
	 *
	 * @return void
	 */
	public function redirectProduct() {

		if ( is_admin() || ! is_single() || ! is_singular() || ! empty( get_query_var( Route::KEY ) ) || ! AdminSetting::get( 'redirect_product_to_content_page' ) ) {
			return;
		}

		$product_id = get_the_ID();
		$content    = Contents::getContentByProduct( $product_id );

		// No need to redirect If not a valid content,
		if ( ! $content || ! Contents::isContentEnabled( $content ) ) {

			// For content pack content will not be found as it is not linked to any
			// Check content pack now rather
			if ( ContentPack::isProductContentPack( $product_id ) ) {
				wp_safe_redirect( function_exists( 'wc_get_page_permalink' ) ? wc_get_page_permalink('shop') : get_home_url() );
				exit;
			}

			return;
		}
		
		$permalink = Contents::getPermalink( $content['content_id'] );
		$status_code = AdminSetting::get( 'redirect_product_status_code' );
		wp_safe_redirect( $permalink, $status_code == 301 ? 301 : 302 );
		exit;
	}

	/**
	 * Hide linked woocommerce products from everywhere
	 *
	 * @return void
	 */
	public function hideProductFromNormalViews( $query ) {

		// Hide product from backend dashboard
		if ( is_admin() && $query->is_main_query() && $query->get('post_type') == 'product' ) {
			
			$existing_meta_queries = $query->get('meta_query');

			$custom_meta_query = array(
				'relation' => 'AND',
				array(
					'key'     => Contents::PRODUCT_META_KEY_FOR_CONTENT,
					'compare' => 'NOT EXISTS',
				),
			);

			// Merge existing meta queries with your custom query
			if ( $existing_meta_queries ) {
				$meta_query = array_merge( $existing_meta_queries, $custom_meta_query );
			} else {
				$meta_query = $custom_meta_query;
			}

			$query->set( 'meta_query', $meta_query );
		}

		// Hide from non admin archive
		if ( ! is_admin() && $query->is_main_query() && ( $query->is_post_type_archive('product') || $query->is_tax('product_cat') || $query->is_tax('product_tag'))) {
			// Add your custom meta key to exclude products without this key
			$meta_query_shop = array(
				'relation' => 'AND', // Use 'AND' or 'OR' based on your needs
				array(
					'key'     => Contents::PRODUCT_META_KEY_FOR_CONTENT,
					'compare' => 'NOT EXISTS',
				),
			);

			// Merge existing meta queries with your custom query for the shop page
			$query->set('meta_query', $meta_query_shop);
		}
	}

	/**
	 * Create content sales entry after linked product purchased
	 *
	 * @param int $order_id
	 * @return void
	 */
	public function afterPurchaseAction( $order_id ) {
		// Process purchase
		Contents::processPurchase( wc_get_order( $order_id ) );
	}

	/**
	 * Change sale status on order status change
	 *
	 * @param int $order_id
	 * @param string $old_status
	 * @param string $new_status
	 * @param object $order
	 * @return void
	 */
	public function orderStatusChange( $order_id, $old_status, $new_status, $order  ) {

		Contents::processPurchase( $order );

		Contents::syncSaleStatus( $order_id, $old_status, $new_status );
	}

	/**
	 * Sync customer ID change on order Update
	 *
	 * @param int $order_id
	 * @return void
	 */
	public function syncOrderInfoUpdate( $order_id ) {
		
		$order           = wc_get_order($order_id);
		$new_customer_id = $order->get_customer_id();

		Field::sales()->updateField(
			array( 'customer_id' => $new_customer_id ),
			array( 'order_id' => $order_id )
		);
	}

	/**
	 * Show a notice in native product creation page
	 *
	 * @return void
	 */
	public function adminNotices() {

		$class = 'notice notice-error';

		// Show guest mode error
		if ( Utilities::isAdminDashboard() && get_option('woocommerce_enable_guest_checkout') === 'yes' ) {
			$message = esc_html__( 'WooCommerce guest checkout must be disabled for Portfoliya monetization', 'portfoliya' );
			printf( '<div class="%1$s"><p>%2$s. <a href="%3$s">%4$s</a></p></div>', esc_attr($class), $message, admin_url( 'admin.php' ) . '?page=wc-settings&tab=account', __('Go Disable', 'portfoliya') );
		}

		// Show content edit warning
		if ( ! isset( $_GET['post'], $_GET['action'] ) || $_GET['action'] !== 'edit' || ! Contents::isProductContent( (int) $_GET['post'] ) ) {
			return;
		}
		$message = esc_html__( 'This product was created programmatically by Portfoliya. Do not edit manually unless you know the consequences.', 'portfoliya' );
		printf('<div class="%1$s"><p style="color:#aa0000;font-weight:bold;">%2$s</p></div>', esc_attr($class), $message);
	}

	/**
	 * Create/update/link/unlink a product with content on content created/update.
	 *
	 * @param int   $content_id The content ID
	 * @param array $inserted_content The prepared array inserted into database
	 * @param array $raw_content The raw post data array
	 * @return void
	 */
	public function setUpMonetization( $content_id, $inserted_content, $raw_content ) {

		// Prepare contribution distribution flags
		$contribute_models = _Array::getArray( AdminSetting::get( 'contribution_distribute_model' ) );
		$has_free          = in_array( 'free', $contribute_models );
		$has_paid          = in_array( 'paid', $contribute_models );
		$has_both          = $has_free && $has_paid;
		$only_paid         = ! $has_free && $has_paid;
		$is_admin          = ( bool ) ( $raw_content['is_admin'] ?? false );

		// Prepare monetization data
		$product           = _Array::getArray( $raw_content['product'] ?? null );
		$product_plans     = _Array::getArray( $product['plans'] ?? null );
		$is_free           = ( $is_admin || $has_both ) ? 'paid' !== ( $product['monetization'] ?? 'free' ) : !$only_paid;
		$content_type      = $inserted_content['content_type'];
		$optional_purchase = ( $is_admin || $has_both ) && (bool) ( $product[ Contents::PURCHASE_OPTIONAL_KEY ] ?? false );
		
		// If free, just unlink the product from content.
		if ( $is_free ) {
			// If free, unlink if there is existing connected product.
			Contents::disconnectProduct( $content_id );
			return;
		}

		// Get the current pack statuses for this content
		$pack_statuses = PackLink::getLinking( $content_id );

		// Get existing product ID 
		$product_id = Contents::resurrectProductID( $content_id );

		// Add missing plans to the array from monetization settings
		$monetization_models = HelpersUtilities::getContentPlanSettings( $content_type );
		foreach ( $monetization_models as $model ) {
			
			// If already present in the plans, skip.
			foreach ( $product_plans as $_plan ) {
				if ( $model['id'] == $_plan['plan']['id'] ) {
					continue 2;
				}
			}

			// Add new plan to the plans array
			$product_plans[] = array(
				'product_id'    => $product_id,
				'regular_price' => $model['default_price'],
				'sale_price'    => $model['default_sale_price'] ?? $model['default_price'],
				'enable'        => $model['enable'] ?? true,
				'plan'          => $model
			);
		}

		// Filter pack plans out of the plans array as it is not supposed to create variation for them per content.
		// And get the pack plan IDs which has been marked as disabled
		$singular_plans = array_filter(
			$product_plans,
			function ( $plan ) use ( &$pack_statuses ) {
				
				$is_singular = 'single' === ( $plan['plan']['sales_model'] ?? null );

				if ( ! $is_singular ) {
					$pack_statuses[ $plan['plan']['id'] ] = ( bool ) $plan['enable'];
				}

				return $is_singular;
			}
		);

		// Store the pack ID linking with this content
		PackLink::updateLinking( $content_id, $pack_statuses );
		
		// Prepare the product first to link to the content
		$product = array(
			'product_id'    => $product_id,
			'product_title' => $inserted_content['content_title'],
			'plans'         => array_values( $singular_plans ),
		);

		// Create/update product
		$product_id = Contents::addProduct( $product, $inserted_content['content_id'] );

		error_log( var_export( $optional_purchase, true ) );
		// Update optional purchase flag meta
		update_post_meta( $product_id, Contents::PURCHASE_OPTIONAL_KEY, $optional_purchase );
	}

	/**
	 * Add bulk contents meta
	 *
	 * @param array $contents
	 * @return array
	 */
	public function addBulkContentMeta( array $contents ) {
		foreach ( $contents as $index => $content ) {
			$contents[ $index ] = Contents::addSingleContentMeta( $content );
		}
		return $contents;
	}

	/**
	 * Mark content as free when linked product gets trashed
	 *
	 * @param int $post_id
	 * @return void
	 */
	public function freeContentOnProductTrash( $post_id ) {
		$content_id = Contents::getContentIdByProductId( $post_id );
		if ( ! empty( $content_id ) ) {
			Contents::disconnectProduct( $content_id, false );
		}
	}

	/**
	 * Create product for content packs
	 *
	 * @param array $settings Updated settings array
	 * @return void
	 */
	public function syncPlanProducts( $settings ) {

		global $wpdb;

		// Gather plan IDs
		$plan_ids = array();

		// Loop through content types in the settings
		foreach ( $settings['contents'] as $content_type => $content ) {
			
			$plans   = _Array::getArray( $content['plans'] ?? null );
			$payload = array();
			

			// Loop through the plans received from settings
			foreach ( $plans as $plan ) {

				$plan_ids[] = $plan['id'];

				if ( 'pack' === ( $plan['sales_model'] ?? null ) ) {
					$payload[] = array(
						'plan'          => $plan,
						'regular_price' => $plan['default_price'],
						'sale_price'    => $plan['default_sale_price'] ?? $plan['default_price'],
						'enable'        => true
					);
				}
			}

			// Create content pack product if there's at least one plans for it.
			if ( ! empty( $payload ) ) {
				ContentPack::createContentPack( $content_type, $payload );
			}

		}
	
		// Disable variations that doesn't exist
		if ( ! empty( $plan_ids ) ) {

			$ids_not_in = "'" . implode( "','", $plan_ids ) . "'";
			$var_name   = Contents::LICENSING_VARIATION;

			$wpdb->query(
				"UPDATE {$wpdb->posts} p
				INNER JOIN {$wpdb->postmeta} m ON p.ID = m.post_id
				SET p.post_status = 'private'
				WHERE m.meta_key = 'attribute_{$var_name}' 
				AND m.meta_value NOT IN ({$ids_not_in})"
			);
		}
	}

	/**
	 * Alter product name in cart page to support custom plan name
	 *
	 * @param string $product_title
	 * @param array $cart_item
	 * @param string $cart_item_key
	 * 
	 * @return string
	 */
	public function overrideProductTitle( $product_title, $cart_item, $cart_item_key ) {		
		return Contents::getProductVariationTitle(
			$cart_item['product_id'], 
			$cart_item['variation_id'], 
			$product_title 
		);
	}

	/**
	 * Alter the variation key to plan in order page ideally
	 *
	 * @param string $key The default key
	 * 
	 * @return string
	 */
	public function alterPlanKey( string $key ) {
		if ( $key === Contents::LICENSING_VARIATION ) {
			return esc_html__( 'Plan', 'portfoliya' );
		}
		return $key;
	}

	/**
	 * Alter plan name in order edit page in backend dashboard
	 *
	 * @param string $display_value
	 * @param object $meta
	 * @return string
	 */
	public function alterPlanName( $display_value, $meta ) {
		if ( $meta->key === Contents::LICENSING_VARIATION ) {
			$display_value = Contents::getPlanNameByKey( $meta->value );
		}
		return $display_value;
	}

	/**
	 * Product+variation title
	 *
	 * @param string $title
	 * @param object $subscription
	 * 
	 * @return string
	 */
	public function productGetName( $title, $item ) {

		if ( is_object( $item ) && method_exists( $item, 'get_meta_data' ) ) {

			$meta     = $item->get_meta_data();
			$meta     = is_array( $meta ) ? $meta : array();
			$plan_key = null;

			// Find the portfoliya meta 
			foreach ( $meta as $m ) {
				if ( is_object( $m ) && Contents::LICENSING_VARIATION === $m->key || strpos( $m->key, Contents::LICENSING_VARIATION_PREFIX ) === 0 ) {
					$plan_key = $m->value;
					break;
				}
			}

			if ( ! empty( $plan_key ) ) {
				$plan_name = Contents::getPlanNameByKey( $plan_key, null );

				if ( ! empty( $plan_name ) ) {
					$title = str_replace( $plan_key, $plan_name, $title );
				}
			}
		}
		
		return $title;
	}

	/**
	 * Alter plan key and name in item meta
	 *
	 * @param array $item_data
	 * @param array $cart_item
	 * @return array
	 */
	public function itemMeta( $item_data, $cart_item ) {
		if ( is_array( $item_data ) ) {
			foreach ( $item_data as $index => $data ) {
				if ( isset( $data['key'], $data['value'] ) && Contents::LICENSING_VARIATION === $data['key'] ) {
					$item_data[ $index ]['display'] = Contents::getPlanNameByKey( $data['value'], $data['value'] );
				}
			}
		}
		return $item_data;
	}

	/**
	 * Undocumented function
	 *
	 * @param [type] $item_id
	 * @param [type] $item
	 * @param [type] $product
	 * @return void
	 */
	public function orderItemInfo( $item_id, $item, $product ) {

		$order = wc_get_order( $item->get_order_id() );

		if ( ! empty( $order ) && is_object( $order ) ) {
			echo '<div 
				data-variation_id="' . $item->get_variation_id() . '"
				data-order_id="' . $item->get_order_id() . '"
				class="Portfoliya_item_info_in_order_page"></div>';
		}
	}

	/**
	 * Add portfoliya post state to product
	 *
	 * @return void
	 */
	public function addPortfoliyaPostState( $post_states, $post ) {

		if ( $post->post_type === 'product' && Contents::isProductContent( $post->ID ) ) {
			$post_states['portfoliya_content_product'] = '<a>
				<img src="' . PortfoliyaMain::$configs->dist_url . 'libraries/menu-icon.svg" style="width: 13px; height: 13px"/>
			</a>';
		}
		
		return $post_states;
	}

	/**
	 * Mark order as completed
	 *
	 * @param int $order_id
	 * @return void
	 */
	public function autoCompleteOrders( $order_id ) {

		if ( ! AdminSetting::get( 'auto_complete_woocommerce_orders' ) ) {
			return;
		}

		$order = wc_get_order( $order_id );
		if ( $order->get_status() == 'completed' ) {
			return;
		}

		$order->update_status( 'completed' );
	}

	/**
	 * Delete Portfoliya sales on delete order
	 *
	 * @param int $order_id
	 * @return void
	 */
	public function deleteSaleOnOrderDelete( $order_id ) {

		// Delete order if the post type is shop order
		if ( get_post_type( $order_id ) === 'shop_order' ) {
			Sale::disableSalesByField( 'order_id', $order_id );
		}
	}

	/**
	 * Show dashboard link in order details page
	 *
	 * @param object $order
	 * @return void
	 */
	public function orderDetailsPageContents( $order ) {
		include Main::$configs->dir . 'templates/order-details-page.php';
	}
}
