<?php
/**
 * Script registrars
 *
 * @package portfoliya
 */

namespace Portfoliya\Setup;

use Portfoliya\Helpers\Utilities;
use Portfoliya\Main;
use Portfoliya\Models\AdminSetting;
use Portfoliya\Models\Contents;
use Portfoliya\Models\User;
use Portfoliya_Pro\Setup\Dashboard as FEDashboard;
use SolidieLib\Variables;

/**
 * Script class
 */
class Scripts {

	private $cache_version;

	/**
	 * Scripts constructor, register script hooks
	 *
	 * @return void
	 */
	public function __construct() {

		$this->cache_version = 'development' === Main::$configs->mode ? time() : Main::$configs->version;

		// Load scripts
		add_action( 'admin_enqueue_scripts', array( $this, 'adminScripts' ), 11 );
		add_action( 'wp_enqueue_scripts', array( $this, 'frontendScripts' ), 11 );

		// Register script translations
		add_action( 'admin_enqueue_scripts', array( $this, 'scriptTranslation' ), 9 );
		add_action( 'wp_enqueue_scripts', array( $this, 'scriptTranslation' ), 9 );

		// Load text domain
		add_action( 'init', array( $this, 'loadTextDomain' ) );

		// JS Variables
		add_action( 'wp_enqueue_scripts', array( $this, 'loadVariables' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'loadVariables' ) );
	}

	/**
	 * Load environment and color variables
	 *
	 * @return void
	 */
	public function loadVariables() {

		// Prepare configs, add color schem
		$configs = Main::$configs;
		$configs->color_scheme = array(
			'color_scheme_materials' => AdminSetting::get( 'color_scheme_materials' ),
			'color_scheme_texts'     => AdminSetting::get( 'color_scheme_texts' ),
		);

		// Get common variables
		$data = ( new Variables( $configs ) )->get();

		// Add additional data to the common data array
		$data = array_replace_recursive(
			$data,
			array(
				'readonly_mode'    => apply_filters( 'portfoliya_readonly_mode', false ), // It's for portfoliya demo site only. No other use is expected.
				'has_admin_bar'    => is_admin_bar_showing(),
				'user'             => array(
					'has_administrative'    => User::hasAdministrativeRole( get_current_user_id() ), 
				),
				'settings'         => array(
					'contents' => AdminSetting::getContentSettings(),
					'general'  => array(),
				),
				'configs'          => array(),
				'permalinks'       => array(
					'inventory_backend' => Utilities::getBackendPermalink( AdminPage::INVENTORY_SLUG ),
					'settings'          => Utilities::getBackendPermalink( AdminPage::SETTINGS_SLUG ),
					'dashboard'         => Utilities::getBackendPermalink( Main::$configs->root_menu_slug ),
					'gallery'           => ( object ) Contents::getGalleryPermalink(),
					'gallery_root'      => Contents::getGalleryPermalink( false ),
				),
			)
		);
		
		// Add file configs for content editor
		if ( is_user_logged_in() && ( is_admin() || ( class_exists( FEDashboard::class ) && FEDashboard::isFrontDashboard() ) ) ) {
			$data['configs'] = array_replace(
				$data['configs'],
				array(
					'max_file_size'  => Utilities::convertToBytes( ini_get( 'upload_max_filesize' ) ),
					'max_post_size' => Utilities::convertToBytes( ini_get( 'post_max_size' ) ),
				)
			);
		}

		// Pass the data through filter
		$data = apply_filters( 'portfoliya_frontend_variables', $data );

		// Register as localize data
		wp_localize_script( 'portfoliya-translations', Main::$configs->app_id, $data );
	}

	/**
	 * Load scripts for admin dashboard
	 *
	 * @return void
	 */
	public function adminScripts() {
		if ( Utilities::isAdminDashboard() ) {
			self::loadTinyMCE();
			wp_enqueue_script( 'portfoliya-backend', Main::$configs->dist_url . 'admin-dashboard.js', array( 'jquery' ), $this->cache_version, true );
		}
	}

	/**
	 * Load resource for TinyMCE editor
	 *
	 * @return void
	 */
	public static function loadTinyMCE() {
		wp_enqueue_style( 'portfoliya-backend-tiny-style', Main::$configs->dist_url . 'libraries/tinymce/css/style.css', array(), Main::$configs->version );
		wp_enqueue_script( 'portfoliya-backend-tiny', Main::$configs->dist_url . 'libraries/tinymce/js/tinymce/tinymce.min.js', array( 'jquery' ), Main::$configs->version );
	}

	/**
	 * Load scripts for frontend view
	 *
	 * @return void
	 */
	public function frontendScripts() {

		if ( is_front_page() || ( ! is_singular() && ! is_single() )) {
			return;
		}

		if ( ! empty( $GLOBALS['portfoliya_gallery_data'] ) ) {
			wp_enqueue_style( 'portfoliya-tiny-styles-css', Main::$configs->dist_url . 'libraries/prism/prism.css', array(), $this->cache_version );
			wp_enqueue_script( 'portfoliya-tiny-styles-js', Main::$configs->dist_url . 'libraries/prism/prism.js', array(), $this->cache_version, true );
		}
		wp_enqueue_script( 'portfoliya-frontend', Main::$configs->dist_url . 'frontend.js', array( 'jquery' ), $this->cache_version, true );
	}

	/**
	 * Load text domain for translations
	 *
	 * @return void
	 */
	public function loadTextDomain() {
		load_plugin_textdomain( Main::$configs->text_domain, false, Main::$configs->dir . 'languages' );
	}

	/**
	 * Load translations
	 *
	 * @return void
	 */
	public function scriptTranslation() {

		$domain = Main::$configs->text_domain;
		$dir    = Main::$configs->dir . 'languages/';

		wp_enqueue_script( 'portfoliya-translations', Main::$configs->dist_url . 'libraries/translation-loader.js', array( 'jquery' ), $this->cache_version, true );
		wp_set_script_translations( 'portfoliya-translations', $domain, $dir );
	}
}
