import React, { useEffect, useState, useContext } from "react";

import { WpDashboardFullPage } from "solidie-materials/backend-dashboard-container/full-page-container.jsx";
import { DropDown } from "solidie-materials/dropdown/dropdown.jsx";
import { __, currency_symbol, data_pointer } from "solidie-materials/helpers.jsx";
import {confirm} from 'solidie-materials/prompts.jsx';
import { request } from "solidie-materials/request.jsx";
import { TableStat } from "solidie-materials/table-stat.jsx";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon.jsx";
import { ContextToast } from "solidie-materials/toast/toast.jsx";
import { Pagination } from "solidie-materials/pagination/pagination.jsx";
import { TextField } from "solidie-materials/text-field/text-field.jsx";
import { DropDownStatus } from "solidie-materials/dropdown-status/dropdown-status.jsx";
import { AdminTitleHead } from "solidie-materials/backend-dashboard-container/title-head";

const statuses = {
	approved: __('Approved'),
	pending: __('Pending'),
	rejected: __('Rejected'),
	banned: __('Revoked')
}

const drop_options = Object.keys(statuses).map(status=>{
	return {
		id: status,
		label: statuses[status]
	}
});

const {is_admin, contribution: {contributor_approval_requires}={}} = window[data_pointer];

export function ContributorList() {

	const {ajaxToast} = useContext(ContextToast);

	const [state, setState] = useState({
		fetching: true,
		segmentation: null,
		changing_status_for: null,
		contributors: [],
	});

	const [filterState, setFilterState] = useState({
		page: 1,
		search: '',
		status: ''
	});

	const setFilter=(name, value)=>{
		setFilterState({
			...filterState,
			page: 1,
			[name]: value
		});
	}

	const fetchContributors=()=>{

		setState({
			...state,
			fetching: true
		});

		request('fetchContributorList', filterState, resp=>{
			const {
				success,
				data: {
					contributors = state.contributors,
					segmentation=state.segmentation
				}
			} = resp;

			setState({
				...state,
				fetching: false,
				contributors,
				segmentation
			});
		})
	}

	const changeContributorStatus=(user_id, status)=>{
		
		confirm(
			__('Sure to change status?'),
			()=>{
				setState({
					...state,
					changing_status_for: user_id
				});

				request('updateContributorStatus', {user_id, status, is_admin: is_admin ? true : false}, resp=>{
					const {
						success,
						data: {
							contributor_status
						}
					} = resp;

					if ( ! success ) {
						ajaxToast(resp);
					}

					setState({
						...state,
						changing_status_for: null,
						contributors: state.contributors.map(c=>(success && c.user_id==user_id) ? {...c, status: contributor_status} : c)
					});
				});
			}
		);
	}

	useEffect(()=>{
		fetchContributors();
	}, [filterState]);

	return <WpDashboardFullPage>
		<div className={'padding-20 height-p-100'.classNames()}>
			<AdminTitleHead title={__('Contributors')}>
				<div className={'d-flex align-items-center column-gap-15'.classNames()}>
					<div>
						<DropDown
							options={drop_options}
							value={filterState.status}
							onChange={v=>setFilter('status', v)}
							placeholder={__('Status')}
						/>
					</div>
					<div>
						<TextField
							placeholder={__('Search..')}
							type='search'
							onChange={v=>setFilter('search', v)}
							value={filterState.search}
						/>
					</div>
				</div>
			</AdminTitleHead>
			
			<table className={'table '.classNames()}>
				<thead>
					<tr>
						<th>{__('Name')}</th>
						<th>{__('Contents')}</th>
						<th>{__('Earned')}</th>
						<th>{__('Withdrawn')}</th>
						<th title={__('Awaiting due to defined delay')}>{__('Awaiting')}</th>
						<th>{__('Withdrawable')}</th>
						{
							!contributor_approval_requires ? null : <th>{__('Status')}</th>
						}
					</tr>
				</thead>
				<tbody>
					{
						state.contributors.map(contributor=>{
							const {
								user_id, 
								avatar_url,
								display_name, 
								user_email, 
								contribution_count=0,
								status,
								earning_stats:{
									earned = 0,
									awaiting = 0,
									available = 0,
									withdrawan = 0
								}
							} = contributor;

							return <tr key={user_id}>
								<td data-th={__('Name')}>
									<div className={'d-flex align-items-center column-gap-10'.classNames()}>
										<div>
											<img src={avatar_url} style={{width: '32px', height: '32px', borderRadius: '50%'}}/>
										</div>
										<div>
											<strong className={'d-block font-size-16 font-weight-500 color-text-80'.classNames()}>
												{display_name}
											</strong>
											<span className={'color-text-50 font-size-13'.classNames()}>
												{user_email}
											</span>
										</div>
									</div>
								</td>
								<td data-th={__('Contents')}>
									<span className={'color-text-70'.classNames()}>
										{contribution_count}
									 </span>
								</td>
								<td data-th={__('Earned')}>
									{currency_symbol}{earned}
								</td>
								<td data-th={__('Withdrawn')}>
									{currency_symbol}{withdrawan}
								</td>
								<td data-th={__('Awaiting')}>
									{currency_symbol}{awaiting}
								</td>
								<td data-th={__('Withdrawable')}>
									{currency_symbol}{available}
								</td>
								{
									!contributor_approval_requires ? null :
									<td data-th={__('Status')}>
										<div className={'d-flex align-items-center column-gap-5'.classNames()}>
											<div data-cylector="status-dropdown">
												<DropDownStatus
													placeholder={__('Select Status')}
													onChange={v=>changeContributorStatus(user_id, v)}
													value={status || ''}
													disabled={state.changing_status_for}
													options={drop_options}
												/>
											</div>
											<div>
												<LoadingIcon show={state.changing_status_for==user_id}/>
											</div>
										</div>
									</td>
								}
							</tr>
						})
					}
					<TableStat
						loading={state.fetching}
						empty={!state.fetching && !state.contributors.length}
						message={__('No result')}
					/>
				</tbody>
			</table>

			<div className={'margin-top-15 d-flex justify-content-center'.classNames()}>
				<div>
					<Pagination
						onChange={page=>setFilter('page', page)}
						pageNumber={filterState.page}
						pageCount={state.segmentation?.page_count}
					/>
				</div>
			</div>
		</div>
	</WpDashboardFullPage>
}
