import React, { useContext, useEffect, useState } from "react";

import {request} from 'solidie-materials/request.jsx';
import {TextField} from 'solidie-materials/text-field/text-field.jsx';
import {DateField} from 'solidie-materials/date-time.jsx';
import {LoadingIcon} from 'solidie-materials/loading-icon/loading-icon.jsx';
import { __, formatDate, isEmpty, data_pointer } from "solidie-materials/helpers.jsx";
import { ToggleSwitch } from "solidie-materials/toggle-switch/ToggleSwitch.jsx";
import { ContextToast } from "solidie-materials/toast/toast.jsx";
import { Modal } from "solidie-materials/modal.jsx";
import { order_statuses } from "solidie-materials/data.jsx";

import { LicenseKeyManager } from "../../frontend-dashboard/pages/purchases/license-keys";

const {readonly_mode} = window[data_pointer];

export function OrderItemInfo({order_id, variation_id}) {

	const {ajaxToast} = useContext(ContextToast);

	const [state, setState] = useState({
		editor: false,
		updating: false,
		sale: {},
		show_licenses: false,
		sale_raw: {} // Unchanged
	});

	const getSale=()=>{
		request('getOrderItemInfo', {order_id, variation_id}, resp=>{
			const {
				success,
				data: {
					sale
				}
			} = resp;

			setState({
				...state,
				sale,
				sale_raw: sale
			});
		});
	}

	const onChange=(name, value)=>{
		setState({
			...state,
			sale : {
				...state.sale,
				[name]: value
			}
		})
	}

	const update=()=>{
		
		setState({
			...state,
			updating: true
		});

		const {
			license_or_content_limit,
			expires_on,
			enabled
		} = state.sale;

		const item_info = {
			license_or_content_limit: license_or_content_limit===null ? 'null' : license_or_content_limit,
			expires_on: expires_on===null ? 'null' : expires_on,
			enabled
		}

		request('updateItemInfo', {item_info, sale_id: state.sale.sale_id}, resp=>{
			const {
				success,
				data: {
					sale
				}
			} = resp;

			setState({
				...state,
				updating: false,
				editor: !success,
				sale: success ? sale : state.sale,
				sale_raw: success ? sale : state.sale_raw
			})

			if ( !success ) {
				ajaxToast(resp);
			}
		});
	}

	const closeModal=()=>{
		setState({
			...state, 
			show_licenses: false
		});
	}
	
	useEffect(()=>{
		getSale();
	}, []);

	const {expires_on, content_type, license_or_content_limit, enabled} = state.sale || {};
	const unlimited = license_or_content_limit===null;
	const lifetime  = expires_on===null;

	return isEmpty( state.sale ) ? null : 
	<div>
		{
			!state.show_licenses ? null :
			<Modal closeOnDocumentClick={true} onClose={closeModal}>
				<div
					className={'d-flex align-items-center margin-bottom-20'.classNames()}
				>
					<div className={'flex-1'.classNames()}>
						<span className={'font-size-20 font-weight-500 color-text'.classNames()}>
							{__('App Purchase Information')}
						</span>
					</div>
					<div>
						<i
							className={'sicon sicon-times font-size-18 color-text-50 cursor-pointer'.classNames()}
							onClick={closeModal}
						></i>
					</div>
				</div>
				<LicenseKeyManager 
					sale_id={state.sale.sale_id}
					can_back={false}
				/>
			</Modal>
		}
		<strong className={'color-material-90'.classNames()}>
			{__('Content Access:')}
		</strong>
		<table className={'table-basic'.classNames()}>
			<tbody>
				<tr>
					<th>{__('Enabled')}</th>
					<td>
						{
							!state.editor ?
								(enabled ? __('Yes') : __('No')) :
								<div className={'d-flex align-items-center column-gap-10'.classNames()}>
									<ToggleSwitch
										checked={enabled ? true : false}
										onChange={v=>onChange('enabled', v)}
										disabled={state.updating}
									/>
									<span>
										{__('Yes')}
									</span>
								</div>
						}
					</td>
				</tr>
				<tr>
					<th>{__('Expires')}</th>
					<td>
						{
							!state.editor ?
								(expires_on ? formatDate(expires_on) : __('Never')) :
								<div>
									<div className={'d-flex align-items-center column-gap-10'.classNames()}>
										<ToggleSwitch
											checked={lifetime}
											onChange={v=>onChange('expires_on', (v ? null : state.sale_raw.expires_on || formatDate(new Date(), 'Y-m-d')))}
											disabled={state.updating}
										/> 
										<span>{__('Lifetime')}</span>
									</div>

									{
										lifetime ? null :
										<div className={'margin-top-10'.classNames()}>
											<DateField
												value={state.sale.expires_on}
												onChange={v=>onChange('expires_on', v)}
												style={{height: '26px'}}
												disabled={state.updating}
											/>
										</div>
									}
								</div>
						}
					</td>
				</tr>
				{
					content_type=='tutorial' ? null :
					<tr>
						<th>{content_type=='app' ? __('License Key Limit') : __('Download Limit')}</th>
						<td>
							{
								!state.editor ? 
									(license_or_content_limit || __('No Limit')):
									<div>
										<div className={'d-flex align-items-center column-gap-10'.classNames()}>
											<ToggleSwitch
												checked={unlimited}
												onChange={v=>onChange('license_or_content_limit', (v ? null : state.sale_raw.license_or_content_limit || 0))}
												disabled={state.updating}
											/> 
											<span>{__('Unlimited')}</span>
										</div>

										{
											unlimited ? null :
											<div className={'margin-top-10'.classNames()}>
												<TextField
													value={state.sale.license_or_content_limit}
													onChange={v=>onChange('license_or_content_limit', (v || '').replace(/\D/g, ''))}
													style={{height: '26px'}}
													disabled={state.updating}
												/>
											</div>
										}
									</div>
							}
						</td>
					</tr>
				}

				{
					!state.sale_raw.parent_order ? null :
					<tr>
						<th>
							{__('Main Order')}
						</th>
						<td>
							<a 
								href={state.sale_raw.parent_order.url}
								target="_blank"
							>
								#{state.sale_raw.parent_order.id}
							</a>
						</td>
					</tr>
				}

				{
					!state.sale_raw.renewals?.length ? null :
					<tr>
						<th>
							{__('Renewals')}
						</th>
						<td>
							{
								state.sale_raw.renewals.map(renewal=>{
									return <div className={'d-block'.classNames()}>
										<a
											key={renewal.sale_id}
											href={renewal.order_url}
											target="_blank"
										>
											#{renewal.order_id}
										</a>
										&nbsp;
										{
											renewal.order_status=='completed' ? null : 
											<i>
												<small>
													({order_statuses[renewal.order_status] || renewal.order_status})
												</small>
											</i>
										}
									</div> 
								})
							}
						</td>
					</tr>
				}
			</tbody>
		</table>
		<div className={'text-align-right margin-top-5'.classNames()}>
			{
				!state.editor ? <div className={'d-flex align-items-center column-gap-8'.classNames()}>
					{
						content_type !== 'app' ? null :
						<>
							<span 
								className={'hover-underline cursor-pointer'.classNames()} 
								onClick={()=>setState({...state, show_licenses: true})}
							>
								{__('License Keys')}
							</span>
							<span>
								|
							</span>
						</>
					}
					<span 
						className={'hover-underline cursor-pointer'.classNames()}
						onClick={()=>setState({
							...state, 
							editor: true
						})}
					>
						{__('Edit')}
					</span> 
				</div> 
				:
				<>
					<button 
						className={'button button-primary button-outlined button-small'.classNames()}
						disabled={state.updating}
						onClick={()=>setState({
							...state, 
							editor: false, 
							sale: state.sale_raw
						})}
					>
						{__('Cancel')}
					</button>
					&nbsp;&nbsp;
					<button 
						className={'button button-primary button-small'.classNames()}
						disabled={readonly_mode || state.updating}
						onClick={update}
					>
						{__('Update')} <LoadingIcon show={state.updating}/>
					</button>
				</>
			}
		</div>
	</div>
}
