import React, { useEffect, useState, useContext } from "react";

import { __, currency_symbol, data_pointer } from "solidie-materials/helpers.jsx";
import { RadioCheckbox } from "solidie-materials/radio-checkbox.jsx";
import { ToggleSwitch } from "solidie-materials/toggle-switch/ToggleSwitch.jsx";
import { NumberField } from "solidie-materials/number-field/number-field.jsx";
import { request } from "solidie-materials/request.jsx";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon.jsx";
import { ContextToast } from "solidie-materials/toast/toast.jsx";

import { AddPlan } from "./add-plan.jsx";
import { subscription_models } from "./plan-settings.jsx";

import * as style from './settings.module.scss';

const label_class = 'color-text-80 font-size-14'.classNames();

const {
	is_admin, 
	is_wc_active,
	permalinks={},
	settings={},
	user: {
		has_administrative = false
	},
	contribution:{
		free: free_contribution,
		paid: paid_contribution
	}={}
} = window[data_pointer];


function PlanSingle({product_id, variation_id, plan={}, expanded_id, toggleEditor, editPlan, content_type}) {

	const {
		id,
		plan_name,
		description,
		regular_price,
		sale_price,
		default_price,
		default_sale_price,
		subscription_model,
		license_or_content_limit,
		enable = plan.auto_enable
	} = plan;

	const _regular_price = regular_price ?? default_price;
	const _sale_price    = sale_price === undefined ? default_sale_price : ((!sale_price || parseFloat(sale_price)>=parseFloat(regular_price)) ? '' : sale_price);

	const active = expanded_id==id;

	return <table 
		className={`table ${active ? 'active' : ''}`.classNames(style)}
		style={{borderRadius: '6px', overflow: 'hidden'}}
		title={`${product_id ? `Product: ${product_id}` : ''}${variation_id ? ` Variation: ${variation_id}` : ''}`}
	>
		<tbody>
			<tr>
				<th colSpan={2}>
					<div 
						className={'d-flex align-items-center padding-vertical-10 padding-horizontal-15 column-gap-15 cursor-pointer'.classNames()}
						onClick={()=>toggleEditor(id)}
					>
						<ToggleSwitch
							checked={enable ? true : false}
							onChange={checked=>editPlan('enable', checked)}
						/>
						<div className={'flex-1'.classNames()}>
							<span 
								className={`${active ? 'font-weight-600' : 'font-weight-400'}`.classNames()}
								title={description}
							>
								{plan_name}
							</span>
						</div>
						<div>
							<strong className={'font-weight-600'.classNames()}>
								{currency_symbol}{_sale_price > 0 ? _sale_price : _regular_price}
							</strong>
							{
								!(_sale_price>0) ? null :
								<sub>
									&nbsp;
									<s>
										{currency_symbol}{_regular_price}
									</s>
								</sub>
							}
						</div>
						<div className={'d-flex align-items-center column-gap-15'.classNames()}>
							<i 
								className={`sicon sicon-arrow-down cursor-pointer`.classNames()} 
								style={active ? {transform: 'rotateX(180deg)'} : {}}
							></i>
						</div>
					</div>
				</th>
			</tr> 
			{
				!active ? null : 
				<>
					<tr>
						<td colSpan={2}>
							<div className={'border-top-1 b-color-material-20'.classNames()}></div>
						</td>
					</tr>

					<tr>
						<td>
							<div className={'d-inline-block'.classNames()}>
								<div>
									<i className={'d-block margin-bottom-15 color-text-60'.classNames()}>
										{__('Note: Changing price here will not modify global plan price. These are for only this content.')}
									</i>
								</div>
								<div 
									style={{display: 'grid', gridTemplateColumns: 'auto auto'}} 
									className={'column-gap-30 row-gap-20 align-items-center'.classNames()}
								>
									<div>
										<span className={label_class}>
											{__('Regular Price')} ({currency_symbol})
										</span>
									</div>
									<div>
										<NumberField
											min={0}
											value={_regular_price}
											decimal_point={true}
											onChange={v=>editPlan('regular_price', v)}
											width='130px'
										/>
									</div>

									<div>
										<span className={label_class}>
											{__('Sale Price')} ({currency_symbol})
										</span>
									</div>
									<div>
										<NumberField
											min={0}
											max={Math.max(0.00, parseFloat(_regular_price)-0.01)}
											value={_sale_price}
											decimal_point={true}
											onChange={v=>editPlan('sale_price', v)}
											width='130px'
										/>
									</div>

									{
										content_type!=='app' ? null :
										<>
											<div>
												<span className={label_class}>
													{__('Subscription')}
												</span>
											</div>
											<div className={'d-flex align-items-center column-gap-8'.classNames()}>
												{subscription_models[subscription_model] || subscription_model}
											</div>

											<div>
												<span className={label_class}>
													{__('License')}
												</span>
											</div>
											<div className={'d-flex align-items-center column-gap-8'.classNames()}>
												{license_or_content_limit}
											</div>
										</>
									}

									
									{
										!description ? null :
										<>
											<div>
												<span className={label_class}>
													{__('Description')}
												</span>
											</div>
											<div>
												{description}
											</div>
										</>
									}

									{
										(!product_id || !is_admin) ? null :
										<>
											<div>
												<a 
													href={`${permalinks.home_url}/wp-admin/post.php?post=${product_id}&action=edit`} 
													target="_blank"
												>
													<span className={'hover-underline cursor-pointer'.classNames()}>
														{__('Open Linked Product')}
													</span>
												</a>
											</div>
										</>
									}
								</div>
							</div>
						</td>
					</tr>
				</>
			}
		</tbody>
	</table>
}


export function PlanFieldContentEditor({onChange, content_type, content_type_label, content={}, field_wrapper_class}) {

	if ( !settings?.contents?.[content_type]?.purchasable ) {
		return null;
	}
	
	const {ajaxToast} = useContext(ContextToast);
	const { product={} } = content;
	const monet_value = 'paid' !== product?.monetization ? 'free' : 'paid';
	const is_paid = (!free_contribution && paid_contribution) || monet_value === 'paid';

	const monetizations = [
		{
			id: 'free',
			label: __('Free'),
		},
		{
			id: 'paid',
			label: <>{__('Paid')}{!is_wc_active ? <small className={'color-error'.classNames()}><i> - {__('Monetization Requires WooCommerce')}</i></small> : null}</>,
			disabled: !is_wc_active
		}
	];

	const [state, setState] = useState({
		fetching: true,
		content_type_plans: [],
		contributor_plan_control: false,
		extended_plan_id: null,
	});

	const setVal=(name, value)=>{
		onChange(
			'product', 
			{
				...product, 
				[name]: value
			}
		);
	}

	const updatePlanPrice=(id, name, value, _plan, merge_me={})=>{

		const {plans=[]} = product;
		const plan = plans.find(plan=>plan.plan.id==id) || {plan: _plan};
		plan[name] = value;
		
		onChange(
			'product',
			{
				...product,
				plans: [
					...plans.filter(plan=>plan.plan.id!=id),
					{...plan, ...merge_me}
				]
			}
		);
	}

	const fetchPlans=()=>{

		request('getContentPlansData', {content_type}, resp=>{
			
			const {
				success,
				data: {
					content_type_plans,
					contributor_plan_control
				}
			} = resp;

			if ( ! success ) {
				ajaxToast(resp);
				return;
			}
			
			setState({
				...state,
				fetching: false,
				content_type_plans,
				contributor_plan_control
			});
		});
	}

	const togglePlanEditor=expanded_id=>{
		setState({
			...state,
			extended_plan_id: state.extended_plan_id==expanded_id ? null : expanded_id
		});
	}

	useEffect(()=>{
		fetchPlans();
	}, []);

	const setup_link   = `${permalinks.settings}#/settings/contents/${content_type}/pricing/`;
	const bundle_plans = state.content_type_plans.filter(p=>p.sales_model==='pack');
	const single_plans = state.content_type_plans.filter(p=>p.sales_model==='single');

	return <>

		{
			(!is_admin && (!free_contribution || !paid_contribution)) ? null :
			<div className={field_wrapper_class}>
				<div>
					<strong className={'d-flex align-items-center column-gap-5 font-weight-600 margin-bottom-5 color-text-80'.classNames()}>
						{__('Monetization')}
					</strong>
				</div>
				<div className={'d-flex column-gap-15'.classNames()}>
					<RadioCheckbox type="radio"
						name="monetization"
						value={monet_value}
						options={monetizations}
						onChange={value=>setVal('monetization', value)}/>
				</div>
			</div>
		}
		
		{
			(!is_paid || (!is_admin && !state.contributor_plan_control)) ? null : 
			<div className={field_wrapper_class}>
				<div>
					<strong className={'d-flex align-items-center column-gap-10 font-weight-600 margin-bottom-5 color-text-80'.classNames()}>
						<span>
							{__('Pricing Plan')}
						</span>
						{
							!has_administrative ? null :
							<a 
								href={setup_link}
								target="_blank"
								className={'sicon sicon-settings-gear color-material-80 interactive'.classNames()}
							></a>
						}
					</strong>
				</div>
				<div>
					{
						!state.content_type_plans.length ? <div>
							{
								state.fetching ? 
									<div className={'d-flex align-items-center column-gap-8'.classNames()}>
										<LoadingIcon show={state.fetching}/> <span>{__('Fetching pricing..')}</span>
									</div> 
									:
									(
										has_administrative ? null :
										<>
											<span className={'d-block'.classNames()}>
												{__('No monetization plan is set up yet.')}
											</span>
											<small>
												{__('Please contact site admin')}
											</small>
										</>
									)
									
							}
						</div> 
						:
						<div className={'d-flex flex-direction-column row-gap-15 margin-bottom-15'.classNames()}>
							{
								single_plans.map(plan=>{
											
									const { id } = plan;

									const {
										regular_price, 
										sale_price, 
										enable = plan.auto_enable ?? false,
										product_id,
										variation_id
									} = (product.plans || []).find(plan=>plan.plan?.id==id) || {};

									return <div key={plan.id}>
										<PlanSingle 
											product_id={product_id}
											variation_id={variation_id}
											plan={{...plan, regular_price, sale_price, enable}}
											content_type={content_type}
											expanded_id={state.extended_plan_id}
											toggleEditor={togglePlanEditor}
											editPlan={(name, value)=>{
												updatePlanPrice(plan.id, name, value, plan, {enable: name=='enable' ? value : enable});
											}}
										/>
									</div>
								})
							}
						</div>
					}

					{
						!has_administrative ? null : 
						<AddPlan 
							content_type={content_type}
							sales_model='single'
							onAdd={fetchPlans}
						/>
					}
				</div>
			</div>
		}

		{
			(!is_paid || (!is_admin && !state.contributor_plan_control)) ? null :
			<>
				{
					!bundle_plans.length ? null :
					<div className={field_wrapper_class}>
						<div>
							<strong className={'d-flex align-items-center column-gap-10 font-weight-600 margin-bottom-5'.classNames()}>
								<span>{__('Supported Bundles')}</span>
								{
									!has_administrative ? null :
									<a 
										href={setup_link}
										target="_blank"
										className={'sicon sicon-settings-gear color-material-80 interactive'.classNames()}
									></a>
								}
							</strong>
						</div>
						<div>
							<div className={'d-flex flex-direction-column row-gap-15'.classNames()}>
								{
									bundle_plans.map(plan=>{

										const {
											id, 
											plan_name, 
											auto_enable=true,
											description,
											subscription_model,
											license_or_content_limit,
										} = plan;

										const {
											enable=auto_enable
										} = (product.plans || []).find(plan=>plan.plan?.id==id) || {};

										const _model = subscription_models[subscription_model] || subscription_model;

										return <div key={id}>
											<label className={'d-inline-flex align-items-flex-start column-gap-10 cursor-pointer'.classNames()}>
												<div>
													<input
														type="checkbox"
														checked={enable ? true : false}
														onChange={e=>updatePlanPrice(id, 'enable', e.currentTarget.checked, plan)}
													/>
												</div>
												<div className={'flex-1'.classNames()}>
													<span className={'d-block margin-bottom-5 font-size-14 font-weight-500 color-text-80'.classNames()} title={description}>
														{plan_name}
													</span>
													<span className={'d-block font-size-13 color-text-60 font-weight-400'.classNames()}>
														{content_type==='tutorial' ? _model : `${_model} ${license_or_content_limit} ${content_type=='app' ? __('Licenses') : __('Downloads')}`}
													</span>
													{
														!description ? null :
														<span className={'d-block font-size-12 color-text-50 font-weight-400 margin-top-5'.classNames()}>
															{description}
														</span>
													}
												</div>
											</label>
										</div>
									})
								}

								{
									!has_administrative ? null : 
									<AddPlan 
										content_type={content_type}
										sales_model='pack'
										onAdd={fetchPlans}
									/>
								}
							</div>
						</div>
					</div>
				}

				{
					!state.content_type_plans.length ? null :
					<div className={field_wrapper_class}>
						<div>
							<strong className={'d-flex align-items-center column-gap-5 font-weight-600 margin-bottom-5 color-text-80'.classNames()}>
								{__('Optional Purchase')}
							</strong>
							<small className={'d-block margin-bottom-5 font-size-13 color-text-60'.classNames()}>
								{content_type === 'tutorial' ? __('Allow access without purchase') : __('Allow downloading without purchase')}
							</small>
						</div>
						<div>
							<div className={'d-flex column-gap-15'.classNames()}>
								<div>
									<ToggleSwitch
										checked={product.is_purchase_optional ? true : false}
										onChange={checked=>setVal('is_purchase_optional', checked)} />
								</div>
								<div>
									{__('Enable')}
								</div>
							</div>
						</div>
					</div>
				}
			</>
		}
	</> 
}
