import React, { useState } from "react";

import {__, currency_symbol, data_pointer, getRandomString, isEmpty, sprintf} from 'solidie-materials/helpers.jsx';
import {TextField} from 'solidie-materials/text-field/text-field.jsx';
import {NumberField} from 'solidie-materials/number-field/number-field.jsx';
import {RadioCheckbox} from 'solidie-materials/radio-checkbox.jsx';
import {confirm} from 'solidie-materials/prompts.jsx';
import {SortableList} from 'solidie-materials/sortable-list.jsx';
import {ToggleSwitch} from 'solidie-materials/toggle-switch/ToggleSwitch.jsx';

import * as style from './settings.module.scss';

const {readonly_mode} = window[data_pointer];

export const subscription_models = {
	month: __('Monthly'),
	year: __('Yearly'),
	lifetime: __('Lifetime'),
}

export function getNewPlan() {
	return {
		id: getRandomString(),
		plan_name: 'Untitled Plan',
		description: '',
		sales_model: 'single',
		subscription_model: 'lifetime',
		default_price: 0,
		auto_enable: true,
		license_or_content_limit: 5
	}
}

export function ItemSingle({plan, expanded_id, standalone=false, toggleEditor, content_type, deletePlan, editPlan}) {

	const {
		id,
		plan_name,
		description,
		sales_model,
		subscription_model,
		default_price,
		default_sale_price,
		auto_enable = true,
		license_or_content_limit
	} = plan;

	const active       = standalone || expanded_id==id;
	const c_type_label = window[data_pointer].settings?.contents[content_type]?.label || __('Content');

	const _regular_price = default_price;
	const _sale_price    = (!default_sale_price || parseFloat(default_sale_price)>=parseFloat(_regular_price)) ? '' : default_sale_price;

	const sales_models = [
		{
			id: 'single',
			label: sprintf(__('Single %s'), c_type_label),
		},
		{
			id: 'pack',
			label: sprintf(__('%s Bundle'), c_type_label),
		}
	];

	return <table 
		className={`table ${active ? 'active' : ''} ${standalone ? 'standalone' : ''}`.classNames(style)}
		style={{borderRadius: '6px', overflow: 'hidden'}}
	>
		<tbody>
			{
				standalone ? null :
				<tr>
					<th colSpan={2}>
						<div 
							className={'d-flex align-items-center padding-vertical-10 padding-horizontal-15 column-gap-15 cursor-pointer'.classNames()}
							onClick={()=>toggleEditor(id)}
						>
							<div>
								<i
									className={'sicon sicon-drag'.classNames()}
									style={{cursor: 'grab'}}
								></i>
							</div>
							<div className={'flex-1 d-flex align-items-center column-gap-10'.classNames()}>
								<span className={`${active ? 'font-weight-600' : 'font-weight-400'}`.classNames()}>
									{plan_name}
								</span>
								{
									sales_model !== 'pack' ? null :
									<i 
										title={`${c_type_label} Bundle`}
										className={'sicon sicon-folder color-text-40'.classNames()}
										style={{fontSize: '10px'}}
									></i>
								}
							</div>
							<div className={'d-flex align-items-center column-gap-15'.classNames()}>
								<div>
									{currency_symbol}{_sale_price>0 ? _sale_price : _regular_price}
									{
										!(_sale_price>0) ? null :
										<sub>
											&nbsp;
											<s>
												{currency_symbol}{_regular_price}
											</s>
										</sub>
									}
								</div>
								
								<i 
									className={'sicon sicon-trash cursor-pointer color-error'.classNames()} 
									onClick={e=>{
										e.stopPropagation(); 
										if (!readonly_mode) {
											deletePlan(id);
										}
									}}></i>

								<i 
									className={`sicon sicon-arrow-down cursor-pointer`.classNames()} 
									style={active ? {transform: 'rotateX(180deg)'} : {}}
								></i>
							</div>
						</div>
					</th>
				</tr> 
			}
			
			{
				!active ? null : 
				<>
					{
						standalone ? null :
						<tr>
							<td colSpan={2}>
								<div className={'border-top-1 b-color-material-10'.classNames()}></div>
							</td>
						</tr>
					}
					<tr>
						<th>{__('Plan Name')}</th>
						<td data-cylector="plan-name">
							<TextField 
								value={plan_name} 
								onChange={value=>editPlan('plan_name', value)}/>
						</td>
					</tr>
					<tr>
						<th>{__('Description')}</th>
						<td data-cylector="description">
							<TextField 
								type="textarea" 
								value={description} 
								onChange={value=>editPlan('description', value)}/>
						</td>
					</tr>

					{
						content_type==='app' ? null : <tr>
							<th>{__('Sales Model')}</th>
							<td data-cylector="sales-model">
								<form className={'d-flex align-items-center column-gap-15'.classNames()}>
									<RadioCheckbox
										type="radio"
										name="sales_model"
										value={sales_model}
										options={sales_models}
										onChange={value=>editPlan('sales_model', value)}/>
								</form>
							</td>
						</tr>
					}

					<tr>
						<th>
							{sales_model=='pack' ? sprintf(__('Auto include all %s'), c_type_label) : sprintf(__('Auto enable for new %s'), c_type_label)}
						</th>
						<td data-cylector="auto-enable">
							<div className={'d-flex align-items-center column-gap-10'.classNames()}>
								<ToggleSwitch
									checked={auto_enable}
									onChange={checked=>editPlan('auto_enable', checked)}
								/>

								<span>
									{__('Enable')}
								</span>
							</div>
						</td>
					</tr>
					
					{
						!(content_type==='app' || sales_model==='pack') ? null : <>
							<tr>
								<th>{__('Subscription Model')}</th>
								<td data-cylector="subscription-model">
									<form className={'d-flex align-items-center column-gap-15 row-gap-10 flex-wrap-wrap'.classNames()}>
										<RadioCheckbox
											type="radio"
											name="subscription_model"
											value={subscription_model}
											options={Object.keys(subscription_models).map(k=>{return {id: k, label: subscription_models[k]}})}
											onChange={value=>editPlan('subscription_model', value)}/>
									</form>
								</td>
							</tr>
							{
								content_type === 'tutorial' ? null :
								<tr>
									<th>
										{content_type=='app' ? __('License Key Limit') : __('Download Limit')}
									</th>
									<td data-cylector="access-limit">
										<NumberField 
											min={0}
											value={license_or_content_limit} 
											onChange={value=>editPlan('license_or_content_limit', value)}
											width='130px'
										/>
									</td>
								</tr>
							}
						</>
					}
					
					<tr>
						<th>
							<span>
								{sales_model==='pack' ? __('Regular Price') : __('Default Regular Price')}
							</span>
						</th>
						
						<td data-cylector="default-price">
							<div className={'d-flex align-items-center column-gap-8'.classNames()}>
								<NumberField 
									min={0}
									value={_regular_price} 
									decimal_point={true}
									onChange={value=>editPlan('default_price', value)}
									width='130px'
								/>
								<span>
									{currency_symbol}
								</span>
							</div>
						</td>
					</tr>
					
					<tr>
						<th>
							<span>
								{sales_model==='pack' ? __('Sale Price') : __('Default Sale Price')}
							</span>
						</th>
						
						<td data-cylector="default-sale-price">
							<div className={'d-flex align-items-center column-gap-8'.classNames()}>
								<NumberField 
									min={0}
									max={Math.max(0.00, parseFloat(_regular_price)-0.01)}
									value={_sale_price} 
									decimal_point={true}
									onChange={value=>editPlan('default_sale_price', value)}
									width='130px'
								/>
								<span>
									{currency_symbol}
								</span>
							</div>
						</td>
					</tr>
				</>
			}
		</tbody>
	</table>
}

export function PlanSettings({content_type, content, onChange}) {

	if ( !window[data_pointer].settings?.contents?.[content_type]?.purchasable ) {
		return null;
	}
	
	const [state, setState] = useState({
		content: {...content},
		expanded_id: null
	});

	const toggleEditor=expanded_id=>{
		setState({
			...state,
			expanded_id: state.expanded_id==expanded_id ? null : expanded_id
		});
	}

	const addPlan=()=>{
		
		const {plans=[]} = state.content;
		const new_plan = getNewPlan();

		dispatchPlans([
			...plans,
			new_plan
		], new_plan.id);
	}

	const editPlan=(id, key, value)=>{
		const {plans=[]} = state.content;
		const index = plans.findIndex(p=>p.id===id);
		if ( index > -1 ) {
			plans[index][key] = value;
		}
		dispatchPlans(plans);
	}

	const deletePlan=(id)=>{
		confirm(
			__('Remove plan?'),
			__('It will be removed from all the contents. Click Save Changes to apply the deletion.'),
			()=>{
				const {plans=[]} = state.content;
				dispatchPlans(plans.filter(p=>p.id!==id));
			}
		);
	}

	const dispatchPlans=(plans, expanded_id)=>{
		setState({
			...state,
			expanded_id: expanded_id || state.expanded_id,
			content: {
				...state.content,
				plans
			}
		});
		onChange('plans', plans);
	}

	return <div>
		<div className={'margin-bottom-10'.classNames()}>
			<span className={'d-block font-size-15 font-weight-500 line-height-24 letter-spacing--16 color-text'.classNames()}>
				{__('Monetization Plans')}
			</span>
			{
				isEmpty(state.content.plans) ? null :
				<span className={'d-block font-size-13 font-weight-400 line-height-24 letter-spacing--16 color-text-70'.classNames()}>
					Note: {__('Modifying single sale plans does\'t affect existing contents.')}&nbsp;
					{content_type !== 'app' ? __('However bundle plan changes will be applied from the next purchase.') : null}
				</span>
			}
		</div>
		<div>
			<SortableList
                className={'row-gap-15'.classNames()}
                onReorder={(plans) => dispatchPlans(plans)}
                items={( state.content.plans || [] ).map(plan=> {
                    return {
                        ...plan,
                        id: plan.id, // Just to make sure it requires ID.
                        rendered: (
                            <ItemSingle
								plan={plan}
								expanded_id={state.expanded_id}
								{...{
									toggleEditor, 
									content_type, 
									deletePlan, 
									editPlan: (key, value)=>editPlan(plan.id, key, value)
								}}
                            />
                        )
                    };
                })}
            />

			<span 
				data-cylector="add-plan"
				className={'d-block margin-top-15 cursor-pointer hover-underline'.classNames()} 
				onClick={addPlan}
			>
				{isEmpty(state.content.plans) ? __('+ Add Plan') : __('+ Add More Plan')}
			</span>
		</div>
	</div>
}
