import React, { useEffect, useState, useContext, useRef } from "react";
import {Link} from 'react-router-dom';

import { __, currency_symbol, data_pointer, formatDate, isEmpty, timeAgoOrAfter } from "solidie-materials/helpers.jsx";
import { request } from "solidie-materials/request.jsx";
import { ContextToast } from "solidie-materials/toast/toast.jsx";
import { TableStat } from "solidie-materials/table-stat.jsx";
import { order_statuses } from "solidie-materials/data.jsx";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon";

const top_cards = {
	gross_revenue: {
		icon: 'fa fa-chart-line',
		label: __('Gross Revenue')
	},
	contributor_commission: {
		icon: 'fa fa-chart-bar',
		label: __('Contributor Commission')
	},
	total_refunds: {
		icon: 'fa fa-chart-area',
		label: __('Total Refunds')
	},
	net_revenue: {
		icon: 'fa fa-chart-pie',
		label: __('Net Revenue')
	}
}

const card_label_class = 'color-text-70'.classNames();
const show_all_class = 'color-material-80 interactive'.classNames();

export function ReportsOverview() {

	const {ajaxToast} = useContext(ContextToast);

	const reff_canv1 = useRef();
	const reff_canv2 = useRef();

	const [state, setState] = useState({
		mounted: false,
		fetching: true,
		overview: {},
		sales_trend: {},
		download_trend: {},
		recent_sales: [],
		recent_contents: [],
		recent_contributors: [],
		recent_withdrawals: [],
		inventory_link: null,
		contributors_link: null,
		withdrawals_link: null
	});

	const getPortfoliyaReports=()=>{

		setState({
			...state,
			fetching: true
		});

		request('getPortfoliyaReports', {}, resp=>{

			const {
				success,
				data:{
					overview={},
					sales_trend={},
					download_trend={},
					recent_sales=[],
					recent_contents=[],
					recent_contributors=[],
					recent_withdrawals=[],
					inventory_link,
					contributors_link,
					withdrawals_link
				}
			} = resp;

			setState({
				...state,
				fetching: false,
				mounted: true,
				overview,
				sales_trend,
				download_trend,
				recent_sales,
				recent_contents,
				recent_contributors,
				recent_withdrawals,
				inventory_link,
				contributors_link,
				withdrawals_link
			});

			if ( ! success ) {
				ajaxToast(resp);
			}
		});
	}

	useEffect(()=>{
		getPortfoliyaReports();
	}, []);

	useEffect(()=>{

		
		$ = window.jQuery;
		

		// Render sales trend
		if ( ! state.mounted ) {
			return;
		}

		const {colors}= window[data_pointer];

		new Chart(
			reff_canv1.current.getContext("2d"), 
			{
				type: "line",
				data: {
					labels: state.sales_trend?.labels || [],
					datasets: [{
						label: `${__('Sales')} ${currency_symbol}`,
						fill: true,
						backgroundColor: colors['material-30'],
						data: state.sales_trend?.data || [],
						lineTension: 0.3
					}]
				},
				options: {
					responsive: true,
					plugins: {
						legend: {
							display: false // This hides the legend
						}
					},
				}
			}
		);

		// Render contents and download trends
		new Chart(
			reff_canv2.current.getContext("2d"), 
			{
				type: "line",
				data: {
					labels: state.download_trend?.labels || [],
					datasets: [{
						label: __('Content Downloads'),
						data: state.download_trend?.data || [],
						fill: true,
						backgroundColor: colors['material-30'],
						lineTension: 0.3
					}]
				},
				options: {
					responsive: true,
					plugins: {
						legend: {
							display: false // This hides the legend
						}
					}
				}
			}
		);
	}, [state.mounted]);

	return <div className="content open">

		<div className="container-fluid pt-4 px-0">
			<div className="row g-4">
				{
					Object.keys(top_cards).map(key=>{
						
						const {icon, label} = top_cards[key];

						return <div key={key} className="col-sm-6 col-xl-3">
							<div className={"rounded d-flex align-items-center justify-content-between p-4 " + 'bg-color-white box-shadow-thin'.classNames()}>
								<i className={`${icon} fa-3x ${'color-material-90'.classNames()}`}></i>
								<div className="ms-3">
									<p className={`mb-2 ${'color-text-80'.classNames()}`}>
										{label}
									</p>
									<h6 className={`mb-0 ${'color-text'.classNames()}`}>
										{currency_symbol}{state.overview[key] || 0}
									</h6>
								</div>
							</div>
						</div>
					})
				}
			</div>
		</div>
		
		<div className="container-fluid pt-4 px-0">
			<div className="row g-4">
				<div className="col-sm-12 col-xl-6">
					<div className={"text-center rounded p-4 " + 'bg-color-white box-shadow-thin'.classNames()}>
						<div className="d-flex align-items-center justify-content-between mb-4">
							<h6 className={"mb-0 " + card_label_class}>{__('Sales')}</h6>
						</div>
						<canvas ref={reff_canv1}></canvas>
					</div>
				</div>
				<div className="col-sm-12 col-xl-6">
					<div className={"text-center rounded p-4 " + 'bg-color-white box-shadow-thin'.classNames()}>
						<div className="d-flex align-items-center justify-content-between mb-4">
							<h6 className={"mb-0 " + card_label_class}>{__('Content Downloads')}</h6>
						</div>
						<canvas ref={reff_canv2}></canvas>
					</div>
				</div>
			</div>
		</div>
		
		<div className="container-fluid pt-4 px-0">
			<div className={"text-center rounded p-4 " + 'bg-color-white box-shadow-thin'.classNames()}>
				<div className="d-flex align-items-center justify-content-between mb-4">
					<h6 className={"mb-0 " + card_label_class}>{__('Recent Sales')}</h6>
					<Link to='/reports/sales/' className={show_all_class}>{__('Show All')}</Link>
				</div>
				<div className="table-responsive">
					<table className="table text-start align-middle table-bordered mb-0">
						<thead>
							<tr className={'color-text-90'.classNames()}>
								<th scope="col">{__('Item')}</th>
								<th scope="col">{__('Sale Price')}</th>
								<th scope="col">{__('Order Status')}</th>
								<th scope="col">{__('Date')}</th>
							</tr>
						</thead>
						<tbody>
							{
								state.recent_sales.map(sale=>{
									return <tr key={sale.sale_id} className={'color-text-80'.classNames()}>
										<td>{sale.content_title}</td>
										<td>{currency_symbol}{sale.sale_price}</td>
										<td>{order_statuses[sale.order_status] || sale.order_status}</td>
										<td>{formatDate(sale.purchase_date)}</td>
									</tr>
								})
							}

							<TableStat 
								empty={isEmpty(state.recent_sales)}
								loading={state.fetching}
								message={__('No Sales Found')}
							/>
						</tbody>
					</table>
				</div>
			</div>
		</div>
		
		<div className="container-fluid pt-4 px-0">
			<div className="row g-4">
				<div className="col-sm-12 col-md-6 col-xl-4">
					<div className={"h-100 rounded p-4 " + 'bg-color-white box-shadow-thin'.classNames()}>
						<div className="d-flex align-items-center justify-content-between mb-2">
							<h6 className={"mb-0 " + card_label_class} title={__('Content recently published')}>
								{__('Recent Contents')}
							</h6>
							<a href={state.inventory_link} className={show_all_class}>
								{__('Show All')}
							</a>
						</div>
						{
							isEmpty(state.recent_contents) ? <div className={'text-align-center'.classNames()} style={{marginTop: '100px'}}>
								{
									state.fetching ? 
										<LoadingIcon show={true} center={true}/> :
										<span>
											{__('No contents yet')}
										</span>
								}
							</div>
							:
							state.recent_contents.map(content=>{
								const {
									content_title,
									content_id,
									created_at,
									media={}
								} = content;

								return <div key={content_id} className="d-flex align-items-center border-bottom py-3">
									<img 
										className="flex-shrink-0" 
										src={media?.thumbnail?.file_url} 
										style={{width: '50px', height: '30px', objectFit: 'cover', borderRadius: '5px'}}
									/>
									<div className="w-100 ms-3">
										<div className="d-flex w-100 justify-content-between">
											<h6 className={"mb-0 " + 'color-text-80'.classNames()}>{content_title}</h6>
											<small className={'white-space-nowrap'.classNames()}>{timeAgoOrAfter(created_at)}</small>
										</div>
									</div>
								</div>
							})
						}
					</div>
				</div>

				<div className="col-sm-12 col-md-6 col-xl-4">
					<div className={"h-100 rounded p-4 " + 'bg-color-white box-shadow-thin'.classNames()}>
						<div className="d-flex align-items-center justify-content-between mb-4">
							<h6 className={"mb-0 " + card_label_class}>{__('Recent Contributors')}</h6>
							<a href={state.contributors_link} className={show_all_class}>
								{__('Show All')}
							</a>
						</div>
						{
							isEmpty(state.recent_contributors) ? <div className={'text-align-center'.classNames()} style={{marginTop: '100px'}}>
								{
									state.fetching ? 
										<LoadingIcon show={true} center={true}/> :
										<span>
											{__('No contributor yet')}
										</span>
								}
							</div>
							:
							state.recent_contributors.map(contributor=>{
								const {
									display_name,
									user_id,
									reg_unix,
									avatar_url,
									user_email
								} = contributor;

								return <div key={user_id} className="d-flex align-items-center border-bottom py-3">
									<img 
										className="rounded-circle flex-shrink-0" 
										src={avatar_url} 
										style={{width: '40px', height: '40px', objectFit: 'cover'}}
									/>
									<div className="w-100 ms-3">
										<div className="d-flex w-100 justify-content-between">
											<h6 className={"mb-0 " + 'color-text-80'.classNames()}>{display_name}</h6>
											<small className={'white-space-nowrap'.classNames()}>{timeAgoOrAfter(reg_unix)}</small>
										</div>
										<span className={'color-text-50'.classNames()}>{user_email}</span>
									</div>
								</div>
							})
						}
					</div>
				</div>

				<div className="col-sm-12 col-md-6 col-xl-4">
					<div className={"h-100 rounded p-4 " + 'bg-color-white box-shadow-thin'.classNames()}>
						<div className="d-flex align-items-center justify-content-between mb-4">
							<h6 className={"mb-0 " + card_label_class}>{__('Recent Payouts')}</h6>
							<a href={state.withdrawals_link} className={show_all_class}>{__('Show All')}</a>
						</div>
						{
							isEmpty(state.recent_withdrawals) ? <div className={'text-align-center'.classNames()} style={{marginTop: '100px'}}>
								{
									state.fetching ? 
										<LoadingIcon show={true} center={true}/> :
										<span>
											{__('No disbursement made yet')}
										</span>
								}
							</div>
							:
							state.recent_withdrawals.map(withdrawal=>{
								const {
									amount,
									display_name,
									withdrawal_id,
									sent_date,
									avatar_url,
									user_email
								} = withdrawal;

								return <div key={withdrawal_id} className="d-flex align-items-center border-bottom py-3">
									<img 
										className="rounded-circle flex-shrink-0" 
										src={avatar_url} 
										style={{width: '40px', height: '40px', objectFit: 'cover'}}
									/>
									<div className="w-100 ms-3">
										<div className="d-flex w-100 justify-content-between">
											<h6 className={"mb-0 " + 'color-text-80'.classNames()}>{display_name}</h6>
											<span className={'white-space-nowrap'.classNames()}>
												{currency_symbol}{amount}
											</span>
										</div>
										<small className={'color-text-50'.classNames()}>{timeAgoOrAfter(sent_date)}</small>
									</div>
								</div>
							})
						}
					</div>
				</div>
				
			</div>
		</div>
	</div>
}
