import React, { useContext, useEffect, useState } from "react";
import {createRoot} from "react-dom/client";
import {Helmet} from "react-helmet";
import { BrowserRouter, Navigate, Route, Routes } from 'react-router-dom';

import { MountPoint } from "solidie-materials/mountpoint.jsx";
import { getElementDataSet, __, getDashboardPath, data_pointer } from "solidie-materials/helpers.jsx";
import { CoverImage } from "solidie-materials/image/image.jsx";

import {Sidebar} from "./navigation/sidebar/Sidebar.jsx";
import { PurchasedContents } from "./pages/purchases/my-purchases.jsx";
import { MyOrders } from "./pages/orders/my-orders.jsx";
import { has_purchasable, SettingsPage } from "./pages/settings/settings.jsx";
import { Earnings } from "./pages/earnings/earnings.jsx";
import { ContributionApplication } from "./pages/contribution-application/contribution-application.jsx";
import { Wishlist } from "./pages/wishlist/wishlist.jsx";
import { ChatListWrapper } from "./pages/chatroom/list/chat-list.jsx";
import { ContextSocket, SocketWrapper } from "./socket-wrapper.jsx";

import * as layout from './style.module.scss';
import * as sidebar_style from './navigation/sidebar/sidebar.module.scss';
import { Inventory } from "../../../components/views/modules/inventory/contents/index.jsx";
import { ContentEditor } from "../../../components/views/modules/inventory/editor/ContentEditor.jsx";

/* Dashboard Layout Adopted from https://codepen.io/trooperandz/pen/EOgJvg */

const {
	has_admin_bar,
	colors:{
		material
	},
	contribution:{
		can_contribute, 
		paid, 
		enabled: contribution_enabled
	},
	permalinks: {
		home_url
	},
	settings: {
		general:{
			contributor_messaging_enable=false
		}
	}
} = window[data_pointer];

function D(props) {

	const {stats: socket_stats} = useContext(ContextSocket);

	const [state, setState] = useState({
		active_slug: null,
		expanded_menu: null,
		sidebar_open: false,
		is_mobile_view: window.innerWidth <= 767,
		sidebar_open: false
	});

	const $ = window.jQuery;
	let {
		avatar_url, 
		first_name, 
		last_name, 
		display_name,
		email
	} = window[data_pointer].user || {};

	const setTemplate=()=>{
		/* Scripts for css grid dashboard */

		setSidenavListeners();

		function toggleClass(el, className) {
			if (el.hasClass(className)) {
				el.removeClass(className);
			} else {
				el.addClass(className);
			}
		}
		
		// Sidenav list sliding functionality
		function setSidenavListeners() {
			const subHeadings = $('.'+'navList__subheading'.classNames(layout));
			const SUBHEADING_OPEN_CLASS = 'navList__subheading--open'.classNames(layout);
			const SUBLIST_HIDDEN_CLASS = 'subList--hidden'.classNames(layout);

			subHeadings.each((i, subHeadingEl) => {
				$(subHeadingEl).on('click', (e) => {
					const subListEl = $(subHeadingEl).siblings();

					// Add/remove selected styles to list category heading
					if (subHeadingEl) {
						toggleClass($(subHeadingEl), SUBHEADING_OPEN_CLASS);
					}

					// Reveal/hide the sublist
					if (subListEl && subListEl.length === 1) {
						toggleClass($(subListEl), SUBLIST_HIDDEN_CLASS);
					}
				});
			});
		}
	}

	const toggleSidebar=(open)=>{
		setState({
			...state,
			sidebar_open: open
		});
	}

	const sidebarListeners=()=>{
		setState({
			...state,
			sidebar_open: window.location.href.endsWith('#navigation')
		});
	}

	useEffect(()=>{
		setTemplate();
		window.addEventListener('popstate', sidebarListeners);
		window.addEventListener('resize', ()=>{
			setState({
				...state,
				sidebar_open: false,
				is_mobile_view: window.innerWidth <= 767
			})
		})
	}, []);

	let should_open = !state.is_mobile_view || state.sidebar_open;
	let sidebar_class = ('sidenav' + (should_open ? ' sidenav--active' : '')).classNames(layout);
	let grid_class = `grid ${should_open ? 'grid--noscroll' : ''} ${has_admin_bar ? 'admin-bar' : ''}`.classNames(layout);

	return <>
		<Helmet>
			<title>
				{props.page_title}
			</title>
		</Helmet>
		<div className={grid_class} data-cylector="fe-wrapper">
			<header className={"header".classNames(layout)}>
				<div className={"d-flex align-items-center".classNames()}>
					<div className={"width-p-100 d-flex padding-horizontal-15".classNames()}>
						<span className={'d-flex align-items-center column-gap-10'.classNames()}>
							{
								! state.is_mobile_view ? null :
								<>
									{
										!socket_stats.any ? null :
										<span className={'red-dot'.classNames(sidebar_style)}></span>
									}
									<i 
										className={`sicon sicon-textalign-left cursor-pointer`.classNames()} 
										onClick={()=>toggleSidebar(true)}
									></i>
								</>
							}
							<span className={'font-size-16 font-weight-600'.classNames()}>
								{props.page_title}
							</span>
						</span>
					</div>
				</div>
				<div className={'padding-horizontal-15'.classNames()}>
					<a href={home_url} target="_blank" className={'d-flex'.classNames()}>
						<svg xmlns="http://www.w3.org/2000/svg"  viewBox="0 0 24 24" width="17px" height="17px"><path fill={material} d="M 12 2.0996094 L 1 12 L 4 12 L 4 21 L 11 21 L 11 15 L 13 15 L 13 21 L 20 21 L 20 12 L 23 12 L 12 2.0996094 z M 12 4.7910156 L 18 10.191406 L 18 11 L 18 19 L 15 19 L 15 13 L 9 13 L 9 19 L 6 19 L 6 10.191406 L 12 4.7910156 z"/></svg>
					</a>
				</div>
			</header>

			<aside className={sidebar_class}>
				<div 
					className={'d-flex align-items-center column-gap-10 padding-15'.classNames()}
					style={{borderBottom: '1px solid rgba(255, 255, 255, .3)'}}
				>
					<div>
						<CoverImage 
							width={50} 
							circle={true} 
							src={avatar_url}
							name={first_name + ' ' + last_name}/>
					</div>
					<div className={'flex-1'.classNames()}>
						<strong className={'d-block color-white font-weight-700 font-size-17'.classNames()}>
							{display_name}
						</strong>
						<small 
							className={'d-block font-weight-400 font-size-13'.classNames()}
							style={{color: '#DDDDDD'}}
						>
							{email}
						</small>
					</div>
				</div>
				<div className={"row-appstore".classNames(layout)}>
					<Sidebar navs={props.navs} toggleSidebar={toggleSidebar}/>
				</div>
			</aside>

			<main className={"main".classNames(layout) + 'overflow-auto'.classNames()} style={{background: 'rgb(245, 245, 245)'}}>
				{props.children}
			</main>

			{
				(!should_open || !state.is_mobile_view) ? null :
				<div 
					className={"sidebar-underlay".classNames(layout)} 
					onClick={()=>toggleSidebar(false)}
				></div>
			}
		</div>
	</> 
}

function Dashboard(props) {

	const menus_for_purchasable = !has_purchasable ? [] : [
		{
			path: 'my-purchases/',
			full_path: 'my-purchases/:content_type?/:sale_id?/',
			text: __('Purchased Contents'),
			icon: '',
			component: PurchasedContents
		},
		{
			path: 'my-orders/',
			full_path: 'my-orders/:order_id?/',
			text: __('My Orders'),
			icon: '',
			component: MyOrders
		},
		(
			(!can_contribute || !paid) ? null :
			{
				path: 'earnings/',
				full_path: 'earnings/:active_tab?/',
				text: __('Earnings'),
				icon: '',
				component: Earnings
			}
		),
	];

	const contr_pages = ! window[data_pointer].contribution?.enabled ? [] : [
		{
			path: 'inventory/',
			full_path: 'inventory/:content_type?/',
			text: __('Inventory'),
			icon: '',
			component: Inventory
		},
		{
			show_menu: false,
			path: 'inventory-edit/',
			full_path: 'inventory/:content_type/editor/:content_id/:segment?/:segment_id?/',
			text: __('Content Editor'),
			icon: '',
			component: ContentEditor
		}
	]

	// my purchases, orders, my account, logout for all. Then Inventory, sales, report, earnings, settings for contributor
	const navs = [
		...contr_pages,
		...menus_for_purchasable,
		(
			(!contribution_enabled || can_contribute) ? null :
			{
				path: 'contribution/',
				full_path: 'contribution/',
				text: __('Inventory'),
				icon: '',
				component: ContributionApplication
			}
		),
		{
			path: 'wishlist/',
			full_path: 'wishlist/',
			text: __('Saved Items'),
			icon: '',
			component: Wishlist
		},
		(
			!contributor_messaging_enable ? null : {
				socket_stat: 'message',
				path: 'messages/',
				full_path: 'messages/:peer_id?/',
				text: __('Messages'),
				icon: '',
				component: ChatListWrapper
			}
		),
		{
			path: 'settings/',
			full_path: 'settings/:active_tab?/',
			text: __('Settings'),
			icon: '',
			component: SettingsPage
		},
	].filter(n=>n && (n.path.indexOf('inventory')===-1 || can_contribute));

  	return <SocketWrapper>
		<BrowserRouter>
			<Routes>
				{
					navs.map((nav)=>{
						const {component: Comp} = nav;
						return <Route
							key={nav.path} 
							path={getDashboardPath(nav.full_path || nav.path)}
							element={
								<D {...props} navs={navs} page_title={nav.text}>
									<Comp {...props} comp_path={nav.path}/>
								</D>
							}/>
					})
				}
				<Route path={'*'} element={<Navigate to={getDashboardPath(navs[0].path)} replace />} />
			</Routes>
		</BrowserRouter>
	</SocketWrapper>
}

// Render entire user dashboard
let dashboard = document.getElementById("Portfoliya_Dashboard");
if (dashboard) {
	createRoot(dashboard).render(
		<MountPoint element={dashboard}>
			<Dashboard {...getElementDataSet(dashboard)} />
		</MountPoint>
	);
}
