import React, { useContext, useEffect, useState } from "react";
import { useNavigate, useParams } from "react-router-dom";
import { Helmet } from "react-helmet";

import { __, data_pointer, getDashboardPath, timeAgoOrAfter } from "solidie-materials/helpers";
import { request } from "solidie-materials/request";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon";

import { ContextSocket, socket_connect_event, using_socket } from "../../../socket-wrapper";
import { ChatThread, socket_event_message } from "../thread/thread";

import no_msg_img from '../../../../../images/no-msg.svg';
import * as style from './list.module.scss';

const {
	user: {
		id: user_id
	},
} = window[data_pointer];

export function InitScreen({fetching, error_message, children}) {
	return <div className={'height-p-100 d-flex align-items-center justify-content-center text-align-center padding-horizontal-15'.classNames()}>
		<div>
			{
				fetching ? <LoadingIcon show={true}/> : 
				<div>
					{children || error_message}
				</div>
			}
		</div>
	</div>
}

export function ChatListWrapper() {

	const navigate = useNavigate();
	const {registerSocketCallback, removeSocketCallback, refreshStats} = useContext(ContextSocket);
	const {peer_id} = useParams();

	const [state ,setState] = useState({
		fetching: true,
		error_message: null
	});

	const [threads, setThreads] = useState([]);

	const fetchThreads=()=>{
		setState({
			...state,
			fetching: true
		});

		request('fetchThreads', {include_thread: peer_id}, resp=>{
			const {success, data: {threads=[], message=__('Something went wrong!')}} = resp;
			setState({
				...state,
				fetching: false,
				error_message: success ? null : message
			});

			setThreads(threads);
		})
	}

	const refreshMessages=()=>{
		fetchThreads();
		refreshStats();
	}

	useEffect(()=>{
		const events = [
			socket_connect_event,
			socket_event_message
		];

		if ( ! using_socket ) {
			fetchThreads();
		} else {
			refreshStats();
			registerSocketCallback(events, 'thread_list_handler', fetchThreads);
			return ()=>removeSocketCallback(events, 'thread_list_handler');
		}

	}, [] );

	const peer_selected = peer_id && peer_id != user_id;
	
	return <>
		<Helmet>
			<title>{__('Messages')}</title>
		</Helmet>
		<div 
			className={'width-p-100 height-p-100 d-flex border-radius-10 bg-color-white overflow-hidden'.classNames()}
			style={{minWidth: '600px', margin: '0 auto'}}
		>
			<div className={'height-p-100'.classNames()} style={{width: '250px', borderRight: '1px solid var(--solidie-color-text-4)'}}>
				{
					!threads.length ? 
					<InitScreen 
						fetching={state.fetching}
					>
						<div className={'d-flex justify-content-center margin-bottom-25'.classNames()}>
							<img src={no_msg_img} style={{width: '240px', height: 'auto'}}/>
						</div>
					</InitScreen> 
					:
					<div className={'padding-15 height-p-100 overflow-auto'.classNames()}>
						{
							threads.map(chat=>{

								const {
									avatar_url, 
									display_name, 
									sender_id,
									is_unread,
									peer_id,
									sent_time: timestamp, 
									message_content
								} = chat;

								return <div 
									key={peer_id} 
									className={'d-flex align-items-center column-gap-15 margin-bottom-15 cursor-pointer'.classNames() + `single ${is_unread ? 'unread' : ''}`.classNames(style)}
									onClick={()=>navigate( getDashboardPath(`/messages/${peer_id}/`))}
								>
									<div className={'avatar'.classNames(style) + 'd-flex flex-shrink-0'.classNames()}>
										<img src={avatar_url}/>
									</div>
									<div className={'flex-1 overflow-hidden'.classNames()}>
										<div>
											<span className={'font-size-16 font-weight-500'.classNames()}>
												{display_name}
											</span>
										</div>
										<div className={'d-flex align-items-center column-gap-8'.classNames()}>
											<div className={'flex-1 font-size-14 font-weight-400 white-space-nowrap overflow-hidden text-overflow-ellipsis'.classNames()}>
												{sender_id==peer_id ? null: 'You:'} {message_content}
											</div>
											<div className={'font-size-13'.classNames()}>
												{timeAgoOrAfter(timestamp)}
											</div>
										</div>
									</div>
								</div>
							})
						}
					</div>
				}
			</div>
			
			<div className={`flex-1 height-p-100 ${!peer_selected ? 'd-flex align-items-center justify-content-center' : ''}`.classNames()}>
				{
					peer_selected ? <ChatThread refreshMessages={refreshMessages}/> : <div>Select a Message Thread</div>
				}
			</div>
		</div>
	</> 
}
