import React, { useContext, useEffect, useRef, useState } from "react";
import { Link, useNavigate, useParams } from "react-router-dom";

import { __, data_pointer, getRandomString, isEmpty, timeAgoOrAfter } from "solidie-materials/helpers";
import { request } from "solidie-materials/request";
import { ContextToast } from "solidie-materials/toast/toast";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon";

import { ContextSocket, socket_connect_event, using_socket } from "../../../socket-wrapper";
import * as style from './thread.module.scss';

export const socket_event_message = 'update_messages_real_time';

export function ChatThread({refreshMessages}) {

	const {user: {id: self_id}, colors} = window[data_pointer];
	const {peer_id} = useParams();

	const navigate = useNavigate();
	const {ajaxToast} = useContext(ContextToast);
	const {registerSocketCallback, removeSocketCallback, emitSocket, connected: socket_connected} = useContext(ContextSocket);
	const scrollable_ref = useRef();
	const textarea_reff = useRef();

	const [state, setState] = useState({
		peer: {},
		error_message: null,
		new_message: '',
		sending: false,
		scroll_flag: null,
		no_more_older: false,
	});

	const [fetching, setFetching] = useState(true);
	const [messages_state, setMessageList] = useState([]);
	const [show_time_for, setShowTimeFor] = useState(null);

	const mergeMessages=(new_messages=[])=>{
		
		const messages    = [...messages_state, ...new_messages];
		const message_ids = [...new Set(messages.map(m=>parseInt(m.message_id)))];
		message_ids.sort((a, b) => a - b);
		
		return message_ids.map(id=>messages.find(m=>m.message_id==id)).filter(m=>!isEmpty(m));
	}

	const fetchOnScroll=(e)=>{

		const {scrollTop} = e.currentTarget || {};
		
		if ( !state.no_more_older && !fetching && scrollTop === 0 ) {			
			fetchMessages(messages_state[0]?.message_id);
		}
	}

	const fetchMessages=(older_than)=>{

		setFetching(true);
		
		const payload = {
			self_id,
			older_than, 
			recipient_id: peer_id, 
			later_than: messages_state[messages_state.length-1]?.message_id
		};

		request('fetchMessages', payload, resp=>{

			const {
				success, 
				data:{
					peer={}, 
					messages=[],
					message=__('Could not fetch messages!')
				}
			} = resp;

			const has_unseen = messages.filter(m=>m.sender_id==peer_id && !m.read_time).length ? true : false;

			// Merge messages with the state

			setState({
				...state,
				peer,
				error_message: success ? null : message,
				no_more_older: older_than && !messages.length,
				scroll_flag: (has_unseen || !messages_state.length) ? getRandomString() : state.scroll_flag
			});

			setFetching(false);
			setMessageList(mergeMessages(messages));
		});
	}

	const sendMessage=()=>{

		if ( state.sending || isEmpty( state.new_message ) ) {
			return;
		}

		setState({
			...state,
			sending: true
		});

		const payload = {
			recipient_id: peer_id, 
			message: state.new_message.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;').replace(/'/g, '&#039;'),
			later_than: messages_state[messages_state.length-1]?.message_id
		}

		request('sendMessage', payload, resp=>{
			
			const {success, data:{messages=[]}} = resp;

			setState({
				...state,
				sending: false,
				new_message: success ? '' : state.new_message
			});
			
			if ( ! resp.success ) {
				ajaxToast(resp);
			} else {

				// Store the updated message list in runtime
				setMessageList(mergeMessages(messages));

				refreshMessages();
				
				// Notify the recipient to get updates
				emitSocket(peer_id, 'update_messages_real_time');
			}
		});
	}
	
	useEffect(()=>{
		scrollable_ref?.current?.scrollTo?.(0, scrollable_ref?.current?.scrollHeight);
	}, [state.scroll_flag]);

	useEffect(()=>{
		const events = [
			socket_connect_event,
			socket_event_message
		];

		if ( ! using_socket ) {
			fetchMessages();
		} else {
			refreshMessages();
			registerSocketCallback(events, 'thread_screen_handler', ()=>fetchMessages());
			return ()=>removeSocketCallback(events, 'thread_screen_handler');
		}

	}, [peer_id]);

	useEffect(()=>{
		const {current} = textarea_reff;
		if ( current ) {
			current.style.height = 'auto';
			current.style.height = (current.scrollHeight + 1) + 'px';
		}
	}, [state.new_message]);

	return <div 
		className={'d-flex flex-direction-column'.classNames()}
		style={{height: '100%'}}
	>
		<div className={'d-flex align-items-center column-gap-15 box-shadow-thin'.classNames() + 'top-bar'.classNames(style)}>
			<div 
				className={'flex-1 d-flex align-items-center column-gap-8 cursor-pointer'.classNames()}
				onClick={()=>state.peer?.user_id ? navigate(`/profile/${state.peer?.user_id}/`) : 0}
			>
				<img src={state.peer?.avatar_url} className={'flex-shrink-0'.classNames()}/>
				<span>
					{state.peer?.display_name || __('Loading User..')}
				</span>
			</div>

			{
				socket_connected ? null :
				<div>
					{
						state.fetching ? 
							<LoadingIcon show={true}/> : 
							<i 
								className={'sicon sicon-reload font-size-16 color-text-80 cursor-pointer'.classNames()}
								onClick={()=>{
									fetchMessages();
									refreshMessages();
								}}
							></i>
					}
				</div>
			}
		</div>
		<div className={'flex-1 overflow-auto'.classNames()} ref={scrollable_ref} onScroll={fetchOnScroll}>

			<LoadingIcon 
				show={fetching} 
				center={true}
				className={'padding-vertical-10'.classNames()}
			/>

			{
				(state.error_message || !messages_state.length) ? 
					<div className={'d-flex align-items-center justify-content-center height-p-100'.classNames()}>
						<div>
							<div className={'text-align-center'.classNames()}>
								{
									fetching ? 
									<LoadingIcon show={true}/> : 
									<span className={'d-block font-size-16 margin-bottom-5'.classNames()}>
										{state.error_message || __('No message found')}
									</span>
								}
							</div>
						</div>
					</div> 
					:
					<div className={'messages'.classNames(style)}>
						{
							messages_state.map(message=>{

								const {message_id, sender_id, message_content, sent_time_unix} = message;
								const is_self = sender_id == self_id;

								return <div 
									key={message_id} 
									className={`message ${is_self ? 'self' : ''}`.classNames(style) + 'd-flex align-items-flex-end column-gap-8'.classNames()}
								>
									{
										is_self ? null : <div className={'d-flex'.classNames()}>
											<img src={state.peer?.avatar_url}/>
										</div>
									}

									<div className={'flex-1'.classNames()}>

										{
											show_time_for!==message_id ? null :
											<div 
												className={`${is_self ? 'text-align-right' : 'text-align-left'} color-text-40 margin-top-5`.classNames()}
												style={{fontSize: '10px'}}
											>
												{timeAgoOrAfter(sent_time_unix)}
											</div>
										}
										
										<div 
											className={`d-flex`.classNames()}
											style={{[is_self ? 'paddingLeft' : 'paddingRight']: '20%', justifyContent: !is_self ? 'flex-start' : 'flex-end'}}
										>
											<div 
												className={`d-inline-block`.classNames() + 'text-contents'.classNames(style)}
												dangerouslySetInnerHTML={{__html: message_content}}
												onClick={()=>setShowTimeFor(show_time_for===message_id ? null : message_id)}
											></div>
										</div>

									</div>
									
								</div>
							})
						}
					</div>
			}
		</div>

		{
			(state.error_message || (fetching && !messages_state.length)) ? null :
			<div 
				className={'d-flex align-items-center column-gap-10 padding-10'.classNames()} 
				style={{backgroundColor: '#eeeeee'}}
			>
				<div className={'flex-1'.classNames()}>
					<textarea 
						className={'d-block bg-color-white border-1 b-color-text-40 width-p-100'.classNames()}
						style={{resize: 'none', borderRadius: '20px', padding: '10px 11px', fontSize: '14px', height: '54px'}}
						value={state.new_message}
						onChange={e=>setState({...state, new_message: e.currentTarget.value})}
						disabled={state.sending}
						ref={textarea_reff}
						placeholder={__('Write message..')}
					></textarea>
				</div>
				<div className={'d-flex'.classNames()}>
					{
						state.sending ? 
							<LoadingIcon show={true}/> :
							<i 
								className={`sicon sicon-arrow-right ${(state.sending || isEmpty(state.new_message)) ? 'color-material-50' : 'color-material cursor-pointer'}`.classNames()}
								onClick={sendMessage}
								style={{fontSize: '25px'}}
							></i>
					}
				</div>
			</div>
		}
	</div>
}
