import React, { useContext, useEffect, useState } from "react";
import { Link, useNavigate, useParams } from "react-router-dom";

import {Tabs} from 'solidie-materials/tabs/tabs.jsx';
import { __, data_pointer, currency_symbol, getDashboardPath, getRandomString, isEmpty, sprintf } from "solidie-materials/helpers.jsx";
import { request } from "solidie-materials/request.jsx";
import { NumberField } from "solidie-materials/number-field/number-field.jsx";
import { DropDown } from "solidie-materials/dropdown/dropdown.jsx";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon.jsx";
import { ContextToast } from "solidie-materials/toast/toast.jsx";
import { Modal } from "solidie-materials/modal.jsx";

import { methods } from "../settings/withdrawal-method.jsx";
import { SalesList } from "../../../modules/sales/sales-list.jsx";
import { WithdrawalRequests } from "../../../modules/withdrawal-requests/withdrawal.jsx";

import * as style from './earnings.module.scss';

function Sales() {
	return <SalesList is_admin={false}/>
}

function Withdrawal({session, refreshSession}) {
	return <WithdrawalRequests 
		session={session} 
		refreshSession={refreshSession}
		is_admin={false}
	/>
}

const tabs = {
	sales: {
		label: __('Sales'),
		component: Sales
	},
	withdrawals: {
		label: __('Withdrawal'),
		component: Withdrawal
	}
}

const {readonly_mode} = window[data_pointer];

export function Earnings() {
	
	const {ajaxToast} = useContext(ContextToast);
	const {active_tab='sales'} = useParams();
	const {component: Comp} = tabs[active_tab] || tabs.sales;

	const navigate = useNavigate();

	const [state, setState] = useState({
		session: getRandomString(),
		fetching: true,
		error_message: null,
		withdrawal_methods: {},
		show_withdrawal_fields: false,
		requesting_withdrawal: false,
		withdrawal_minimum: 1,
		withdrawal_info: {
			method: '',
			amount: ''
		},
		stats: {
			withdrawn: 0, 
			available: 0, 
			awaiting: 0
		}
	});

	const onChange=(name, value)=>{
		setState({
			...state,
			withdrawal_info: {
				...state.withdrawal_info,
				[name]: value
			}
		});
	}

	const requestWithdrawal=()=>{
		
		setState({
			...state,
			requesting_withdrawal: true
		});

		request('requestWithdrawal', state.withdrawal_info, resp=>{
			
			ajaxToast(resp);

			setState({
				...state,
				requesting_withdrawal: false,
				show_withdrawal_fields: !resp.success,
				session: resp.success ? getRandomString() : state.session
			});
		});
	}

	const getEarnings=()=>{
		
		setState({
			...state,
			fetching: true
		});

		request('getMyEarningStats', {}, resp=>{
			const {
				success,
				data: {
					stats={},
					withdrawal_methods={},
					withdrawal_minimum=1,
					message=__('Something went wrong!')
				}
			} = resp;

			setState({
				...state,
				fetching: false,
				error_message: success ? null : message,
				stats: success ? stats : state.stats,
				withdrawal_methods: success ? withdrawal_methods : state.withdrawal_methods,
				withdrawal_minimum,
				withdrawal_info: {
					...state.withdrawal_info,
					amount: withdrawal_minimum
				}
			})
		});
	}

	const toggleModal=(show=false)=>{
		setState({
			...state,
			show_withdrawal_fields: show
		});
	}

	useEffect(()=>{
		getEarnings();
	}, [state.session]);

	const top_cards = {
		available: {
			label: <>
				{__('Available to')}&nbsp;
				<a 
					href="#" 
					className={'color-material interactive'.classNames()}
					data-cylector="withdraw-btn"
					onClick={e=>{
						e.preventDefault();
						toggleModal(true);
					}}
				>
					{__('Withdraw')}
				</a>
			</>
		},
		awaiting: {
			label: __('Awaiting to be Available')
		},
		withdrawn: {
			label: __('Total Withdrawn')
		}
	}

	return <div>
		{
			!state.show_withdrawal_fields ? null :
			<Modal closeOnDocumentClick={true} onClose={()=>toggleModal(false)} nested={true}>
				{
					isEmpty(state.withdrawal_methods) ? 
					<div className={'text-align-center padding-20'.classNames()}>
						<span className={'d-block margin-bottom-15'.classNames()}>
							{__('Set up withdrawal method first')}
						</span>
						<Link 
							className={'button button-primary button-outlined button-small'.classNames()}
							to={getDashboardPath('settings/withdrawal/')}
						>
							{__('Setup')}
						</Link>
					</div>
					:
					(
						state.withdrawal_minimum>state.stats.available ? 
						<div className={'color-warning text-align-center'.classNames()}>
							{__('Insufficient balance.')}<br/><br/>
							{sprintf(__('Minimum threshold is %s'), `${currency_symbol}${state.withdrawal_minimum}`)}
						</div>
						:
						<div>
							<div className={'margin-bottom-25'.classNames()}>
								<strong className={'font-size-24 margin-bottom-25 font-weight-500'.classNames()}>
									{__('Request Withdrawal')}
								</strong>
							</div>

							<div className={'d-flex align-items-center column-gap-15 margin-bottom-20 bg-color-material-3 border-1 b-color-material-7 border-radius-8 padding-20'.classNames()}>
								<div className={'flex-2'.classNames()}>
									<label className={'d-block margin-bottom-5 font-weight-500'.classNames()}>
										{__('Amount')} ({currency_symbol})
									</label>
									<NumberField
										min={state.withdrawal_minimum}
										max={state.stats.available}
										value={state.withdrawal_info.amount}
										onChange={v=>onChange('amount', v)}
										decimal_point={true}
									/>
								</div>
								<div className={'flex-5'.classNames()} data-cylector="method">
									<label className={'d-block margin-bottom-5 font-weight-500'.classNames()}>
										{__('Withdrawal Method')}
									</label>
									<DropDown
										value={state.withdrawal_info.method}
										onChange={v=>onChange('method', v)}
										options={Object.keys(state.withdrawal_methods).map(gateway=>{
											return {
												id: gateway,
												label: methods[gateway]?.label
											}
										})}
									/>
								</div>
							</div>
							
							<div className={'d-flex align-items-center column-gap-15 justify-content-flex-end'.classNames()}>
								<button className={'button button-primary button-outlined'.classNames()} onClick={e=>toggleModal(false)}>
									{__('Cancel')}
								</button>
								<button 
									className={'button button-primary'.classNames()}
									disabled={readonly_mode || state.requesting_withdrawal || isEmpty(state.withdrawal_info.method) || isEmpty(state.withdrawal_info.amount)}
									onClick={requestWithdrawal}
									data-cylector="submit-withdrawal"
								>
									{__('Submit')} <LoadingIcon show={state.requesting_withdrawal}/>
								</button>
							</div>
						</div>
					)
				}
			</Modal>
		}
		
		<div className={'top-cards'.classNames(style)}>
			{
				Object.keys(top_cards).map(key=>{
					const {label} = top_cards[key];

					return <div key={key} className={'bg-color-white box-shadow-thin border-radius-8'.classNames()}>
						<strong className={'d-block margin-bottom-15 font-weight-700 font-size-24 color-text-90'.classNames()}>
							{currency_symbol}{state.stats[key] || 0}
						</strong>
						<span className={'d-block font-weight-500 font-size-14 color-text-50'.classNames()}>
							{label}
						</span>
					</div>
				})
			}
		</div>

		<div className={'margin-bottom-15'.classNames()}>
			<div className={'d-inline-block'.classNames()} style={{width: '200px'}}>
				<Tabs
					onNavigate={t=>navigate(getDashboardPath(`earnings/${t}/`))} 
					active={active_tab} 
					theme='transparent'
					tabs = {
						Object.keys(tabs).map(k=>{
							return {
								id: k,
								label: tabs[k].label
							}
						})
					}
				/>
			</div>
		</div>

		<Comp 
			session={state.session} 
			refreshSession={()=>setState({
				...state, 
				session: getRandomString()
			})}
		/>
	</div>
}
