import React, { useContext, useEffect, useState } from "react";

import { __, copyToClipboard, currency_symbol, formatDate, formatDateTime, getBack, getDashboardPath } from "solidie-materials/helpers.jsx";
import { InitState } from "solidie-materials/init-state.jsx";
import { request } from "solidie-materials/request.jsx";
import { confirm } from "solidie-materials/prompts.jsx";
import { ContextToast } from "solidie-materials/toast/toast.jsx";
import { TableStat } from "solidie-materials/table-stat.jsx";
import { order_statuses } from "solidie-materials/data.jsx";

import { Link } from "react-router-dom";

export function RenewalsTable({renewals}) {
	return <table >
		<tbody>
			{
				renewals.map(renewal=>{
					return <tr key={renewal.sale_id}>
						<th>
							{currency_symbol}{renewal.sale_price}
							&nbsp;
							{
								renewal.order_status=='completed' ? null : 
								<i>
									<small>
										({order_statuses[renewal.order_status] || renewal.order_status})
									</small>
								</i>
							}
						</th>
						<td>
							{formatDate(renewal.purchase_date)}
						</td>
					</tr>
				})
			}
		</tbody>
	</table>
}

export function LicenseKeyManager({sale_id, can_back=true}) {
	
	const {ajaxToast, addToast} = useContext(ContextToast);

	const [state, setState] = useState({
		fetching: true,
		deleting: {},
		error_message: null,
		sale: {},
		license_keys: [],
		renewals: []
	});

	const takeAction=(license_action, payload={})=>{
		request('licenseKeyAction', {sale_id, license_action, ...payload}, resp=>{
			
			const {
				success,
				data: {
					license_keys=[],
					sale={},
					renewals=[],
					message
				}
			} = resp;

			setState({
				...state,
				fetching: false,
				sale: success ? sale : state.sale,
				license_keys: success ? license_keys : state.license_keys,
				renewals: success ? renewals : state.renewals
			});

			if ( message ) {
				ajaxToast(resp);
			}
		});
	}

	useEffect(()=>{
		takeAction();
	}, []);

	if ( state.fetching || state.error_message ) {
		return <InitState fetching={state.fetching} error_message={state.error_message}/>
	}

	return <div style={{maxWidth: '700px', margin: 'auto'}} data-cylector="purchase-single-wrapper">

		{
			!can_back ? null :
			<div className={'d-flex align-items-center column-gap-8 margin-top-15 margin-bottom-25'.classNames()}>
				<Link 
					to={getDashboardPath(`my-purchases`)}
					className={'sicon sicon-arrow-left font-size-24 color-text-90 interactive'.classNames()}
					onClick={getBack}
				/>
				<a 
					href={state.sale.content_permalink}
					className={'font-size-18 color-text-90 interactive'.classNames()}
					target="_blank"
				>
					{state.sale.content_title}
				</a>
			</div>
		}
		
		<div className={'margin-bottom-20'.classNames()}>
			<strong className={'d-block margin-bottom-5 font-size-18 font-weight-500'.classNames()}>
				{__('Overview')}
			</strong>
			<table className={'table vertical-head font-size-14'.classNames()}>
				<tbody>
					<tr>
						<td data-th={__('Order ID')}>
							{state.sale.order_id}
						</td>

						<td data-th={__('Expires')}>
							{state.sale.expires_on ? formatDate(state.sale.expires_on) : __('Never')}
						</td>

						<td data-th={__('License Key Limit')}>
							{state.sale.license_or_content_limit || __('No Limit')}
						</td>

						<td data-th={__('Created Keys')}>
							{state.license_keys.length}
						</td>

						<td data-th={__('Connected Keys')}>
							{state.license_keys.filter(l=>l.endpoint).length}
						</td>
						
						{
							!state.renewals.length ? null :
							<td data-th={__('Renewals')}>
								<RenewalsTable renewals={state.renewals}/>
							</td>
						}
					</tr>
				</tbody>
			</table>
		</div>
		
		<div>
			<strong className={'d-flex align-items-center column-gap-10 margin-bottom-5'.classNames()}>
				<strong className={'d-block margin-bottom-5 font-size-18 font-weight-500'.classNames()}>
					{__('License Keys')}
				</strong>
				{
					(state.sale.license_or_content_limit && state.sale.license_or_content_limit<=state.license_keys.length) ? null :
					<i 
						className={'sicon sicon-add-circle cursor-pointer'.classNames()}
						onClick={()=>takeAction('add')}
					></i>
				}
			</strong>
			<table className={'table'.classNames()}>
				<thead>
					<tr>
						<th>{__('Key')}</th>
						<th>{__('Connected To')}</th>
						<th>{__('Action')}</th>
					</tr>
				</thead>
				<tbody className={'font-size-14'.classNames()}>
					{
						state.license_keys.map(license=>{
							const {license_id, license_key, endpoint} = license;
							return <tr key={license_id}>
								<td data-th={__('Key')}>
									<span 
										className={'hover-underline cursor-pointer'.classNames()}
										title={__('Click to copy')}
										onClick={()=>copyToClipboard(license_key, addToast)}
									>
										{license_key}
									</span>
								</td>
								<td data-th={__('Connected To')}>
									{endpoint || <>N\A</>}
								</td>
								<td data-th={__('Action')}>
									<div className={'d-flex align-items-center column-gap-15'.classNames()}>
										<i 
											className={'sicon sicon-trash color-error cursor-pointer'.classNames()}
											title={__('Delete key')}
											onClick={()=>{
												confirm(
													__('Sure to delete?'), 
													()=>takeAction('delete', {license_id}))
												}
											}
										></i>
												
										<i 
											className={'sicon sicon-reload color-material cursor-pointer'.classNames()}
											title={__('Regenerate Key')}
											onClick={()=>{
												confirm(
													__('Sure to regenerate?'),
													()=>takeAction('regenerate', {license_id})
												)
											}}
										></i>
									</div>
								</td>
							</tr>
						})
					}
					<TableStat 
						empty={!state.license_keys?.length} 
						loading={state.fetching}
						message={__('No license key found!')}/>
				</tbody>
			</table>
		</div>
	</div>
}