import React, { useEffect, useState } from 'react'
import { Link, useNavigate, useParams } from 'react-router-dom';

import { request } from 'solidie-materials/request.jsx';
import { __, data_pointer, formatDate } from 'solidie-materials/helpers.jsx';
import { TableStat } from 'solidie-materials/table-stat.jsx';
import { Options } from "solidie-materials/dropdown/dropdown.jsx";
import { order_statuses } from "solidie-materials/data.jsx";

import { SinglePurchase } from './single';
import { LicenseKeyManager } from './license-keys';

const {permalinks: {checkout_url}} = window[data_pointer];

export function PurchasedContents(){

	const navigate = useNavigate();

	const {sale_id, content_type} = useParams();

	const [state, setState] = useState({
		fetching: true,
		contents:[]
	});

	const getContents=()=>{
		setState({
			...state,
			fetching: true
		});

		request('getPurchasedContents', {}, resp=>{
			
			let {data:{contents=[]}} = resp;

			setState({
				...state, 
				contents,
				fetching: false
			});
		})
	}

	useEffect(()=>{
		getContents();
	}, []);

	const _current_content = sale_id ? state.contents.find(c=>c.sale_id==sale_id) : {};

	return sale_id ? (
		content_type==='app' ? 
			<LicenseKeyManager sale_id={sale_id}/> : 
			<SinglePurchase sale_id={sale_id} content={_current_content} fetching={state.fetching}/>
	) 
	: 
	<div>
		<table className={'table '.classNames()}>
			<thead>
				<tr>
					<th>{__('Content')}</th>
					<th>{__('Plan')}</th>
					<th>{__('Purchase Date')}</th>
					<th>{__('Expire Date')}</th>
					<th></th>
				</tr>
			</thead>
			<tbody>
				{state.contents.map(content => {
					let {
						content_id,
						content_slug,
						content_type,
						sale_id,
						content_title, 
						content_permalink, 
						purchase_date,
						media={},
						order_status,
						order_status_label,
						expires_on,
						product={},
						renewals=[],
						tutorial_url,
						release,
					} = content;
					
					const purchased_plan = product.plan || {};
					const {variation_id} = purchased_plan || {};
					const {plan_name, subscription_model, sales_model, id: plan_id} = purchased_plan.plan || {};

					const completed     = order_status === 'completed';
					const download_url  = completed ? release?.download_url : null;
					// const non_completes = renewals.map(r=>r.order_status=='completed' ? null : (order_statuses[r.order_status] || r.order_status)).filter(s=>s);

					const actions = [
						{
							id: 'download', 
							label: __('Download'),
							icon: `sicon sicon-download1 color-success`.classNames(),
							open: download_url
						},
						{
							id: 'read', 
							label: __('Start Reading'),
							icon: `sicon sicon-book-open color-success`.classNames(),
							open: tutorial_url
						},
						{
							id: 'explore', 
							label: __('Explore'),
							icon: `sicon sicon-folder color-success`.classNames(),
							open: sales_model === 'pack' ? `${window[data_pointer].permalinks.gallery[content_type]}?content_pack_plan=${plan_id}` : null
						},
						{
							id: 'renew', 
							label: __('Renew'),
							icon: `sicon sicon-clock-fast-forward color-material`.classNames(),
							open: (content_slug && variation_id && subscription_model!=='lifetime') ? `${checkout_url}?add-to-cart=${variation_id}` : null
						},
					].filter(({open})=>open);

					return <tr key={sale_id}>
						<td data-th={__('Content')}>
							<div className={'d-flex align-items-center column-gap-15'.classNames()}>
								<div>
									<img 
										src={media?.thumbnail?.file_url} 
										style={{width: '50px', height: '30px', objectFit: 'cover', objectPosition: 'center', borderRadius: '5px'}}
									/>
								</div>
								<div className={'flex-1'.classNames()}>
									<Link
										to={`${content_type}/${sale_id.toString()}`} 
										className={'d-block font-size-16 font-weight-500 color-text-90 interactive'.classNames()}
										data-cylector="content-single-link"
									>
										{content_title || <i>{__('Deleted Content')}</i>}
										{
											completed ? null :
											<i>
												<small className={'d-block color-warning'.classNames()}>
													{order_status_label}
												</small>
											</i>
										}
									</Link>
								</div>
							</div>
						</td>
						<td data-th={__('Plan')} className={'color-text-70 font-size-15'.classNames()}>
							{plan_name}
						</td>
						<td data-th={__('Purchase Date')} className={'color-text-70 font-size-15'.classNames()}>

							{formatDate( purchase_date )}

							{
								/* !non_completes.length ? null :
								<i className={'d-block font-size-13 color-text-60 font-weight-400'.classNames()}>
									{non_completes.join(', ')}
								</i> */
							}
						</td>
						<td data-th={__('Expire Date')} className={'color-text-70 font-size-15'.classNames()}>
							{expires_on ? formatDate(expires_on) : __('Never')}
						</td>
						<td data-th={__('Action')}>
							<div className={'d-flex align-items-center column-gap-15'.classNames()}>
								{
									actions.map(action=>{
										return <a 
											key={action.id}
											title={action.label}
											className={action.icon + `d-flex font-size-14 cursor-pointer`.classNames()}
											href={action.open}
											target='_blank'
										></a>
									})
								}
							</div>
						</td>
					</tr>
				})}
				
				<TableStat 
					loading={state.fetching} 
					empty={!state.contents.length}
				/>
			</tbody>
		</table>
	</div>
}
