import React, { useEffect, useState } from "react";
import { Link } from "react-router-dom";

import {request} from 'solidie-materials/request.jsx';
import {InitState} from 'solidie-materials/init-state.jsx';
import { __, data_pointer, formatDate, getBack, getDashboardPath, sprintf } from "solidie-materials/helpers.jsx";
import {TableStat} from 'solidie-materials/table-stat.jsx';
import { RenewalsTable } from "./license-keys";

const {settings:{contents={}}} = window[data_pointer];

export function SinglePurchase({content, fetching}) {
	
	const {sale_id, content_type, content_title, content_permalink} = content || {};
	const is_tutorial = content_type === 'tutorial';

	const [state, setState] = useState({
		fetching: true,
		sale: {},
		downloads: [],
		error_message: null,
		renewals: []
	});

	const getSinglePurchaseInfo=()=>{

		if ( !sale_id ) {
			return;
		}

		setState({
			...state,
			fetching: true
		});

		request('getSinglePurchaseInfo', {sale_id}, resp=>{
			const {
				success, 
				data:{
					downloads=[], 
					sale={}, 
					renewals=[],
					message=__('Something went wrong!')
				}
			} = resp;

			setState({
				...state,
				fetching: false,
				downloads,
				sale,
				renewals,
				error_message: success ? null : message
			});
		});
	}
	
	useEffect(()=>{
		getSinglePurchaseInfo();
	}, [sale_id]);

	if ( fetching || state.fetching || state.error_message ) {
		return <InitState 
				fetching={state.fetching} 
				error_message={state.error_message}/>
	}

	return <div style={{maxWidth: '700px', margin: 'auto'}} data-cylector="purchase-single-wrapper">

		<div className={'d-flex align-items-center column-gap-8 margin-top-15 margin-bottom-25'.classNames()}>
			<Link 
				to={getDashboardPath(`my-purchases`)}
				className={'sicon sicon-arrow-left font-size-24 color-text-90 interactive'.classNames()}
				onClick={getBack}
			/>
			<a 
				href={content_permalink}
				className={`font-size-18 color-text-90 ${content_permalink ? 'interactive' : ''}`.classNames()}
				target="_blank"
			>
				{content_title}
			</a>
		</div>

		<strong className={'d-block margin-bottom-5 font-size-18 font-weight-500'.classNames()}>
			{__('Overview')}
		</strong>
		<table className={'table vertical-head margin-bottom-20'.classNames()}>
			<tbody>
				<tr className={'font-size-14'.classNames()}>
					<td data-th={__('Order ID')}>{state.sale.order_id}</td>
					<td data-th={__('Expires')}>{state.sale.expires_on ? formatDate(state.sale.expires_on) : __('Never')}</td>

					{
						is_tutorial ? null :
						<td data-th={__('Downloads')}>
							{state.downloads.length || 0}/{state.sale.license_or_content_limit || <>&infin;</>}
						</td>
					}

					{
						!state.renewals.length ? null :
						<td data-th={__('Renewals')}>
							<RenewalsTable renewals={state.renewals}/>
						</td>
					}
				</tr>
			</tbody>
		</table>

		<strong className={'d-block margin-bottom-5 font-size-18 font-weight-500'.classNames()}>
			{!is_tutorial ? __('Download History') : sprintf(__('Opened %s'), contents.tutorial.label)}
		</strong>
		<table className={'table no-responsive'.classNames()}>
			<thead>
				<tr>
					<th>{__('Content')}</th>
					<th>{__('Downlod Date')}</th>
					<th></th>
				</tr>
			</thead>
			<tbody className={'font-size-14'.classNames()}>
				{
					state.downloads.map(down=>{
						const {
							download_time,
							content={},
						} = down;

						const {
							media={},
							content_title,
							content_permalink,
							release
						} = content;

						return <tr key={download_time}>
							<td>
								<div className={'d-flex align-items-center column-gap-15'.classNames()}>
									<div>
										<img src={media?.thumbnail?.file_url} style={{width: '35px', height: 'auto'}}/>
									</div>
									<div className={'flex-1'.classNames()}>
										<a href={content_permalink} target='_blank' className={'d-block'.classNames()}>
											{content_title} 
										</a>
									</div>
								</div>
							</td>
							<td className={'white-space-nowrap'.classNames()}>
								{formatDate(download_time)}
							</td>
							<td>
								<a 
									href={is_tutorial ? `${content_permalink}0/` : release?.download_url} 
									className={'white-space-nowrap button button-primary button-small'.classNames()}
									target="_blank"
								>
									{is_tutorial ? __('Start Learning') : __('Download')}
								</a>
							</td>
						</tr>
					})
				}
				<TableStat 
					empty={!state.downloads?.length} 
					loading={state.fetching}
					message={!is_tutorial ? __('Downloaded contents will apepar here.') : __('No history')}/>
			</tbody>
		</table>
	</div>
}
