import React, { useContext, useEffect, useState } from "react";

import { __, data_pointer, isEmpty } from "solidie-materials/helpers.jsx";
import { Tabs } from "solidie-materials/tabs/tabs.jsx";
import { TextField } from "solidie-materials/text-field/text-field.jsx";
import { RadioCheckbox } from "solidie-materials/radio-checkbox.jsx";
import { request } from "solidie-materials/request";
import { ContextToast } from "solidie-materials/toast/toast";
import { patterns } from "solidie-materials/data.jsx";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon";
import { InitState } from "solidie-materials/init-state";

export const methods = {
	paypal: {
		label: __('Paypal'),
		fields: [
			{
				name: 'email',
				label: __('Paypal Email'),
				required: true,
				regex: patterns.email
			}
		]
	},
	payoneer: {
		label: __('Payoneer'),
		fields: [
			{
				name: 'email',
				label: __('Payoneer Email'),
				required: true,
				regex: patterns.email
			}
		]
	},
	interac: {
		label: __('Interac'),
		fields: [
			{
				name: 'email',
				label: __('Interac e-Transfer Email'),
				required: true,
				regex: patterns.email
			}
		]
	},
	wise: {
		label: __('Wise'),
		fields: [
			{
				name: 'email',
				label: __('Wise Email'),
				required: true,
				regex: patterns.email
			}
		]
	},
}

const {readonly_mode} = window[data_pointer];

export function WithdrawalSetting() {

	const {ajaxToast} = useContext(ContextToast);

	const [state, setState] = useState({
		active_tab: null,
		has_changes: false,
		saving: false,
		fetching: true,
		error_message: null,
		values: {}
	});

	const method_keys = Object.keys(state.values);

	const onChange=(name, value)=>{
		
		const {active_tab: method, values={}} = state;
		values[method] = values[method] || {};

		setState({
			...state,
			has_changes: true,
			values: {
				...values,
				[method]: {
					...values[method],
					[name]: value
				}
			}
		});
	}

	const saveChanges=()=>{

		const {active_tab: method} = state;
		const fields = state.values[method];
		
		setState({
			...state,
			saving: true
		});

		request('saveWithdrawalMethod', {fields, method}, resp=>{
			
			setState({
				...state,
				saving: false,
			});

			ajaxToast(resp);
		});
	}

	const getSavedMethods=()=>{
		setState({
			...state,
			fetching: true
		});

		request('getWithdrawalMethods', {}, resp=>{

			const {
				success, 
				data: {
					methods={}, 
					message=__('Something went wrong!')
				}
			} = resp;

			setState({
				...state,
				fetching: false,
				active_tab: Object.keys(methods)[0] || state.active_tab,
				error_message: success ? null : message,
				values: methods
			});
		});
	}

	useEffect(()=>{
		getSavedMethods();
	}, []);

	// Determine if the form can be submitted
	const can_submit = methods[state.active_tab]?.fields?.filter?.(field=>{
		let value = state.values[state.active_tab]?.[field.name] || '';

		if ( ! isNaN( value ) ) {
			value = value.toString();
		}
		
		return field.regex && !field.regex.test(value.trim());
	}).length===0;

	if ( state.fetching || state.error_message ) {
		return <InitState fetching={state.fetching} error_message={state.error_message}/>
	}

	if ( isEmpty(state.values) ) {
		return <div className={'font-size-14 color-warning text-align-center'.classNames()}>
			No withdrawal method enabled. <br/>
			Please contact site authority.
		</div>
	}

	return <div>

		{
			method_keys.length < 2 ? null :
			<Tabs
				onNavigate={t=>setState({...state, active_tab: t})} 
				active={state.active_tab} 
				theme='button'
				tabs = {
					method_keys.map(k=>{
						return {
							id: k,
							label: methods[k].label
						}
					})
				}
			/>
		}
		
		<br/>

		{
			methods[state.active_tab]?.fields?.map?.(field=>{
				
				const {label, name, type='text', required=false, options=[]} = field;
				const value = state.values[state.active_tab]?.[name] || '';

				return <div key={name} className={'margin-bottom-15'.classNames()}>
					<label className={'d-block font-size-16 font-weight-400 margin-bottom-10'.classNames()}>
						{label}{required ? <span className={'color-error'.classNames()}>*</span> : null}
					</label>
					<div data-cylector="field-wrapper">
						{
							type !== 'text' ? null :
							<TextField
								type={type}
								value={value}
								name={name}
								onChange={v=>onChange(name, v)}
							/>
						}
						
						{
							type !== 'radio' ? null :
							<div className={'d-flex align-items-center column-gap-15'.classNames()}>
								<RadioCheckbox
									type={type}
									name={name}
									value={value}
									options={options}
									onChange={(value) => onChange(name, value)}
								/>
							</div>
						}
					</div>
				</div>
			})
		}

		<div className={'text-align-right'.classNames()}>
			<button 
				className={'button button-primary'.classNames()} 
				onClick={saveChanges}
				disabled={readonly_mode || state.saving || !state.has_changes || !can_submit}
				data-cylector="save-withdrawal-settings"
			>
				{__('Save')} <LoadingIcon show={state.saving}/>
			</button>
		</div>
	</div>
}
