import React, { createContext, useEffect, useState } from "react";

import io from '../../../node_modules/socket.io/client-dist/socket.io';
import { data_pointer } from "solidie-materials/helpers";
import { request } from "solidie-materials/request";

const {
	nonce,
	nonce_action,
	permalinks: {
		socket_url
	}, 
	user:{
		id: user_id
	}
} = window[data_pointer];

export const ContextSocket           = createContext();
export const using_socket            = user_id && socket_url;
export const socket_connect_event    = 'socket_on_connect_event_key';
export const socket_disconnect_event = 'socket_on_disconnect_event_key';

export function SocketWrapper({children}) {

	const [socket, setSocket] = useState();
	const [connected, setConnected] = useState(false);
	const [handlers, setHandlers] = useState({});
	const [stats, setStats] = useState({
		message: 0,
		notification: 0,
		any: 0
	});

	const registerSocketCallback=(_event, id, callback)=>{
		
		const _handlers = handlers;
		const events    = Array.isArray(_event) ? _event : [_event];

		for ( let n=0; n<events.length; n++ ) {
			
			const event = events[n];

			if ( !_handlers[event] ) {
				_handlers[event]={};
			}

			_handlers[event][id]=callback;


			// Execute connect hook if socket is conncted already
			if ( event === socket_connect_event && connected ) {
				callback();
			}
		}
	
		setHandlers(_handlers);
	}

	const removeSocketCallback=(_event, id)=>{
		
		let _handlers = handlers;
		const events  = Array.isArray(_event) ? _event : [_event];

		for ( let n=0; n<events.length; n++ ) {
			
			const event = events[n];
			
			if (_handlers[event]?.[id]) {
				delete _handlers[event][id];
			}
		}
		
		setHandlers(_handlers);
	}

	const triggerHook=(event, data)=>{
		const _handlers = handlers[event] || {};
		Object.keys(_handlers).forEach(id=>{
			_handlers[id](data);
		});
	}

	const fetchStats=()=>{
		request('getSocketStats', {}, resp=>{
			const {data:{message=0, notification=0}} = resp;
			setStats({
				...stats,
				message,
				notification,
				any: message || notification || 0
			});
		})
	}
	
	const emitSocket=(recipient_ids, event, data)=>{

		if ( ! socket ) {
			return;
		}

		// Make sure it is array
		const _ids = Array.isArray(recipient_ids) ? recipient_ids : [recipient_ids];
	
		_ids.forEach(recipient_id=>socket.emit('message', {recipient_id, event, data}));
	}

	useEffect(()=>{
		if ( ! using_socket ) {
			return;
		}

		// Make socket connection
		const socket = io(socket_url, {query:{user_id, nonce, nonce_action}});

		// Register inbound message dispatcher
		socket.on('message', ({event, data}) => {
			triggerHook(event, data);
			fetchStats();
		});

		// Dispatch on connect event
		socket.on('connect', () => {
			setConnected(true);
			triggerHook(socket_connect_event);
		});

		socket.on('disconnect', () => {
			setConnected(false);
			triggerHook(socket_disconnect_event);
		});

		setSocket(socket);
	}, []);

	useEffect(()=>{
		fetchStats();
	}, [connected] );

	return <ContextSocket.Provider 
		value={{
			registerSocketCallback, 
			removeSocketCallback, 
			emitSocket,
			refreshStats: fetchStats, 
			stats, 
			connected
		}}
	>
		{children}
	</ContextSocket.Provider>
}