import React, { useContext, useEffect, useState } from "react";

import { __, currency_symbol, data_pointer, flattenArray, getDashboardPath } from "solidie-materials/helpers.jsx";
import { request } from "solidie-materials/request.jsx";
import { ContextToast } from "solidie-materials/toast/toast.jsx";
import { InitState } from "solidie-materials/init-state.jsx";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon.jsx";

import { subscription_models } from "../../admin-dashboard/plan-settings/plan-settings";

import * as style from './cart.module.scss';

const {permalinks:{cart_url}={}, readonly_mode} = window[data_pointer];

/**
 * Show donwload button, renew button, error message etc. based the current condition.
 */
function DecideAction(props) {

	const {
		plan={}, 
		cart=[], 
		purchases=[], 
		addToCart, 
		adding,
		has_payment_gateway
	} = props;

	const {
		variation_id,
		product_id,
	} = plan;

	const gateway_notice = has_payment_gateway===false ? <small className={'color-error'.classNames()}><i>{__('No payment gateway to proceed')}</i></small> : null;
	
	/**
	 * Get purchase status
	 * @param {Array} purchases 
	 * @returns bool|null True for valid, false expired, null
	 */
	function hasValidPurchase(purchases) {

		let expired_all = true;

		// Find the variation that is valid to show download button for
		const valid_purchases = purchases.filter(purchase=>{
			const {
				limit_reached,
				is_expired
			} = purchase;

			// If at least one non expired purchase is there, negate the the flag for all expired.
			if ( ! is_expired ) {
				expired_all = false;
			}

			return ! is_expired && ! limit_reached;
		});

		return valid_purchases.length ? true : ( expired_all ? 'expired_all' : 'limit_reached' );
	}

	// No matter what, if the variation is in cart, show view cart link button.
	if ( cart.indexOf(variation_id)>-1 ) {
		return <a 
			className={'button button-primary button-full-width'.classNames()} 
			href={cart_url}
		>
			{__('View Cart')}
		</a>
	}
	
	// If the variation is purchased, show renew/download button or error message for limit reach
	if( purchases.length > 0 ) {
		
		const purchase_status = hasValidPurchase(purchases);

		// If there's valid purchase, show purchase view button.
		if ( purchase_status === true ) {

			// Multiple purchase for a variation can be there. 
			// In terms of downloading the same variation, multiple purchase will act like a combined single one to keep the UX simple.

			return <a 
				className={'button button-primary button-full-width'.classNames()}
				href={getDashboardPath('my-purchases/')}
			>
				{__('View Purchases')}
			</a>
		}

		// So downloadable purchase not found.
		// Show renew button. Renewal will reset seasonal download count too.
		// No need to show renewal button for multiple sale of same variation
		// Renewing any of them will allow downloading as all works like a combined
		return <div>
			<button 
				className={'button button-primary button-full-width'.classNames()} 
				onClick={()=>addToCart(product_id, variation_id)}
				disabled={!has_payment_gateway || adding}
				title={purchase_status==='expired_all' ? __('Subscription Expired') : __('Access Limit Reached')}
			>
				{__('Renew Now')} <LoadingIcon show={adding}/>
			</button>
			{gateway_notice}
		</div> 
	}

	// If all the condition fails, it means the variation is neither in cart nor purchased
	// Show add to cart button then
	return <div>
		<button 
			className={'button button-primary button-full-width'.classNames()} 
			onClick={()=>addToCart(product_id, variation_id)}
			disabled={!has_payment_gateway || readonly_mode || adding}
			title={has_payment_gateway===false ? __('No payment gateway is setup yet') : null}
			data-cylector="add-to-cart"
		>
			{__('Add to Cart')} <LoadingIcon show={adding}/>
		</button>
		{gateway_notice}
	</div> 
}

export function CartButtons({content={}}) {

	const {ajaxToast} = useContext(ContextToast);
	const {product={}, release} = content;
	const {plans: _plans=[], product_id, is_purchase_optional} = product;

	const plans = _plans.filter(p=>p.enable);

	const [state, setState] = useState({
		fetching: true,
		adding_to_cart: false,
		cart: [],
		purchases: {},
		selected_plan: null,
		has_payment_gateway: null
	});

	const addToCart=(product_id, variation_id)=>{
		setState({
			...state,
			adding_to_cart: true
		});

		request('addToCart', {product_id, variation_id}, resp=>{
			const {success, data:{cart=[]}} = resp;

			ajaxToast(resp);
			
			setState({
				...state, 
				adding_to_cart: false,
				cart: success ? cart : state.cart,
			});
		});
	}

	const getCart=()=>{
		setState({
			...state,
			fetching: true
		});

		request('getCart', {content_id: content.content_id}, resp=>{
			const {
				data:{
					cart=[], 
					purchases={},
					has_payment_gateway
				}} = resp;

			setState({
				...state,
				fetching: false,
				purchases,
				cart,
				has_payment_gateway,
				selected_plan: plans.find(p=>(cart[product_id] || []).indexOf(p.variation_id)>-1) || plans[0] || null 
			});
		});
	}

	useEffect(()=>{
		getCart();
	}, []);

	if ( state.fetching ) {
		return <InitState fetching={state.fetching}/>
	}

	if ( ! plans.length ) {
		return <div className={'border-1 border-radius-10 b-color-material-20 padding-horizontal-15 padding-vertical-30 text-align-center'.classNames()}>
			{__('No plan found to purchase')}
		</div>
	}

	const _purchases    = flattenArray(Object.keys(state.purchases).map(variation_id=>state.purchases[variation_id].filter(purchase=>purchase.is_download_allowed)));
	let _order_status   = _purchases[0]?.order_status;
	let _download_url   = _purchases[0]?.release?.download_url;
	let _download_icon  = 'sicon sicon-download'.classNames();
	let _download_label = __('Donwload');
	
	if ( content.content_type === 'tutorial' ) {
		_download_url   = _purchases.length ? '0/' : null;
		_download_icon  = 'sicon sicon-book-open'.classNames();
		_download_label = __('Start Learning');
	}

	if ( _order_status !== 'completed' ) {
		_download_url = null;
	}

	if (!_download_url && is_purchase_optional) {
		_download_url = release?.download_url;
	}

	return <div>
		<div className={'cart'.classNames(style) + 'border-1 b-color-material-20 border-radius-10'.classNames()}>
			{
				plans.map((plan, index)=>{
					const {
						plan: {
							id: plan_key, 
							plan_name, 
							description,
							sales_model,
							subscription_model,
							license_or_content_limit,
						},
						regular_price, 
						sale_price, 
						variation_id,
						product_id,
						enable
					} = plan;

					const cart = state.cart[product_id] || [];
					const is_active = plans.length===1 || state.selected_plan?.variation_id==variation_id;
					const in_cart = cart.indexOf(variation_id)>-1;
					const is_purchased = state.purchases[variation_id] ? true : false;
					const _model = subscription_models[subscription_model] || subscription_model;

					return <div 
						key={variation_id} 
						className={`variation ${is_active ? 'active' : ''}`.classNames(style) + `padding-vertical-20 padding-horizontal-15 cursor-pointer ${index<plans.length-1 ? 'border-bottom-1 b-color-material-20' : ''}`.classNames()}
						onClick={()=>setState({...state, selected_plan: plan})}
						title={in_cart ? __('Added to cart') : (is_purchased ? __('Purchased already') : null)}
					>
						<div className={'d-flex align-items-center column-gap-15'.classNames()}>
							<div>
								{
									(in_cart || is_purchased) ? 
										<i className={`sicon ${is_purchased ? 'sicon-briefcase1' : 'sicon-local_grocery_store'} color-material-90 font-size-18 color-material`.classNames()}></i> :
										<input type="radio" checked={is_active} onChange={()=>{}}/>
								}
							</div>
							<div className={'flex-1'.classNames()}>
								<strong className={'d-block font-size-16 color-text-80 font-weight-600'.classNames()}>
									{plan_name}
								</strong>

								{
									!(content.content_type === 'app' || sales_model === 'pack') ? null :
									<span className={'margin-top-5 color-text-70 font-size-14 font-weight-500'.classNames()}>
										{
											content.content_type === 'tutorial' ? 
												_model : 
												`${_model} ${license_or_content_limit} ${content.content_type === 'app' ? __('Licenses') : __('Downloads')}`
										}
									</span>
								}

								{
									(!is_active || ! description) ? null :
									<span className={'d-block font-size-13 font-weight-400 color-text-50 margin-top-5'.classNames()}>
										{description}
									</span>
								}
							</div>
							<div>
								<strong className={'font-size-17 color-text font-weight-700'.classNames()}>
									{currency_symbol}{__(sale_price ?? regular_price)}
								</strong>
								{
									!(sale_price<regular_price) ? null : <sub>
										&nbsp;&nbsp;
										<s>
											{currency_symbol}{__(regular_price)}
										</s>
									</sub>
								}
							</div>
						</div>
					</div>
				})
			}
		</div> 

		<div className={'d-flex column-gap-10 margin-top-15'.classNames()}>
			{
				!state.selected_plan ? null :
				<div className={'flex-1'.classNames()}>
					<DecideAction 
						plan={state.selected_plan} 
						addToCart={addToCart}
						cart={state.cart[state.selected_plan.product_id] || []}
						purchases={state.purchases[state.selected_plan.variation_id] || []}
						adding={state.adding_to_cart}
						has_payment_gateway={state.has_payment_gateway}
					/>
				</div>
			}

			{
				_download_url===null ? null :
				<div className={`${!state.selected_plan ? 'width-p-100' : ''}`.classNames()}>
					<a 
						className={`button button-primary ${!state.selected_plan ? 'button-full-width' : ''}`.classNames()}
						href={_download_url}
					>
						<i className={_download_icon}></i> {!state.selected_plan ? _download_label : '' }
					</a>
				</div>
			}
		</div>
	</div> 
}
