import React, {useEffect, useState} from "react";

import { request } from "solidie-materials/request.jsx";
import { Pagination } from 'solidie-materials/pagination/pagination.jsx';
import { __, currency_symbol, formatDate, isEmpty } from "solidie-materials/helpers.jsx";
import { TableStat } from "solidie-materials/table-stat.jsx";

export const order_statuses = {
	pending        : __('Pending payment'),
	processing     : __('Processing'),
	'on-hold'      : __('On hold'),
	completed      : __('Completed'),
	cancelled      : __('Cancelled'),
	refunded       : __('Refunded'),
	failed         : __('Failed')
}

export function SalesList({is_admin=false}) {

	const [state, setState] = useState({
		sales: [],
		segmentation: {},
		fetching: true,
		error_message: null,
		filter: {
			page: 1
		}
	});

	const getSales=()=>{
		request('getSalesList', {...state.filter, is_admin}, resp=>{
			setState({
				...state,
				fetching: false,
				sales: resp.data?.sales || [],
				segmentation: resp.data.segmentation
			})
		});
	}

	const setFilter=(name, value)=>{
		setState({
			...state,
			filter: {
				...state.filter,
				[name]: value
			}
		});
	}

	useEffect(()=>{
		getSales();
	}, [state.filter]);

	return <>
		<table className={'table  margin-bottom-20'.classNames()}>
			<thead>
				<tr>
					<th>{__('Content')}</th>
					{
						!is_admin ? <th>
							{__('Earning')}
						</th> 
						: 
						<>
							<th>
								{__('Sale Price')}
							</th>
							<th>
								{__('Contributor Commission')}
							</th>
						</>
					}
					<th>{__('Order Status')}</th>
					<th>{__('Date')}</th>
				</tr>
			</thead>
			<tbody>
				{
					state.sales.map(sale=>{
						return <tr key={sale.sale_id} data-cylector="sales-single-row">
							<td data-th={__('Content Title')}>
								<div>
									<span className={'d-block'.classNames()}>
										{sale.content_title}
									</span>
									<small className={'color-text-60'.classNames()}>
										{sale.variation?.plan?.plan_name}
									</small>
								</div>
							</td>
							{
								!is_admin ? <td data-th={__('Earning')}>
									<div className={'d-flex align-items-center column-gap-8'.classNames()}>
										<span>
											{currency_symbol}{sale.commission}
										</span>
										{
											! sale.is_awaiting ? null :
											<small className={'color-text-50'.classNames()}>
												({__('Awaiting')})
											</small>
										}
									</div>
								</td> 
								: 
								<>
									<td data-th={__('Sale Price')}>
										{currency_symbol}{sale.sale_price}
									</td>
									<td data-th={__('Contributor Commission')}>
										{currency_symbol}{sale.commission}
									</td>
								</>
							}
							
							<td data-th={__('Order Status')}>
								{order_statuses[sale.order_status] || sale.order_status}
							</td>

							<td data-th={__('Date')}>
								{formatDate(sale.purchase_date)}
							</td>
						</tr>
					})
				}
				
				<TableStat 
					empty={isEmpty(state.sales)}
					loading={state.fetching}
					message={state.error_message || __('No sales yet!')}
				/>
			</tbody>
		</table>
	
		<div className={'d-flex justify-content-center'.classNames()}>
			<Pagination
				onChange={(page) => setFilter('page', page)}
				pageNumber={state.filter.page}
				pageCount={state.segmentation?.page_count || 1}
			/>
		</div>
	</>
}
