import React, { useContext, useEffect, useState } from "react";

import { request } from "solidie-materials/request.jsx";
import { TableStat } from "solidie-materials/table-stat.jsx";
import { __, currency_symbol, data_pointer, formatDate, formatTime, isEmpty } from "solidie-materials/helpers.jsx";
import { Pagination } from "solidie-materials/pagination/pagination.jsx";
import { DropDown } from "solidie-materials/dropdown/dropdown";
import { TextField } from "solidie-materials/text-field/text-field";
import { ContextToast } from "solidie-materials/toast/toast.jsx";
import { LoadingIcon } from "solidie-materials/loading-icon/loading-icon.jsx";
import { DropDownStatus } from "solidie-materials/dropdown-status/dropdown-status.jsx";
import { Modal } from "solidie-materials/modal.jsx";
import {confirm} from 'solidie-materials/prompts.jsx';
import {AdminTitleHead} from 'solidie-materials/backend-dashboard-container/title-head.jsx';

import { methods } from "../../frontend-dashboard/pages/settings/withdrawal-method.jsx";

export const withdrawal_statuses = {
	pending: __('Pending'), 
	processing: __('Processing'), 
	sent: __('Sent'), 
	rejected: __('Rejected'), 
	cancelled: __('Cancelled')
}

const dropdown_options = Object.keys(withdrawal_statuses).map(status=>{
	return {
		id: status,
		label: withdrawal_statuses[status]
	}
});

const {is_admin, readonly_mode} = window[data_pointer];

export function WithdrawalRequests({session, refreshSession=()=>{}}) {

	const {ajaxToast} = useContext(ContextToast);

	const [state, setState] = useState({
		fetching: true,
		error_message: null,
		changing_status_for: null,
		segmentation: null,
		withdrawals: [],
		expanded_withdrawal: null
	});

	const [filterState, setFilterState]=useState({
		page: 1,
		contributor_email: '',
		status: ''
	});

	const setFilter=(name, value)=>{
		setFilterState({
			...filterState,
			[name]: value
		});
	}

	const getWithdrawals=()=>{

		setState({
			...state,
			fetching: true
		});

		request('getWithdrawalRequests', {...filterState, is_admin: is_admin ? true : false}, resp=>{
			const {
				success,
				data: {
					withdrawals=[],
					segmentation,
					message=__('Something went wrong!')
				}
			} = resp;

			setState({
				...state,
				fetching: false,
				withdrawals,
				segmentation,
				error_message: success ? null : message
			});
		});
	}

	const changeStatus = (withdrawal_id, status, message) => {
		
		confirm(
			message || __('Change withdrawal status?'),
			()=>{
				setState({
					...state,
					changing_status_for: withdrawal_id
				});

				request('changeWithdrawalStatus', {withdrawal_id, status}, resp=>{
					setState({
						...state,
						changing_status_for: false,
						withdrawals: resp.success ? state.withdrawals.map(w=>w.withdrawal_id==withdrawal_id ? {...w, withdrawal_status: status} : w) : state.withdrawals
					});
					
					ajaxToast(resp);

					refreshSession();
				});
			}
		);
	}

	const toggleMethod=(expanded_withdrawal)=>{
		setState({
			...state,
			expanded_withdrawal
		});
	}

	useEffect(()=>{
		getWithdrawals();
	}, [session, filterState]);

	return <>
	
		{
			!state.expanded_withdrawal ? null :
			<Modal closeOnDocumentClick={true} onClose={()=>toggleMethod(null)}>
				<div
					className={'d-flex align-items-center margin-bottom-20'.classNames()}
				>
					<div className={'flex-1'.classNames()}>
						<span className={'font-size-16 font-weight-500 color-text'.classNames()}>
							{__('Receiving Account')}
						</span>
					</div>
					<div>
						<i
							className={'sicon sicon-times font-size-18 color-text-50 cursor-pointer'.classNames()}
							onClick={()=>toggleMethod(null)}
						></i>
					</div>
				</div>
				<span>
					{state.expanded_withdrawal.method_details.email}
				</span>
			</Modal>
		}

		{
			!is_admin ? null :
			<AdminTitleHead title={__('Withdrawals')}>
				<div className={'d-flex align-items-center column-gap-15'.classNames()}>
					<div>
						<DropDown
							placeholder={__('Status')}
							value={filterState.status}
							onChange={status=>setFilter('status', status)}
							options={dropdown_options}
						/>
					</div>
					<div>
						<TextField
							placeholder={__('Contributor Email')}
							value={filterState.contributor_email || ''}
							onChange={v=>setFilter('contributor_email', v)}
							type='search'
						/>
					</div>
				</div>
			</AdminTitleHead>
		}
		
		<table className={'table '.classNames()}>
			<thead>
				<tr>
					{
						!is_admin ? null :
						<th>
							{__('Contributor')}
						</th>
					}
					<th className={'white-space-nowrap'.classNames()}>{__('Amount')}</th>
					<th>{__('Method')}</th>
					<th>{__('Status')}</th>
					<th>{__('Date')}</th>
				</tr>
			</thead>
			<tbody>
				{
					state.withdrawals.map(withdrawal=>{
						
						const {
							withdrawal_id, 
							withdrawal_method, 
							withdrawal_status, 
							request_date, 
							sent_date,
							amount,
							method_details,
							avatar_url,
							display_name,
							user_email
						} = withdrawal;

						const changing_status = state.changing_status_for==withdrawal_id;

						return <tr key={withdrawal_id} data-cylector="withdrawal-single">
							{
								!is_admin ? null :
								<td data-th={__('Contributor')}>
									<div className={'d-flex align-items-center column-gap-15'.classNames()}>
										<img src={avatar_url} style={{width: '32px', height: '32px', borderRadius: '50%'}}/>
										<div>
											<strong className={'d-block font-size-16 font-weight-500 color-text-80'.classNames()}>
												{display_name}
											</strong>
											<small className={'color-text-50 font-size-13'.classNames()}>
												{user_email}
											</small>
										</div>
									</div>
								</td>
							}
							<td data-th={__('Amount')}>
								<span className={'color-text-70'.classNames()}>
									{currency_symbol}{amount}
								</span>
							</td>
							<td data-th={__('Method')}>
								<div>
									<strong 
										className={'cursor-pointer color-text-70 interactive'.classNames()}
										onClick={()=>toggleMethod(withdrawal)}
									>
										<span>{methods[withdrawal_method]?.label}</span>
									</strong>
								</div>
							</td>
							<td data-th={__('Status')}>
								<div 
									className={'d-flex align-items-center column-gap-10'.classNames()}
									title={(is_admin && withdrawal_status==='pending') ? __('Contributor can cancel the request as long as it is in Pending status') : null}
								>
									{
										(!is_admin || withdrawal_status=='cancelled') ? 
										<span className={`${withdrawal_status=='cancelled' ? 'color-text-50' : 'color-text-70'}`.classNames()}>
											{withdrawal_statuses[withdrawal_status] || withdrawal_status}
										</span> 
										:
										<div data-cylector="withdrawal-status">
											<DropDownStatus
												placeholder={__('Select Status')}
												value={withdrawal_status}
												onChange={status=>changeStatus(withdrawal_id, status)}
												options={dropdown_options.filter(o=>o.id!='cancelled')}
												disabled={state.changing_status_for}
											/>
										</div>
									}
									
									{
										(changing_status || is_admin || withdrawal_status!='pending') ? null : 
											<i 
												className={'sicon sicon-times font-size-14 color-error cursor-pointer interactive'.classNames()}
												disabled={readonly_mode || state.changing_status_for}
												onClick={()=>changeStatus(withdrawal_id, 'cancelled', __('Sure to cancel withdrawal request?'))}
											/>
									}
									
									<LoadingIcon show={changing_status}/>
								</div>
							</td>
							<td data-th={__('Date')}>
								<div>
									<span className={'d-block font-size-14 font-weight-400 color-text-70 margin-bottom-5'.classNames()}>
										{formatDate(request_date)}
									</span>
									<span className={'d-block font-size-13 font-weight-400 color-text-50'.classNames()}>
										{formatTime(request_date)}
									</span>
								</div>
							</td>
						</tr>
					})
				}
				<TableStat 
					empty={isEmpty(state.withdrawals)}
					loading={state.fetching}
					message={state.error_message || __('No request!')}
				/>
			</tbody>
		</table>
		
		<div className={'d-flex justify-content-center margin-top-15'.classNames()}>
			<Pagination
				onChange={(page) => setFilter('page', page)}
				pageNumber={filterState.page}
				pageCount={state.segmentation?.page_count || 1}
			/>
		</div>
	</>
}
